@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_WindowCarbon.cpp,v 1.4 2003/01/01 22:32:35 maarten Exp $
+ * $Id: GHOST_WindowCarbon.cpp,v 1.7 2003/07/09 16:01:27 ton Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 
 /**
 
- * $Id: GHOST_WindowCarbon.cpp,v 1.4 2003/01/01 22:32:35 maarten Exp $
+ * $Id: GHOST_WindowCarbon.cpp,v 1.7 2003/07/09 16:01:27 ton Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * @author	Maarten Gribnau
  * @date	May 10, 2001
@@ -50,10 +50,11 @@ AGLContext GHOST_WindowCarbon::s_firstaglCtx = NULL;
 const GHOST_TInt32 GHOST_WindowCarbon::s_sizeRectSize = 16;
 #endif //GHOST_DRAW_CARBON_GUTTER
 
-static const GLint sPreferredFormatWindow[7] = {
+static const GLint sPreferredFormatWindow[9] = {
 AGL_RGBA,			GL_TRUE,
 AGL_DOUBLEBUFFER,	GL_TRUE,
 AGL_DEPTH_SIZE,		16,
+AGL_AUX_BUFFERS,     1,
 AGL_NONE,
 };
 
@@ -328,7 +329,8 @@ GHOST_TSuccess GHOST_WindowCarbon::setOrder(GHOST_TWindowOrder order)
 {
 	GHOST_ASSERT(getValid(), "GHOST_WindowCarbon::setOrder(): window invalid")
     if (order == GHOST_kWindowOrderTop) {
-        ::BringToFront(m_windowRef);
+        //::BringToFront(m_windowRef); is wrong, front window should be active for input too
+		::SelectWindow(m_windowRef);
     }
     else {
         ::SendBehind(m_windowRef, nil);
@@ -532,7 +534,7 @@ void GHOST_WindowCarbon::loadCursor(bool visible, GHOST_TStandardCursor cursor)
 		switch (cursor) {
 		default:
 		GCMAP( GHOST_kStandardCursorDefault,				kThemeArrowCursor);
-		GCMAP( GHOST_kStandardCursorRightArrow,				kThemeArrowCursor);
+		GCMAP( GHOST_kStandardCursorRightArrow,				kThemeAliasArrowCursor);
 		GCMAP( GHOST_kStandardCursorLeftArrow,				kThemeArrowCursor);
 		GCMAP( GHOST_kStandardCursorInfo, 					kThemeArrowCursor);
 		GCMAP( GHOST_kStandardCursorDestroy,				kThemeArrowCursor);
@@ -542,8 +544,8 @@ void GHOST_WindowCarbon::loadCursor(bool visible, GHOST_TStandardCursor cursor)
 		GCMAP( GHOST_kStandardCursorWait,					kThemeWatchCursor);
 		GCMAP( GHOST_kStandardCursorText,					kThemeIBeamCursor);
 		GCMAP( GHOST_kStandardCursorCrosshair,				kThemeCrossCursor);
-		GCMAP( GHOST_kStandardCursorUpDown,					kThemeArrowCursor);
-		GCMAP( GHOST_kStandardCursorLeftRight,				kThemeResizeLeftRightCursor);
+		GCMAP( GHOST_kStandardCursorUpDown,					kThemeClosedHandCursor);
+		GCMAP( GHOST_kStandardCursorLeftRight,				kThemeClosedHandCursor);
 		GCMAP( GHOST_kStandardCursorTopSide,				kThemeArrowCursor);
 		GCMAP( GHOST_kStandardCursorBottomSide,				kThemeArrowCursor);
 		GCMAP( GHOST_kStandardCursorLeftSide,				kThemeResizeLeftCursor);
