@@ -1,10 +1,10 @@
 
-/*  subsurf.c      MIXED MODEL
+/*  subsurf.c
  * 
  *  jun 2001
  *  
  * 
- * $Id: subsurf.c,v 1.3 2002/11/25 12:01:52 mein Exp $
+ * $Id: subsurf.c,v 1.6 2003/04/26 13:07:59 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -129,10 +129,14 @@ static float *Vec3AddT(float *t, float *a, float *b) {
 static float *Vec2Add(float *ta, float *b) {
 	ta[0]+= b[0];
 	ta[1]+= b[1];
-	ta[2]+= b[2];
 	return ta;
 }
 
+static float *Vec2MulNT(float *t, float *a, float n) {
+	t[0]= a[0]*n;
+	t[1]= a[1]*n;
+	return t;
+}
 static float *Vec3MulNT(float *t, float *a, float n) {
 	t[0]= a[0]*n;
 	t[1]= a[1]*n;
@@ -147,6 +151,11 @@ static float *Vec3Add(float *ta, float *b) {
 	return ta;
 }
 
+static float *Vec2MulN(float *ta, float n) {
+	ta[0]*= n;
+	ta[1]*= n;
+	return ta;
+}
 static float *Vec3MulN(float *ta, float n) {
 	ta[0]*= n;
 	ta[1]*= n;
@@ -545,7 +554,7 @@ static void hypermesh_subdivide(HyperMesh *me, HyperMesh *nme) {
 			Vec2CpyI(uvco_mid, 0.0, 0.0);
 			for (j=0; j<f->nverts; j++)
 				Vec2Add(uvco_mid, f->uvco[j]);
-			Vec3MulN(uvco_mid, (float)(1.0/f->nverts));
+			Vec2MulN(uvco_mid, (float)(1.0/f->nverts));
 
 			for (j=0; j<f->nverts; last= j, j++)
 				Vec2AvgT(uvco_edge[j], f->uvco[last], f->uvco[j]);
@@ -892,3 +901,44 @@ DispList* subsurf_mesh_to_displist(Mesh *me, DispList *dl, short subdiv)
 
 	return subsurf_subdivide_to_displist(hme, subdiv);
 }
+
+void subsurf_calculate_limit_positions(Mesh *me, float (*positions_r)[3]) 
+{
+	/* Finds the subsurf limit positions for the verts in a mesh 
+	 * and puts them in an array of floats. Please note that the 
+	 * calculated vert positions is incorrect for the verts 
+	 * on the boundary of the mesh.
+	 */
+	HyperMesh *hme= hypermesh_from_mesh(me, NULL);
+	HyperMesh *nme= hypermesh_new();
+	float edge_sum[3], face_sum[3];
+	HyperVert *hv;
+	LinkNode *l;
+	int i;
+
+	hypermesh_subdivide(hme, nme);
+
+	for (i= me->totvert-1,hv=hme->verts; i>=0; i--,hv=hv->next) {
+		int N= 0;
+                
+		edge_sum[0]= edge_sum[1]= edge_sum[2]= 0.0;
+		face_sum[0]= face_sum[1]= face_sum[2]= 0.0;
+
+		for (N=0,l=hv->edges; l; N++,l= l->next) {
+			Vec3Add(edge_sum, ((HyperEdge*) l->link)->ep->co);
+		}
+		for (l=hv->faces; l; l= l->next) {
+			Vec3Add(face_sum, ((HyperFace*) l->link)->mid->co);
+		}
+
+		positions_r[i][0] = 
+			(hv->nmv->co[0]*N*N + edge_sum[0]*4 + face_sum[0])/(N*(N+5));
+		positions_r[i][1] = 
+			(hv->nmv->co[1]*N*N + edge_sum[1]*4 + face_sum[1])/(N*(N+5));
+		positions_r[i][2] = 
+			(hv->nmv->co[2]*N*N + edge_sum[2]*4 + face_sum[2])/(N*(N+5));
+	}
+
+	hypermesh_free(nme);
+	hypermesh_free(hme);
+}
