@@ -1,5 +1,9 @@
-/**
- * $Id: util.c,v 1.3 2002/11/25 12:01:55 mein Exp $
+/* util.c
+ *
+ * various string, file, list operations.
+ *
+ *
+ * $Id: util.c,v 1.8 2003/08/12 20:37:45 aphex Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -28,7 +32,7 @@
  * Contributor(s): none yet.
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
- * Diverse string, file, list operations.
+ * 
  */
 
 #include <stdio.h>
@@ -65,6 +69,11 @@
 #include <sys/time.h>
 #endif
 
+#ifdef __APPLE__
+#include <sys/param.h>
+#include <CoreFoundation/CoreFoundation.h>
+#endif
+
 /* local */
 
 static int add_win32_extension(char *name);
@@ -104,7 +113,7 @@ int BLI_stringdec(char *string, char *kop, char *staart, unsigned short *numlen)
 	
 	if (len == len2) {
 		if (len > 4) {
-			/* .jf0 en .jf1 voor jstreams afvangen */
+			/* handle .jf0 en .jf1 for jstreams */
 			if (strncasecmp(string + len - 4, ".jf", 3) == 0) len -= 4;
 			else if (strncasecmp(string + len - 4, ".tga", 4) == 0) len -= 4;
 			else if (strncasecmp(string + len - 4, ".jpg", 4) == 0) len -= 4;
@@ -180,7 +189,7 @@ void BLI_newname(char * name, int add)
 	
 	pic = BLI_stringdec(name, head, tail, &digits);
 	
-	/* gaan we van 100 -> 99 of van 10 naar 9 */
+	/* are we going from 100 -> 99 or from 10 -> 9 */
 	if (add < 0 && digits < 4 && digits > 0) {
 		int i, exp;
 		exp = 1;
@@ -258,17 +267,17 @@ void BLI_insertlink(ListBase *listbase, void *vprevlink, void *vnewlink)
 {
 	struct Link *prevlink= vprevlink, *newlink= vnewlink;
 
-	/* newlink komt na prevlink */
+	/* newlink comes after prevlink */
 
 	if (newlink == 0) return;
 	if (listbase == 0) return;
 
-	if(listbase->first==0) { /* lege lijst */
+	if(listbase->first==0) { /* empty list */
 		listbase->first= newlink;
 		listbase->last= newlink;
 		return;
 	}
-	if (prevlink== 0) {	/* inserten voor eerste element */
+	if (prevlink== 0) {	/* insert before first element */
 		newlink->next= listbase->first;
 		newlink->prev= 0;
 		newlink->next->prev= newlink;
@@ -276,7 +285,7 @@ void BLI_insertlink(ListBase *listbase, void *vprevlink, void *vnewlink)
 		return;
 	}
 
-	if (listbase->last== prevlink) /* aan einde lijst */
+	if (listbase->last== prevlink) /* at end of list */
 		listbase->last = newlink;
 
 	newlink->next= prevlink->next;
@@ -289,17 +298,17 @@ void BLI_insertlinkbefore(ListBase *listbase, void *vnextlink, void *vnewlink)
 {
 	struct Link *nextlink= vnextlink, *newlink= vnewlink;
 
-	/* newlink komt voor nextlink */
+	/* newlink before nextlink */
 
 	if (newlink == 0) return;
 	if (listbase == 0) return;
 
-	if(listbase->first==0) { /* lege lijst */
+	if(listbase->first==0) { /* empty list */
 		listbase->first= newlink;
 		listbase->last= newlink;
 		return;
 	}
-	if (nextlink== 0) {	/* inserten aan einde lijst */
+	if (nextlink== 0) {	/* insert at end of list */
 		newlink->prev= listbase->last;
 		newlink->next= 0;
 		((struct Link *)listbase->last)->next= newlink;
@@ -307,7 +316,7 @@ void BLI_insertlinkbefore(ListBase *listbase, void *vnextlink, void *vnewlink)
 		return;
 	}
 
-	if (listbase->first== nextlink) /* aan begin lijst */
+	if (listbase->first== nextlink) /* at beginning of list */
 		listbase->first = newlink;
 
 	newlink->next= nextlink;
@@ -482,13 +491,39 @@ char *BLI_gethome(void) {
 	#else /* Windows */
 		char * ret;
 		static char dir[512];
+		char tmpdir[512];
 
 		ret = getenv("HOME");
 		if(ret) {
 			if (BLI_exists(ret)) return ret;
 		}
-		
-		// add user profile support for WIN 2K / NT
+
+		/* 
+		   "change-over" period - blender still checks in
+		   old locations, but Ctrl+U now saves in ~/.blender
+		*/
+
+		BLI_getInstallationDir(dir);
+
+		if (BLI_exists(dir))
+		{
+			//strcat(dir,"/.blender/");
+			strcpy(tmpdir,dir);
+			strcat(tmpdir,"/.blender/.B.blend");
+			if (BLI_exists(tmpdir))
+			{
+				strcat(dir,"/.blender/");
+				return(dir);
+			}
+		}
+
+		/* 
+		   everything below this point to be removed -
+		   blender should use the same %HOME% across
+		   all versions of Windows... (aphex)
+		*/
+				
+		/* add user profile support for WIN 2K / NT */
 		ret = getenv("USERPROFILE");
 		if (ret) {
 			if (BLI_exists(ret)) { /* from fop, also below... */
@@ -501,6 +536,11 @@ char *BLI_gethome(void) {
 				}
 			}
 		}
+
+		/* 
+		   Saving in the Windows dir is less than desirable. 
+		   Use as a last resort ONLY! (aphex)
+		*/
 		
 		ret = getenv("WINDOWS");		
 		if (ret) {
@@ -512,7 +552,7 @@ char *BLI_gethome(void) {
 			if(BLI_exists(ret)) return ret;
 		}
 		
-		return "C:\\Temp";	
+		return "C:\\Temp";	/* sheesh! bad, bad, bad! (aphex) */
 	#endif
 }
 
@@ -830,3 +870,21 @@ void BLI_where_am_i(char *fullname, char *name)
 	}
 }
 
+/* 
+ * returns absolute path to the app bundle
+ * only useful on OS X 
+ */
+#ifdef __APPLE__
+char* BLI_getbundle(void) {
+	CFURLRef bundleURL;
+	CFStringRef pathStr;
+	char path[MAXPATHLEN];
+	CFBundleRef mainBundle = CFBundleGetMainBundle();
+
+	bundleURL = CFBundleCopyBundleURL(mainBundle);
+	pathStr = CFURLCopyFileSystemPath(bundleURL, kCFURLPOSIXPathStyle);
+	CFStringGetCString(pathStr, path, MAXPATHLEN, kCFStringEncodingASCII);
+	return path;
+}
+#endif
+
