@@ -1,7 +1,7 @@
 /*  python.c      MIXED MODEL
  * 
  *  june 99
- * $Id: opy_draw.c,v 1.3 2002/11/25 10:13:52 mein Exp $
+ * $Id: opy_draw.c,v 1.4 2003/07/07 16:12:31 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -528,9 +528,9 @@ static void py_slider_update(void *butv, void *data2_unused)
 
 	g_window_redrawn = 0;
 	curarea->win_swap= WIN_BACK_OK; 
-	UIfrontbuf = 1; 
+	//UIfrontbuf = 1; 
 	spacetext_do_pywin_buttons(curarea->spacedata.first, uiButGetRetVal(but));
-	UIfrontbuf = 0;
+	//UIfrontbuf = 0;
 
 	if (!g_window_redrawn) /* if Redraw already called */
 		Windowmodule_Redraw(0, Py_BuildValue("(i)", SPACE_VIEW3D));
