@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_lamp_types.h (mar-2001 nzc)
  *
- * $Id: DNA_lamp_types.h,v 1.5 2002/12/27 13:10:28 mein Exp $ 
+ * $Id: DNA_lamp_types.h,v 1.8 2003/07/20 15:54:58 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -56,7 +56,7 @@ typedef struct Lamp {
 	float clipsta, clipend, shadspotsize;
 	float bias, soft;
 	
-	/* texact is voor buttons */
+	/* texact is for buttons */
 	short texact, shadhalostep;
 	
 	struct MTex *mtex[8];
@@ -86,12 +86,12 @@ typedef struct Lamp {
 #define LA_OSATEX		512
 /* use bit 11 for shadow tests... temp only -nzc- */
 #define LA_DEEP_SHADOW  1024
+#define LA_NO_DIFF		2048
+#define LA_NO_SPEC		4096
 
 /* mapto */
 #define LAMAP_COL		1
 
-/* bit isolated... */
-#define MAXLAMP		256
 
-#endif
+#endif /* DNA_LAMP_TYPES_H */
 
