@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_screen_types.h (mar-2001 nzc)
  *
- * $Id: DNA_screen_types.h,v 1.5 2002/12/27 13:10:28 mein Exp $ 
+ * $Id: DNA_screen_types.h,v 1.7 2003/06/27 15:29:36 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -48,7 +48,7 @@ typedef struct bScreen {
 	struct Scene *scene;
 	short startx, endx, starty, endy;	/* framebuffer coords */
 	short sizex, sizey;
-	short scenenr, screennr;			/* alleen voor pupmenu */
+	short scenenr, screennr;			/* only for pupmenu */
 	short full, rt;
 	short mainwin, winakt;
 } bScreen;
@@ -63,7 +63,7 @@ typedef struct ScrVert {
 typedef struct ScrEdge {
 	struct ScrEdge *next, *prev;
 	ScrVert *v1, *v2;
-	short border;			/* 1 als op rand screen */
+	short border;			/* 1 when at edge of screen */
 	short flag;
 	int pad;
 } ScrEdge;
@@ -81,12 +81,12 @@ typedef unsigned short dna_ushort_fix;
 typedef struct ScrArea {
 	struct ScrArea *next, *prev;
 	ScrVert *v1, *v2, *v3, *v4;
-	bScreen *full;			/* als area==full, dit is de parent */
+	bScreen *full;			/* if area==full, this is the parent */
 	float winmat[4][4];
 	rcti totrct, headrct, winrct;
 
 	short headwin, win;
-	short headertype;		/* 0=niets, 1= down, 2= up */
+	short headertype;		/* 0=no header, 1= down, 2= up */
 	char spacetype, butspacetype;	/* SPACE_...  */
 	short winx, winy;		/* size */
 	char head_swap, head_equal;
@@ -101,7 +101,7 @@ typedef struct ScrArea {
 
 #define MAXWIN		128
 
-/* Als je EDGEWIDTH verandert, ook globale array edcol[] goedzetten */
+/* If you change EDGEWIDTH, also do the global arrat edcol[]  */
 #define EDGEWIDTH	5
 #define EDGEWIDTH2	(2)
 #define AREAGRID	4
@@ -128,6 +128,8 @@ typedef struct ScrArea {
 #define B_SCROLL 8
 #define HOR_SCROLL 12
 
+/* dunno who thought this below is nice code, but be warned, the values are written in
+   a file, and cannot be switched or altered. enum here is out of focus (ton) */
 enum {
 	SPACE_EMPTY,
 	SPACE_VIEW3D,
