@@ -1,7 +1,7 @@
 /**
  * blenkernel/DNA_userdef_types.h (mar-2001 nzc)
  *
- *	$Id: DNA_userdef_types.h,v 1.6 2003/01/28 11:13:17 phase Exp $
+ *	$Id: DNA_userdef_types.h,v 1.13 2003/07/13 20:16:49 larstiq Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,18 +38,26 @@
 typedef struct UserDef {
 	short flag, dupflag;
 	int savetime;
-	char tempdir[64];
-	char fontdir[64];
-	char renderdir[64];
-	char textudir[64];
-	char plugtexdir[64];
-	char plugseqdir[64];
-	char pythondir[64];
-	char sounddir[64];
+	char tempdir[160];	// FILE_MAXDIR length
+	char fontdir[160];
+	char renderdir[160];
+	char textudir[160];
+	char plugtexdir[160];
+	char plugseqdir[160];
+	char pythondir[160];
+	char sounddir[160];
 	short versions, vrmlflag;	// tmp for export, will be replaced by strubi
 	int gameflags;
 	int wheellinescroll;
-	short uiflag, pad2;
+	short uiflag, language;
+	int userpref;
+	short console_buffer;	//console vars here for tuhopuu compat, --phase
+	short console_out;
+	int mixbufsize;
+	int fontsize;
+	short encoding;
+	short transopts;
+	char fontname[64];
 } UserDef;
 
 extern UserDef U; /* from usiblender.c !!!! */
@@ -78,8 +86,23 @@ extern UserDef U; /* from usiblender.c !!!! */
 #define	KEYINSERTOBJ	2
 #define WHEELZOOMDIR	4
 #define FILTERFILEEXTS	8
+#define DRAWVIEWINFO	16
+#define EVTTOCONSOLE	32		//print ghost events, here for tuhopuu compat. --phase
+#define FLIPINFOMENU	64
+#define FLIPFULLSCREEN	128
+#define ALLWINCODECS	256
+
+/* transopts */
+
+#define	TR_TOOLTIPS		1
+#define	TR_BUTTONS		2
+#define TR_MENUS		4
+#define TR_FILESELECT	8
+#define TR_TEXTEDIT		16
+#define TR_ALL			32
 
 /* dupflag */
+
 #define DUPMESH			1
 #define DUPCURVE		2
 #define DUPSURF			4
@@ -93,16 +116,15 @@ extern UserDef U; /* from usiblender.c !!!! */
 #define	DUPACT			1024
 
 /* gameflags */
+
 #define USERDEF_VERTEX_ARRAYS_BIT        0
 #define USERDEF_DISABLE_SOUND_BIT        1
 #define USERDEF_DISABLE_MIPMAP_BIT       2
 
-
 #define USERDEF_VERTEX_ARRAYS        (1 << USERDEF_VERTEX_ARRAYS_BIT)
 #define USERDEF_DISABLE_SOUND        (1 << USERDEF_DISABLE_SOUND_BIT)
 #define USERDEF_DISABLE_MIPMAP       (1 << USERDEF_DISABLE_MIPMAP_BIT)
 
-
 /* vrml flag */
 
 #define USERDEF_VRML_LAYERS		1
