@@ -1,5 +1,5 @@
 /**
- * $Id: edgeRender.c,v 1.3 2002/11/25 12:02:03 mein Exp $
+ * $Id: edgeRender.c,v 1.5 2003/05/21 15:55:00 sirdude Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -73,7 +73,7 @@
 
 #ifdef RE_EDGERENDERSAFE
 char edgeRender_h[] = EDGERENDER_H;
-char edgeRender_c[] = "$Id: edgeRender.c,v 1.3 2002/11/25 12:02:03 mein Exp $";
+char edgeRender_c[] = "$Id: edgeRender.c,v 1.5 2003/05/21 15:55:00 sirdude Exp $";
 #include "errorHandler.h"
 #endif
 
@@ -168,7 +168,7 @@ int  zBufferEdgeRenderObjects(void);
 /**
  * Add edge pixels to the original image. It blends <bron> over <doel>.
  */
-void addEdgeOver(char *doel, char *bron);
+void addEdgeOver(unsigned char *dst, unsigned char *src);
 
 /* ------------------------------------------------------------------------- */
 
@@ -367,7 +367,7 @@ void renderEdges(char *colourRect)
 		}
 	}
 		
-	/* alle getallen in zbuffer 3 naar rechts shiften */
+	/* shift values in zbuffer 3 to the right */
   	rz = edgeBuffer;
 	if(rz==0) return;
 	
@@ -498,37 +498,48 @@ void renderEdges(char *colourRect)
 
 /* ------------------------------------------------------------------------- */
 
-void addEdgeOver(char *doel, char *bron)   /* telt bron bij doel */
-{	
-	float c;
-	int mul;
-	
-	if( bron[3] == 0) return;
-	if( bron[3] == 255) { /* is getest, scheelt  */
-		*((unsigned int *)doel)= *((unsigned int *)bron);
-		return;
-	}
+/* adds src to dst */
+void addEdgeOver(unsigned char *dst, unsigned char *src)   
+{
+   unsigned char inverse;
+   unsigned char alpha;
+   unsigned int  c;
 
-	/* This must be a special blend-mode, because we get a 'weird' data      */
-	/* input format now. With edge = (c_e, a_e), picture = (c_p, a_p), we    */
-	/* get: result = ( c_e*a_e + c_p(1 - a_e), a_p ).                        */
-	
-	mul = 255 - bron[3];
+   alpha = src[3];
+  
+   if( alpha == 0) return;
+   if( alpha == 255) { 
+      /* when full opacity, just copy the pixel */
+      /* this code assumes an int is 32 bit, fix 
 
-	/* Not sure if the conversion to float is really necessary here... I will*/
-	/* think about it another day.                                           */
-	c = ((mul * doel[0] + bron[0] * bron[3])/255.0);
-	if(c>255) {doel[0]=255;} else { doel[0]= c;}
-	c = ((mul * doel[1] + bron[1] * bron[3])/255.0);
-	if(c>255) {doel[1]=255;} else { doel[1]= c;}
-	c = ((mul * doel[2] + bron[2] * bron[3])/255.0);
-	if(c>255) {doel[2]=255;} else { doel[2]= c;}
+      *(( unsigned int *)dst)= *((unsigned int *)src);
+	replaced with memcpy
+		*/
+       memcpy(dst,src,4);
+       return;
+   }
+  
+   /* This must be a special blend-mode, because we get a 'weird' data      */
+   /* input format now. With edge = (c_e, a_e), picture = (c_p, a_p), we    */
+   /* get: result = ( c_e*a_e + c_p(1 - a_e), a_p ).                        */
+  
+   inverse = 255 - alpha;
+  
+   c = ((unsigned int)inverse * (unsigned int) dst[0] + (unsigned int)src[0] *
+	(unsigned int)alpha) >> 8;
+   dst[0] = c;
+
+   c = ((unsigned int)inverse * (unsigned int) dst[1] + (unsigned int)src[1] *
+	(unsigned int)alpha) >> 8;
+   dst[1] = c;
+   c = ((unsigned int)inverse * (unsigned int) dst[2] + (unsigned int)src[2] *
+	(unsigned int)alpha) >> 8;
+   dst[2] = c;
 }
 
 void calcEdgeRenderColBuf(char* colTargetBuffer)
 {
 
-/*      int part; */
     int keepLooping = 1;
 	int sample;
 	
@@ -659,7 +670,7 @@ void fillEdgeRenderFace(float *v1, float *v2, float *v3)
 		else	             { minv=v3; midv=v2; maxv=v1;}
 	}
 
-	if(minv[1] == maxv[1]) return;	/* beveiliging 'nul' grote vlakken */
+	if(minv[1] == maxv[1]) return;	/* security, for 'zero' size faces */
 
 	my0  = ceil(minv[1]);
 	my2  = floor(maxv[1]);
@@ -670,7 +681,7 @@ void fillEdgeRenderFace(float *v1, float *v2, float *v3)
 
 	if(my0<Aminy) my0= Aminy;
 
-	/* EDGES : DE LANGSTE */
+	/* EDGES : THE LONGEST */
 	xx1= maxv[1]-minv[1];
 	if(xx1>2.0/65536.0) {
 		z0= (maxv[0]-minv[0])/xx1;
@@ -685,7 +696,7 @@ void fillEdgeRenderFace(float *v1, float *v2, float *v3)
 		dx0= 0;
 		xs0= 65536.0*(MIN2(minv[0],maxv[0]));
 	}
-	/* EDGES : DE BOVENSTE */
+	/* EDGES : THE TOP ONE */
 	xx1= maxv[1]-midv[1];
 	if(xx1>2.0/65536.0) {
 		z0= (maxv[0]-midv[0])/xx1;
@@ -700,7 +711,7 @@ void fillEdgeRenderFace(float *v1, float *v2, float *v3)
 		dx1= 0;
 		xs1= 65536.0*(MIN2(midv[0],maxv[0]));
 	}
-	/* EDGES : DE ONDERSTE */
+	/* EDGES : THE BOTTOM ONE */
 	xx1= midv[1]-minv[1];
 	if(xx1>2.0/65536.0) {
 		z0= (midv[0]-minv[0])/xx1;
@@ -725,9 +736,9 @@ void fillEdgeRenderFace(float *v1, float *v2, float *v3)
 	if(vec0[2]==0.0) return;
 
 	if(midv[1] == maxv[1]) omsl= my2;
-	if(omsl < Aminy) omsl= Aminy-1;  /* dan neemt ie de eerste lus helemaal */
+	if(omsl < Aminy) omsl= Aminy-1;  /* that way it does the first loop entirely */
 
-	while (my2 > Amaxy) {  /* my2 kan groter zijn */
+	while (my2 > Amaxy) {  /* my2 can really be larger */
 		xs0+=dx0;
 		if (my2<=omsl) {
 			xs2+= dx2;
@@ -852,7 +863,7 @@ void fillEdgeRenderEdge(float *vec1, float *vec2)
 	
 	if(fabs(dx) > fabs(dy)) {
 
-		/* alle lijnen van links naar rechts */
+		/* alle lines from left to right */
 		if(vec1[0]<vec2[0]) {
 			VECCOPY(v1, vec1);
 			VECCOPY(v2, vec2);
@@ -900,7 +911,7 @@ void fillEdgeRenderEdge(float *vec1, float *vec2)
 	}
 	else {
 	
-		/* alle lijnen van onder naar boven */
+		/* all lines from top to bottom */
 		if(vec1[1]<vec2[1]) {
 			VECCOPY(v1, vec1);
 			VECCOPY(v2, vec2);
