@@ -1,5 +1,5 @@
 /**
- * $Id: drawimasel.c,v 1.5 2003/02/09 11:31:17 phase Exp $
+ * $Id: drawimasel.c,v 1.9 2003/04/28 14:43:20 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -82,15 +82,15 @@ void rectwrite_imasel(int winxmin, int winymin, int winxmax, int winymax, int x1
 	
 	oldxim= xim;
 		
-	/* coordinaten hoe 't op scherm komt */
+	/* coordinates how it will be put at screen */
 	x2= x1+ zoomx*xim;
 	y2= y1+ zoomy*yim;
 
-	/* partiele clip */
+	/* partial clip */
 	if(x1<=winxmin) {
-		/* recten bij OpenGL mogen niet links/onder van windowrand beginnen */
+		/* with OpenGL, rects are not allowed to start outside of the left/bottom window edge */
 		cx= winxmin-x1+(int)zoomx;
-		/* zorg ervoor dat de rect pixelnauwkeurig wordt neergezet */
+		/* make sure the rect will be drawn pixel-exact */
 		cx/= zoomx;
 		cx++;
 		x1+= zoomx*cx;
@@ -418,6 +418,12 @@ static void str_image_type(int ftype, char *name)
 	if( ftype & PNG )       { strcat(name, "png "); }
 	if( ftype & BMP )       { strcat(name, "bmp "); }
 	if( ftype & AMI )       { strcat(name, "iff ");	  }
+#ifdef WITH_QUICKTIME
+	if( ftype & QUICKTIME ) { strcat(name, "quicktime "); }
+#endif
+#ifdef WITH_FREEIMAGE
+	if( ftype & FREEIMAGE ) { strcat(name, "freeimage "); }
+#endif
 }
 
 void draw_sima_area(SpaceImaSel *simasel)
@@ -572,7 +578,7 @@ void draw_sima_area(SpaceImaSel *simasel)
 						glRecti(sx, sy,  ex, ey); uiEmboss(sx,sy, ex,ey, 1);
 					}
 					if (ima->disksize/1000 > 1000){ sprintf(infostr,  "%s  %.2fMb  x%i y%i  %i bits ",ima->file_name,(ima->disksize/1024)/1024.0, ima->orgx, ima->orgy, ima->orgd);
-					}else{ sprintf(infostr,  "%s  %dKb  x%i y%i  %i bits ",  ima->file_name,ima->disksize/1024,          ima->orgx, ima->orgy, ima->orgd);
+					}else{ sprintf(infostr,  "%s  %dKb  %ix%i  %i bits ",  ima->file_name,ima->disksize/1024,          ima->orgx, ima->orgy, ima->orgd);
 					}	
 					if (ima->anim == 1){ strcat (infostr, "movie"); }else{
 						str_image_type(ima->ibuf_type, naam);
@@ -610,7 +616,7 @@ void draw_sima_area(SpaceImaSel *simasel)
 				ima = ima->next;
 			}
 			
-			if ((simasel->mode & 8) == 8) {	 /* if loep */
+			if ((simasel->mode & 8) == 8) {	 /* if magnify */
 				
 				if (bitset(simasel->fase, IMS_KNOW_IMA) && (simasel->hilite_ima)) {
 			
@@ -851,9 +857,9 @@ void pibplay(SpaceImaSel *simasel)
 
 
 
-/* ************** hoofdtekenfunktie ************** */
+/* ************** main drawing function ************** */
 
-void drawimasel()	/* hoofdtekenfunktie */
+void drawimaselspace(ScrArea *sa, void *spacedata)
 {
 	SpaceImaSel *simasel;
 	simasel= curarea->spacedata.first;
