@@ -1,5 +1,5 @@
 /**
- * $Id: drawobject.c,v 1.4 2003/02/01 08:25:09 sgefant Exp $
+ * $Id: drawobject.c,v 1.15 2003/07/21 17:46:55 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -136,7 +136,7 @@ static void init_gl_materials(Object *ob)
 		matbuf[0][1][2]= defmaterial.specb;
 		matbuf[0][1][3]= 1.0;
 		
-		/* ook matnr 1, displists! */
+		/* do material 1 too, for displists! */
 		VECCOPY(matbuf[1][0], matbuf[0][0]);
 		VECCOPY(matbuf[1][1], matbuf[0][1]);
 	}
@@ -240,9 +240,10 @@ static void draw_icon_centered(float *pos, unsigned int *rect, int rectsize)
 	glRasterPos3fv(pos);
 	
 		/* use bitmap to shift rasterpos in pixels */
-	glBitmap(1, 1, 0.0, 0.0, -hsize, -hsize, &dummy);
-	glFinish(); /* for sun */
-	
+	glBitmap(0, 0, 0.0, 0.0, -hsize, -hsize, &dummy);
+#ifdef __sun__
+	glFinish(); 
+#endif	
 	glDrawPixels(rectsize, rectsize, GL_RGBA, GL_UNSIGNED_BYTE, rect);
 }
 
@@ -450,25 +451,25 @@ static void tekenshadbuflimits(Lamp *la, float mat[][4])
 
 static void spotvolume(float *lvec, float *vvec, float inp)
 {
-	/* camera staat op 0,0,0 */
+	/* camera is at 0,0,0 */
 	float temp[3],plane[3],mat1[3][3],mat2[3][3],mat3[3][3],mat4[3][3],q[4],co,si,hoek;
 
 	Normalise(lvec);
-	Normalise(vvec);				/* is dit de goede vector ? */
+	Normalise(vvec);				/* is this the correct vector ? */
 
-	Crossf(temp,vvec,lvec);		/* vergelijking van vlak door vvec en lvec */
-	Crossf(plane,lvec,temp);		/* en dan het vlak loodrecht daarop en evenwijdig aan lvec */
+	Crossf(temp,vvec,lvec);		/* equation for a plane through vvec en lvec */
+	Crossf(plane,lvec,temp);		/* a plane perpendicular to this, parrallel with lvec */
 
 	Normalise(plane);
 
-	/* nu hebben we twee vergelijkingen: die van de kegel en die van het vlak, maar we hebben
-	drie onbekenden We halen nu een onbekende weg door het vlak naar z=0 te roteren */
-	/* Ik heb geen flauw idee of dat mag, we moeten tenslotte twee oplossingen krijgen, maar we
-	proberen het gewoon: vlak vector moet (0,0,1) worden*/
+	/* now we've got two equations: one of a cone and one of a plane, but we have
+	three unknowns. We remove one unkown by rotating the plane to z=0 (the plane normal) */
 
-	/* roteer om uitproduct vector van (0,0,1) en vlakvector, inproduct graden */
-	/* volgens defenitie volgt dat uitproduct is (plane[1],-plane[0],0), en cos() = plane[2]);*/
+	/* rotate around cross product vector of (0,0,1) and plane normal, dot product degrees */
+	/* according definition, we derive cross product is (plane[1],-plane[0],0), en cos = plane[2]);*/
 
+	/* translating this comment to english didnt really help me understanding the math! :-) (ton) */
+	
 	q[1] = plane[1] ; 
 	q[2] = -plane[0] ; 
 	q[3] = 0 ;
@@ -485,7 +486,7 @@ static void spotvolume(float *lvec, float *vvec, float inp)
 
 	QuatToMat3(q,mat1);
 
-	/* lampvector nu over acos(inp) graden roteren */
+	/* rotate lamp vector now over acos(inp) degrees */
 
 	vvec[0] = lvec[0] ; 
 	vvec[1] = lvec[1] ; 
@@ -629,7 +630,7 @@ static void draw_limit_line(float sta, float end, unsigned int col)
 
 void drawcamera(Object *ob)
 {
-	/* een staande piramide met (0,0,0) als top */
+	/* a standing up pyramid with (0,0,0) as top */
 	Camera *cam;
 	World *wrld;
 	float vec[8][4], tmat[4][4], fac, facx, facy, depth;
@@ -638,7 +639,7 @@ void drawcamera(Object *ob)
 	glDisable(GL_LIGHTING);
 	glDisable(GL_CULL_FACE);
 	
-	/* zo is ie altijd te zien */
+	/* that way it's always visible */
 	fac= cam->drawsize;
 	if(G.vd->persp>=2) fac= cam->clipsta+0.1;
 	
@@ -646,7 +647,7 @@ void drawcamera(Object *ob)
 	facx= fac*1.28;
 	facy= fac*1.024;
 	
-	vec[0][0]= 0; vec[0][1]= 0; vec[0][2]= 0.001;	/* GLBUG: z niet op nul vanwege picking op entry */
+	vec[0][0]= 0; vec[0][1]= 0; vec[0][2]= 0.001;	/* GLBUG: for picking at iris Entry (well thats old!) */
 	vec[1][0]= facx; vec[1][1]= facy; vec[1][2]= depth;
 	vec[2][0]= facx; vec[2][1]= -facy; vec[2][2]= depth;
 	vec[3][0]= -facx; vec[3][1]= -facy; vec[3][2]= depth;
@@ -674,7 +675,7 @@ void drawcamera(Object *ob)
 	if(G.vd->persp>=2) return;
 	if(G.f & G_BACKBUFSEL) return;
 	
-	/* pijl aan top */
+	/* arrow on top */
 	vec[0][2]= depth;
 
 	glBegin(GL_QUADS);
@@ -916,6 +917,7 @@ static void drawlattice(Object *ob)
 
 /* ***************** ******************** */
 
+/* window coord, assuming all matrices are set OK */
 void calc_meshverts(void)
 {
 	EditVert *eve;
@@ -937,6 +939,7 @@ void calc_meshverts(void)
 	MTC_Mat4SwapMat4(G.vd->persmat, mat);
 }
 
+/* window coord for current window, sets matrices temporal */
 void calc_meshverts_ext(void)
 {
 
@@ -947,6 +950,40 @@ void calc_meshverts_ext(void)
 	
 }
 
+/* window coord for current window, sets matrices temporal, sets (eve->f & 2) when not visible */
+void calc_meshverts_ext_f2(void)
+{
+	EditVert *eve;
+	float mat[4][4];
+	
+	/* matrices */
+	areawinset(curarea->win);
+	mymultmatrix(G.obedit->obmat);
+	
+	if(G.edve.first==0) return;
+	eve= G.edve.first;
+
+	MTC_Mat4SwapMat4(G.vd->persmat, mat);
+	mygetsingmatrix(G.vd->persmat);
+
+	eve= G.edve.first;
+	while(eve) {
+		eve->f &= ~2;
+		if(eve->h==0) {
+			project_short_noclip(eve->co, &(eve->xs));
+			if( eve->xs >= 0 && eve->ys >= 0 && eve->xs<curarea->winx && eve->ys<curarea->winy);
+			else eve->f |= 2;
+		}
+		eve= eve->next;
+	}
+	
+	/* restore */
+	MTC_Mat4SwapMat4(G.vd->persmat, mat);
+	myloadmatrix(G.vd->viewmat);
+	
+}
+
+
 static void calc_Nurbverts(Nurb *nurb)
 {
 	Nurb *nu;
@@ -1039,7 +1076,7 @@ void tekenvertices_ext(int mode)
 	
 	glDrawBuffer(GL_FRONT);
 
-	/* alle views aflopen */
+	/* check all views */
 	tempsa= curarea;
 	sa= G.curscreen->areabase.first;
 	while(sa) {
@@ -1423,7 +1460,7 @@ static void drawDispListsolid(ListBase *lb, Object *ob)
 
 				set_gl_material(dl->col+1);
 								
-				/* voor poly's is er maar 1 normaal nodig */
+				/* voor polys only one normal needed */
 				if(index3_nors_incr==0) {
 					while(parts--) {
 
@@ -1638,6 +1675,39 @@ static void drawDispListshaded(ListBase *lb, Object *ob)
 	glShadeModel(GL_FLAT);
 }
 
+/* wrappers for shaded+wire and solid+wire */
+static void drawMeshWireExtra(Object *ob) {
+	GLfloat origwidth;
+	GLint origcolor[4];
+	
+	glGetFloatv(GL_LINE_WIDTH, &origwidth);
+	glGetIntegerv(GL_CURRENT_COLOR, origcolor);
+	
+	glLineWidth(1.2);
+
+	cpack(0x4F4F4F);
+	drawmeshwire(ob);
+
+	glColor4iv(origcolor);
+	glLineWidth(origwidth);
+}
+
+static void drawDispListWireExtra(ListBase *lb) {
+	GLfloat origwidth;
+	GLint origcolor[4];
+
+	glGetFloatv(GL_LINE_WIDTH, &origwidth);
+	glGetIntegerv(GL_CURRENT_COLOR, origcolor);
+
+	glLineWidth(1.2);
+
+	cpack(0x4F4F4F);
+	drawDispListwire(lb);
+
+	glLineWidth(origwidth);
+	glColor4iv(origcolor);
+}
+
 static void drawmeshsolid(Object *ob, float *nors)
 {
 	Mesh *me;
@@ -1719,12 +1789,15 @@ static void drawmeshsolid(Object *ob, float *nors)
 			if(G.zbuf) glEnable(GL_DEPTH_TEST);
 		}
 		
-		if(ob->dtx == OB_DRAWWIRE) {
-			drawmeshwire(ob);
-		}
+		if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 
 	}
-	else {
+	else {		/* [nors] should never be zero, but is weak code... the displist 
+				   system needs a make over (ton)
+		          
+				   Face select and vertex paint calls drawmeshsolid() with nors = NULL!
+				   It's still weak code but hey, as ton says, the whole system needs 
+		           a good thrashing! ;) (aphex) */
 
 		start= 0; end= me->totface;
 		set_buildvars(ob, &start, &end);
@@ -1773,7 +1846,7 @@ static void drawmeshsolid(Object *ob, float *nors)
 					}
 					
 	
-					/* dit GL_QUADS grapje is op snelheid getest: factor 2! */
+					/* this GL_QUADS joke below was tested for speed: a factor 2! */
 						
 					if(v4) {if(glmode==GL_TRIANGLES) {glmode= GL_QUADS; glEnd(); glBegin(GL_QUADS);}}
 					else {if(glmode==GL_QUADS) {glmode= GL_TRIANGLES; glEnd(); glBegin(GL_TRIANGLES);}}
@@ -1891,7 +1964,7 @@ static void drawmeshshaded(Object *ob, unsigned int *col1, unsigned int *col2)
 	me= ob->data;
 	mface= me->mface;
 	
-	/* tekent ie geen hide */
+	/* then it does not draw hide */
 	if( (G.f & G_FACESELECT) && ob==((G.scene->basact) ? (G.scene->basact->object) : 0)) tface= me->tface;
 	else tface= 0;
 	
@@ -1982,10 +2055,7 @@ static void drawmeshshaded(Object *ob, unsigned int *col1, unsigned int *col2)
 	glShadeModel(GL_FLAT);
 	if(twoside) glDisable(GL_CULL_FACE);
 	
-	if(ob->dtx == OB_DRAWWIRE) {
-		cpack(0x5F5F5F);
-		drawmeshwire(ob);
-	}
+	if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 }
 
 static void drawDispList(Object *ob, int dt)
@@ -2023,26 +2093,17 @@ static void drawDispList(Object *ob, int dt)
 					/* vertexpaint only true when selecting */
 				if (vertexpaint) {
 					drawmeshsolid(ob, NULL);
-					if(ob->dtx == OB_DRAWWIRE) {
-						cpack(0x5F5F5F);
-						drawmeshwire(ob);
-					}
+					if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);					
 				} else {
 					init_gl_materials(ob);
 					two_sided(me->flag & ME_TWOSIDED);
 					drawDispListsolid(lb, ob);
-					if(ob->dtx == OB_DRAWWIRE) {
-						cpack(0x5F5F5F);
-						drawDispListwire(lb);
-					}
+					if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
 				}
 			}
 			else {
 				drawmeshsolid(ob, dl->nors);
-				if(ob->dtx == OB_DRAWWIRE) {
-					cpack(0x5F5F5F);
-					drawmeshwire(ob);
-				}
+				if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 			}
 			
 		}
@@ -2053,7 +2114,7 @@ static void drawDispList(Object *ob, int dt)
 				MFace *curface;
 				int i;
 				unsigned char r,g,b;
-				float val1,val2,val3,val4;
+				float val1,val2,val3,val4=0;
 				
 				wtcol = curwt= MEM_callocN (sizeof (unsigned char) * me->totface*4*4, "weightmap");
 				
@@ -2078,34 +2139,26 @@ static void drawDispList(Object *ob, int dt)
 				}
 				
 				drawmeshshaded(ob, (unsigned int*)wtcol, 0);
-				if(ob->dtx == OB_DRAWWIRE) {
-					cpack(0x5F5F5F);
-					drawmeshwire(ob);
-				}
+				if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
+				
 				MEM_freeN (wtcol);
 				
 			}
 			else
 #endif
 			if( G.f & (G_VERTEXPAINT+G_TEXTUREPAINT)) {
-				/* in deze volgorde: vertexpaint heeft soms al mcol gemaakt */
+				/* in order: vertexpaint already made mcol */
 ///*
 
 //*/
 				if(me->mcol) {
 					drawmeshshaded(ob, (unsigned int *)me->mcol, 0);
-					if(ob->dtx == OB_DRAWWIRE) {
-						cpack(0x5F5F5F);
-						drawmeshwire(ob);
-					}
+					if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 				} else if(me->tface) {
 					tface_to_mcol(me);
 					drawmeshshaded(ob, (unsigned int *)me->mcol, 0);	
 					MEM_freeN(me->mcol); me->mcol= 0;
-					if(ob->dtx == OB_DRAWWIRE) {
-						cpack(0x5F5F5F);
-						drawmeshwire(ob);
-					}
+					if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 				}
 				else 
 					drawmeshwire(ob);
@@ -2121,16 +2174,10 @@ static void drawDispList(Object *ob, int dt)
 				if(dl) {
 					if(mesh_uses_displist(me)) {
 						drawDispListshaded(&me->disp, ob);
-						if(ob->dtx == OB_DRAWWIRE) {
-							cpack(0x5F5F5F);
-							drawDispListwire(&me->disp);
-						}
+						if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(&me->disp);
 					} else {
 						drawmeshshaded(ob, dl->col1, dl->col2);
-						if(ob->dtx == OB_DRAWWIRE) {
-							cpack(0x5F5F5F);
-							drawmeshwire(ob);
-						}
+						if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 					}
 				}
 			}
@@ -2152,7 +2199,7 @@ static void drawDispList(Object *ob, int dt)
 			dl= lb->first;
 			if(dl==0) return;
 			
-			/* regel: dl->type INDEX3 altijd vooraan in lijst */
+			/* rule: dl->type INDEX3 is always first in list */
 			if(dl->type!=DL_INDEX3) {
 				curve_to_filledpoly(ob->data, lb);
 				dl= lb->first;
@@ -2164,20 +2211,13 @@ static void drawDispList(Object *ob, int dt)
 			if(dt==OB_SHADED) {
 				if(ob->disp.first==0) shadeDispList(ob);
 				drawDispListshaded(lb, ob);
-				if(ob->dtx == OB_DRAWWIRE) {
-					cpack(0x5F5F5F);
-					drawDispListwire(lb);
-				}
-
+				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
 			}
 			else {
 				init_gl_materials(ob);
 				two_sided(0);
 				drawDispListsolid(lb, ob);
-				if(ob->dtx == OB_DRAWWIRE) {
-					cpack(0x5F5F5F);
-					drawDispListwire(lb);
-				}
+				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
 
 			}
 			index3_nors_incr= 1;
@@ -2202,20 +2242,14 @@ static void drawDispList(Object *ob, int dt)
 			if(dt==OB_SHADED) {
 				if(ob->disp.first==0) shadeDispList(ob);
 				drawDispListshaded(lb, ob);
-				if(ob->dtx == OB_DRAWWIRE) {
-					cpack(0x5F5F5F);
-					drawDispListwire(lb);
-				}
+				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
 			}
 			else {
 				init_gl_materials(ob);
 				two_sided(0);
 			
 				drawDispListsolid(lb, ob);
-				if(ob->dtx == OB_DRAWWIRE) {
-					cpack(0x5F5F5F);
-					drawDispListwire(lb);
-				}
+				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
 			}
 		}
 		else {
@@ -2233,22 +2267,14 @@ static void drawDispList(Object *ob, int dt)
 				dl= lb->first;
 				if(dl && dl->col1==0) shadeDispList(ob);
 				drawDispListshaded(lb, ob);
-				if(ob->dtx == OB_DRAWWIRE) {
-					cpack(0x5F5F5F);
-					drawDispListwire(lb);
-				}
-
+				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
 			}
 			else {
 				init_gl_materials(ob);
 				two_sided(0);
 			
 				drawDispListsolid(lb, ob);	
-				if(ob->dtx == OB_DRAWWIRE) {
-					cpack(0x5F5F5F);
-					drawDispListwire(lb);
-				}
-
+				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
 			}
 		}
 		else drawDispListwire(lb);
@@ -2377,24 +2403,91 @@ static void drawmeshwire(Object *ob)
 
 	if(ob==G.obedit || (G.obedit && ob->data==G.obedit->data)) {
 		
+		if(G.f & (G_FACESELECT+G_DRAWFACES)) {	/* faces */
+			
+			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+			glEnable(GL_BLEND);
+
+			evl= G.edvl.first;
+			while(evl) {
+				if(evl->v1->h==0 && evl->v2->h==0 && evl->v3->h==0) {
+					
+					if(1) {
+						if(vlakselectedAND(evl, 1)) glColor4ub(200, 100, 200, 60);
+						else glColor4ub(0, 50, 150, 30);
+						
+						glBegin(evl->v4?GL_QUADS:GL_TRIANGLES);
+						glVertex3fv(evl->v1->co);
+						glVertex3fv(evl->v2->co);
+						glVertex3fv(evl->v3->co);
+						if(evl->v4) glVertex3fv(evl->v4->co);
+						glEnd();
+						
+					} else {
+						if(vlakselectedAND(evl, 1)) cpack(0x559999);
+						else cpack(0x664466);
+					
+						if(evl->v4 && evl->v4->h==0) {
+						
+							CalcCent4f(cent, evl->v1->co, evl->v2->co, evl->v3->co, evl->v4->co);
+							glBegin(GL_LINE_LOOP);
+								VecMidf(fvec, cent, evl->v1->co); glVertex3fv(fvec);
+								VecMidf(fvec, cent, evl->v2->co); glVertex3fv(fvec);
+								VecMidf(fvec, cent, evl->v3->co); glVertex3fv(fvec);
+								VecMidf(fvec, cent, evl->v4->co); glVertex3fv(fvec);
+							glEnd();
+						}
+						else {
+	
+							CalcCent3f(cent, evl->v1->co, evl->v2->co, evl->v3->co);
+							glBegin(GL_LINE_LOOP);
+								VecMidf(fvec, cent, evl->v1->co); glVertex3fv(fvec);
+								VecMidf(fvec, cent, evl->v2->co); glVertex3fv(fvec);
+								VecMidf(fvec, cent, evl->v3->co); glVertex3fv(fvec);
+							glEnd();
+						}
+					}
+				}
+				evl= evl->next;
+			}
+			glDisable(GL_BLEND);
+		}
+
 		if(G.zbuf==0 && mesh_uses_displist(me)) {
 			cpack(0x505050);
 			drawDispListwire(&me->disp);
 		}
 		cpack(0x0);
 		
-		eed= G.eded.first;
-
-		glBegin(GL_LINES);
-		while(eed) {
-			if(eed->h==0) {
-				glVertex3fv(eed->v1->co);
-				glVertex3fv(eed->v2->co);
+		if(G.f & G_DRAWEDGES) {	/* Use edge Highlighting */
+			glShadeModel(GL_SMOOTH);
+			
+			eed= G.eded.first;
+			glBegin(GL_LINES);
+			while(eed) {
+				if(eed->h==0) {
+					if(eed->v1->f & 1) cpack(0x559999); else cpack(0x0);
+					glVertex3fv(eed->v1->co);
+					if(eed->v2->f & 1) cpack(0x559999); else cpack(0x0);
+					glVertex3fv(eed->v2->co);
+				}
+				eed= eed->next;
 			}
-			eed= eed->next;
+			glEnd();
+			glShadeModel(GL_FLAT);
+		}
+		else {
+			eed= G.eded.first;
+			glBegin(GL_LINES);
+			while(eed) {
+				if(eed->h==0) {
+					glVertex3fv(eed->v1->co);
+					glVertex3fv(eed->v2->co);
+				}
+				eed= eed->next;
+			}
+			glEnd();
 		}
-		glEnd();
-		
 		if(ob!=G.obedit) return;
 		
 		calc_meshverts();
@@ -2430,38 +2523,6 @@ static void drawmeshwire(Object *ob)
 
 			glEnd();
 		}
-		if(G.f & (G_FACESELECT+G_DRAWFACES)) {	/* vlakken */
-			
-			evl= G.edvl.first;
-			while(evl) {
-				if(evl->v1->h==0 && evl->v2->h==0 && evl->v3->h==0) {
-					
-					if(vlakselectedAND(evl, 1)) cpack(0x559999);
-					else cpack(0x664466);
-				
-					if(evl->v4 && evl->v4->h==0) {
-					
-						CalcCent4f(cent, evl->v1->co, evl->v2->co, evl->v3->co, evl->v4->co);
-						glBegin(GL_LINE_LOOP);
-							VecMidf(fvec, cent, evl->v1->co); glVertex3fv(fvec);
-							VecMidf(fvec, cent, evl->v2->co); glVertex3fv(fvec);
-							VecMidf(fvec, cent, evl->v3->co); glVertex3fv(fvec);
-							VecMidf(fvec, cent, evl->v4->co); glVertex3fv(fvec);
-						glEnd();
-					}
-					else {
-
-						CalcCent3f(cent, evl->v1->co, evl->v2->co, evl->v3->co);
-						glBegin(GL_LINE_LOOP);
-							VecMidf(fvec, cent, evl->v1->co); glVertex3fv(fvec);
-							VecMidf(fvec, cent, evl->v2->co); glVertex3fv(fvec);
-							VecMidf(fvec, cent, evl->v3->co); glVertex3fv(fvec);
-						glEnd();
-					}
-				}
-				evl= evl->next;
-			}
-		}
 	}
 	else {
 		
@@ -2475,7 +2536,7 @@ static void drawmeshwire(Object *ob)
 			
 			mvert= me->mvert;
 			mface= me->mface;
-			
+
 			ok= 0;
 			if(me->totface==0) ok= 1;
 			else {
@@ -2868,7 +2929,7 @@ static void drawnurb(Object *ob, Nurb *nurb, int dt)
 	float vec[3];
 	int a, nr, skip;
 
-	/* eerst handles niet select */
+	/* first non-selected handles */
 	nu= nurb;
 	while(nu) {
 		if((nu->type & 7)==CU_BEZIER) {
@@ -2877,7 +2938,7 @@ static void drawnurb(Object *ob, Nurb *nurb, int dt)
 		nu= nu->next;
 	}
 	
-	/* dan DispList */
+	/* then DispList */
 	
 	cpack(0);
 	cu= ob->data;
@@ -3247,7 +3308,7 @@ void draw_object(Base *base)
 	
 	ob= base->object;
 
-	/* keys tekenen? */
+	/* draw keys? */
 	if(base==(G.scene->basact) || (base->flag & (SELECT+BA_WASSEL))) {
 		if(warning_recursive==0 && ob!=G.obedit) {
 			if(ob->ipo && ob->ipo->showkey && (ob->ipoflag & OB_DRAWKEY)) {
@@ -3318,12 +3379,12 @@ void draw_object(Base *base)
 		}
 	}
 	
-	/* patch? kinderen met timeoffs verprutsen ouders. Hoe los je dat op! */
+	/* patch? children objects with a timeoffs change the parents. How to solve! */
 	/* if( ((int)ob->ctime) != F_(G.scene->r.cfra)) where_is_object(ob); */
 
 	mymultmatrix(ob->obmat);
 
-	/* welke wire kleur */
+	/* which wire color */
 	if((G.f & (G_BACKBUFSEL+G_PICKSEL)) == 0) {
 		project_short(ob->obmat[3], &base->sx);
 		
@@ -3350,7 +3411,7 @@ void draw_object(Base *base)
 	if(G.zbuf==0 && dt>OB_WIRE) dt= OB_WIRE;
 	dtx= 0;
 	
-	/* faceselect uitzondering: ook solid tekenen als dt==wire, behalve in editmode */
+	/* faceselect exception: also draw solid when dt==wire, except in editmode */
 	if(ob==((G.scene->basact) ? (G.scene->basact->object) : 0) && (G.f & (G_FACESELECT+G_VERTEXPAINT+G_TEXTUREPAINT+G_WEIGHTPAINT))) {
 		if(ob->type==OB_MESH) {
 			
@@ -3426,18 +3487,12 @@ void draw_object(Base *base)
 						drawDispListsolid(&me->disp, ob);
 						/* this seems to be the place where the wire for subsurfs 
 						 * gets drawn.. so we draw an extra wire in grey here (editmode) */
-						if(ob->dtx & OB_DRAWWIRE) {
-							cpack(0x5F5F5F);
-							drawDispListwire(&me->disp);
-						}
+						if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(&me->disp);
 						drawmeshwire(ob);
 					}
 					else {
 						drawmeshsolid(ob, 0);
-						if(ob->dtx & OB_DRAWWIRE) {
-							cpack(0x5F5F5F);
-							drawDispListwire(&me->disp);
-						}
+						if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(&me->disp);
 					}
 				}
 				if(ob==G.obedit && (G.f & G_PROPORTIONAL)) draw_prop_circle();
@@ -3453,10 +3508,7 @@ void draw_object(Base *base)
 					if(G.f & G_BACKBUFSEL) drawmeshsolid(ob, 0);					
 					else if(G.f & G_FACESELECT || G.vd->drawtype==OB_TEXTURE) {
 						draw_tface_mesh(ob, ob->data, dt);
-						if(ob->dtx & OB_DRAWWIRE) {
-							cpack(0x5F5F5F);
-							drawmeshwire(ob);
-						}
+						if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 					}
 					else drawDispList(ob, dt);
 				}
@@ -3466,7 +3518,7 @@ void draw_object(Base *base)
 				&& ((G.f & (G_BACKBUFSEL+G_PICKSEL)) == 0) ) {
 				paf = give_parteff(ob);
 				if( paf ) {
-					if(col) cpack(0xFFFFFF);	/* zichtbaarheid */
+					if(col) cpack(0xFFFFFF);	/* for visibility */
 					if(paf->flag & PAF_STATIC) draw_static_particle_system(ob, paf);
 					else draw_particle_system(ob, paf);
 					cpack(col);
@@ -3488,8 +3540,6 @@ void draw_object(Base *base)
 		case OB_CURVE:
 		case OB_SURF:
 			cu= ob->data;
-			/* een pad niet solid tekenen: wel dus!!! */
-			/* if(cu->flag & CU_PATH) if(dt>OB_WIRE) dt= OB_WIRE; */
 			
 			if(ob==G.obedit) {
 				drawnurb(ob, editNurb.first, dt);
@@ -3508,7 +3558,7 @@ void draw_object(Base *base)
 			drawaxes(1.0);
 			break;
 		case OB_LAMP:
-			/* doet myloadmatrix */
+			/* does a myloadmatrix */
 			drawlamp(ob);
 			break;
 		case OB_CAMERA:
@@ -3531,7 +3581,7 @@ void draw_object(Base *base)
 		}
 	}
 
-	/* draw extra: na gewone draw ivm makeDispList */
+	/* draw extra: after normal draw because of makeDispList */
 	if(dtx) {
 		if(G.f & G_SIMULATION);
 		else if(dtx & OB_AXIS) {
@@ -3573,7 +3623,7 @@ void draw_object(Base *base)
 	if(G.f & G_SIMULATION) return;
 
 	if((G.f & (G_BACKBUFSEL+G_PICKSEL))==0) {
-		/* hulplijnen e.d. */
+		/* help lines and so */
 		if(ob->parent && (ob->parent->lay & G.vd->lay)) {
 			setlinestyle(3);
 			glBegin(GL_LINES);
@@ -3607,6 +3657,9 @@ void draw_object(Base *base)
 			}
 			else {
 				if(base->flag & SELECT) rect= rect_sel;
+				/* The center of the active object (which need not 
+				 * be selected) gets drawn as if it were selected
+				 */
 				else if(base==(G.scene->basact)) rect= rect_sel;
 				else rect= rect_desel;
 			}
@@ -3629,7 +3682,7 @@ void draw_object_ext(Base *base)
 	ScrArea *tempsa, *sa;
 	View3D *vd;
 	
-	if(G.vd==0) return;
+	if(G.vd==NULL || base==NULL) return;
 	
 	if(G.vd->drawtype > OB_WIRE) {
 		G.zbuf= 1;
@@ -3637,15 +3690,15 @@ void draw_object_ext(Base *base)
 	}
 	
 	G.f |= G_DRAW_EXT;
-	
+
 	glDrawBuffer(GL_FRONT);
 
-	/* alle views aflopen */
+	/* check all views */
 	tempsa= curarea;
 	sa= G.curscreen->areabase.first;
 	while(sa) {
 		if(sa->spacetype==SPACE_VIEW3D) {
-			/* er wordt beperkt in beide buffers getekend: selectbuffer! */
+			/* limited drawing in both buffers: selectbuffer! */
 			
 			vd= sa->spacedata.first;
 			if(base->lay & vd->lay) {
@@ -3662,7 +3715,6 @@ void draw_object_ext(Base *base)
 	
 	G.f &= ~G_DRAW_EXT;
 
-	glFinish();
 	glDrawBuffer(GL_BACK);
 	
 	if(G.zbuf) {
