@@ -1,5 +1,5 @@
 /**
- * $Id: drawtext.c,v 1.8 2003/02/04 12:30:52 phase Exp $
+ * $Id: drawtext.c,v 1.12 2003/07/09 20:09:07 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -68,6 +68,7 @@
 #include "BIF_keyval.h"
 #include "BIF_interface.h"
 #include "BIF_drawtext.h"
+#include "BIF_editfont.h"
 #include "BIF_spacetypes.h"
 #include "BIF_usiblender.h"
 #include "BIF_screen.h"
@@ -84,8 +85,8 @@
 #define TEXTXLOC	38
 
 /* locals */
-void drawtextspace(void);
-void winqreadtextspace(unsigned short event, short val, char ascii);
+void drawtextspace(ScrArea *sa, void *spacedata);
+void winqreadtextspace(struct ScrArea *sa, void *spacedata, struct BWinEvent *evt);
 
 static void *last_txt_find_string= NULL;
 
@@ -532,7 +533,7 @@ static void do_selection(SpaceText *st, int selecting)
 		txt_undo_add_toop(st->text, UNDO_STO, sell, selc, linep2, charp2);
 }
 
-void drawtextspace(void)
+void drawtextspace(ScrArea *sa, void *spacedata)
 {
 	SpaceText *st= curarea->spacedata.first;
 	Text *text;
@@ -561,7 +562,7 @@ void drawtextspace(void)
 	else st->viewlines= 0;
 	
 	if(st->showlinenrs) {
-		cpack(C_DERK);
+		cpack(0x8c787c);
 		glRecti(23,  0, (st->lheight==15)?63:59,  curarea->winy - 2);
 	}
 
@@ -577,10 +578,10 @@ void drawtextspace(void)
 	for (i=0; i<st->viewlines && tmp; i++, tmp= tmp->next) {
 		if(st->showlinenrs) {
 			if(((float)(i + linecount + 1)/10000.0) < 1.0) {
-				sprintf(linenr, "%04d", i + linecount + 1);
+				sprintf(linenr, "%4d", i + linecount + 1);
 				glRasterPos2i(TXT_OFFSET - 7, curarea->winy-st->lheight*(i+1));
 			} else {
-				sprintf(linenr, "%05d", i + linecount + 1);
+				sprintf(linenr, "%5d", i + linecount + 1);
 				glRasterPos2i(TXT_OFFSET - 11, curarea->winy-st->lheight*(i+1));
 			}
 			BMF_DrawString(spacetext_get_font(st), linenr);
@@ -742,8 +743,140 @@ static int jumptoline_interactive(SpaceText *st) {
 	}
 }
 
-void winqreadtextspace(unsigned short event, short val, char ascii)
+
+int bufferlength;
+static char *copybuffer = NULL;
+
+void txt_copy_selectbuffer (Text *text)
 {
+	int length=0;
+	TextLine *tmp, *linef, *linel;
+	int charf, charl;
+	
+	if (!text) return;
+	if (!text->curl) return;
+	if (!text->sell) return;
+
+	if (!txt_has_sel(text)) return;
+	
+	if (copybuffer) {
+		MEM_freeN(copybuffer);
+		copybuffer= NULL;
+	}
+
+	if (text->curl==text->sell) {
+		linef= linel= text->curl;
+		
+		if (text->curc < text->selc) {
+			charf= text->curc;
+			charl= text->selc;
+		} else{
+			charf= text->selc;
+			charl= text->curc;
+		}
+	} else if (txt_get_span(text->curl, text->sell)<0) {
+		linef= text->sell;
+		linel= text->curl;
+
+		charf= text->selc;		
+		charl= text->curc;
+	} else {
+		linef= text->curl;
+		linel= text->sell;
+		
+		charf= text->curc;
+		charl= text->selc;
+	}
+
+	if (linef == linel) {
+		length= charl-charf;
+
+		copybuffer= MEM_mallocN(length+1, "cut buffera");
+		
+		BLI_strncpy(copybuffer, linef->line + charf, length+1);
+	} else {
+		length+= linef->len - charf;
+		length+= charl;
+		length++; /* For the '\n' */
+		
+		tmp= linef->next;
+		while (tmp && tmp!= linel) {
+			length+= tmp->len+1;
+			tmp= tmp->next;
+		}
+		
+		copybuffer= MEM_mallocN(length+1, "cut bufferb");
+		
+		strncpy(copybuffer, linef->line+ charf, linef->len-charf);
+		length= linef->len-charf;
+		
+		copybuffer[length++]='\n';
+		
+		tmp= linef->next;
+		while (tmp && tmp!=linel) {
+			strncpy(copybuffer+length, tmp->line, tmp->len);
+			length+= tmp->len;
+			
+			copybuffer[length++]='\n';			
+			
+			tmp= tmp->next;
+		}
+		strncpy(copybuffer+length, linel->line, charl);
+		length+= charl;
+		
+		copybuffer[length]=0;
+	}
+
+	bufferlength = length;
+}
+
+
+void txt_paste_clipboard(Text *text) {
+#ifdef _WIN32
+	char * buffer = NULL;
+
+	if ( OpenClipboard(NULL) ) {
+		HANDLE hData = GetClipboardData( CF_TEXT );
+		buffer = (char*)GlobalLock( hData );
+		txt_insert_buf(text, buffer);
+		GlobalUnlock( hData );
+		CloseClipboard();
+	}
+#endif
+}
+
+void txt_copy_clipboard(Text *text) {
+#ifdef _WIN32
+	txt_copy_selectbuffer(text);
+
+	if (OpenClipboard(NULL)) {
+		HLOCAL clipbuffer;
+		char* buffer;
+
+		EmptyClipboard();
+		clipbuffer = LocalAlloc(LMEM_FIXED,((bufferlength+1)));
+		buffer = (char *) LocalLock(clipbuffer);
+
+		strncpy(buffer, copybuffer, bufferlength);
+		buffer[bufferlength] =  '\0';
+		LocalUnlock(clipbuffer);
+		SetClipboardData(CF_TEXT,clipbuffer);
+		CloseClipboard();
+	}
+
+	if (copybuffer) {
+		MEM_freeN(copybuffer);
+		copybuffer= NULL;
+	}
+#endif
+}
+
+
+void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
+{
+	unsigned short event= evt->event;
+	short val= evt->val;
+	char ascii= evt->ascii;
 	SpaceText *st= curarea->spacedata.first;
 	Text *text= st->text;
 	char *py_filename;
@@ -754,11 +887,13 @@ void winqreadtextspace(unsigned short event, short val, char ascii)
 		return;
 	}
 
+	/* smartass code to prevent the events below from not working! */
+	if (!isprint(ascii) || (G.qual & ~LR_SHIFTKEY)) ascii= 0;
+
 	text= st->text;
 	
 	if (!text) {
-		if (val && !ELEM(G.qual, 0, LR_SHIFTKEY)) {
-			if (event==FKEY && (G.qual & LR_ALTKEY) && (G.qual & LR_SHIFTKEY)) {
+			if (event==RIGHTMOUSE) {
 				switch (pupmenu("File %t|New %x0|Open... %x1")) {
 				case 0:
 					st->text= add_empty_text();
@@ -771,7 +906,22 @@ void winqreadtextspace(unsigned short event, short val, char ascii)
 					activate_fileselect(FILE_SPECIAL, "LOAD TEXT FILE", G.sce, add_text_fs);
 					break;
 				}
-			} else if (event==QKEY) {
+			}
+			if (val && !ELEM(G.qual, 0, LR_SHIFTKEY)) {
+			if (event==FKEY && (G.qual & LR_ALTKEY) && (G.qual & LR_SHIFTKEY)) {
+				switch (pupmenu("File %t|New %x0|Open... %x1")) {
+				case 0:
+					st->text= add_empty_text();
+					st->top= 0;
+				
+					allqueue(REDRAWTEXT, 0);
+					allqueue(REDRAWHEADERS, 0);
+					break;
+				case 1:
+					activate_fileselect(FILE_SPECIAL, "LOAD TEXT FILE", G.sce, add_text_fs);
+					break;
+			}
+		} else if (event==QKEY) {
 				if(okee("QUIT BLENDER")) exit_usiblender();
 			}
 		}
@@ -796,6 +946,35 @@ void winqreadtextspace(unsigned short event, short val, char ascii)
 		if (val) {
 			do_textscroll(st, 1);
 		}
+	} else if (event==RIGHTMOUSE) {
+		if (val) {
+			p= pupmenu("File %t|New %x0|Open... %x1|Save %x2|Save As...%x3");
+
+			switch(p) {
+				case 0:
+					st->text= add_empty_text();
+					st->top= 0;
+					
+					allqueue(REDRAWTEXT, 0);
+					allqueue(REDRAWHEADERS, 0);
+					break;
+
+				case 1:
+					activate_fileselect(FILE_SPECIAL, "LOAD TEXT FILE", G.sce, add_text_fs);
+					break;
+					
+				case 3:
+					text->flags |= TXT_ISMEM;
+					
+				case 2:
+					txt_write_file(text);
+					do_draw= 1;
+					break;
+
+				default:
+					break;
+			}
+		}
 	} else if (ascii) {
 		if (txt_add_char(text, ascii)) {
 			pop_space_text(st);
@@ -959,11 +1138,11 @@ void winqreadtextspace(unsigned short event, short val, char ascii)
 
 		switch(event) {
 		case AKEY:
-			if (G.qual & LR_CTRLKEY) {
+			if (G.qual & LR_ALTKEY) {
 				txt_move_bol(text, G.qual & LR_SHIFTKEY);
 				do_draw= 1;
 				pop_space_text(st);
-			} else if (G.qual & LR_ALTKEY) {
+			} else if (G.qual & LR_CTRLKEY) {
 				txt_sel_all(text);
 				do_draw= 1;
 			}
@@ -971,7 +1150,11 @@ void winqreadtextspace(unsigned short event, short val, char ascii)
 
 		case CKEY:
 			if (G.qual & LR_ALTKEY || G.qual & LR_CTRLKEY) {
-				txt_copy_sel(text);
+				if(G.qual & LR_SHIFTKEY)
+					txt_copy_clipboard(text);
+				else
+					txt_copy_sel(text);
+
 				do_draw= 1;	
 			}
 			break;
@@ -1004,6 +1187,13 @@ void winqreadtextspace(unsigned short event, short val, char ascii)
 			}
 			break;
 			
+		case MKEY:
+			if (G.qual & LR_ALTKEY) {
+				txt_export_to_object(text);
+				do_draw= 1;	
+			}
+			break;
+
 		case PKEY:
 			if (G.qual & LR_ALTKEY) {
 				if (!BPY_txt_do_python(st)) {
@@ -1055,7 +1245,8 @@ void winqreadtextspace(unsigned short event, short val, char ascii)
 			
 		case UKEY:
 			if (G.qual & LR_ALTKEY) {
-				if (G.qual & LR_SHIFTKEY) txt_print_undo(text);
+				if (G.qual & LR_SHIFTKEY) txt_do_redo(text);
+					//txt_print_undo(text); //debug buffer in console
 				else {
 					txt_do_undo(text);
 					do_draw= 1;
@@ -1065,7 +1256,11 @@ void winqreadtextspace(unsigned short event, short val, char ascii)
 
 		case VKEY:
 			if (G.qual & LR_ALTKEY || G.qual & LR_CTRLKEY) {
-				txt_paste(text);
+				if(G.qual & LR_SHIFTKEY)
+					txt_paste_clipboard(text);
+				else
+					txt_paste(text);
+
 				do_draw= 1;	
 				pop_space_text(st);
 			}
