@@ -1,5 +1,5 @@
 /**
- * $Id: drawview.c,v 1.3 2002/11/25 12:02:05 mein Exp $
+ * $Id: drawview.c,v 1.14 2003/07/21 17:46:55 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -97,6 +97,8 @@
 #include "BSE_view.h"
 #include "BSE_drawview.h"
 #include "BSE_headerbuttons.h"
+#include "BSE_seqaudio.h"
+
 
 #include "RE_renderconverter.h"
 
@@ -234,7 +236,7 @@ void init_gl_stuff(void)
 void two_sided(int val)
 {
 
-	/* twosided aan: geft errors bij x flip! */
+	/* twosided on: gives errors with x flip! */
 	glLightModeliv(GL_LIGHT_MODEL_TWO_SIDE, (GLint *)&val);
 }
 
@@ -272,7 +274,7 @@ void circ(float x, float y, float rad)
 	gluDeleteQuadric(qobj);
 }
 
-/* ********** IN ONTWIKKELING ********** */
+/* **********  ********** */
 
 static void draw_bgpic(void)
 {
@@ -298,7 +300,7 @@ static void draw_bgpic(void)
 	if(ima==0) return;
 	if(ima->ok==0) return;
 	
-	/* plaatje testen */
+	/* test for image */
 	if(ima->ibuf==0) {
 	
 		if(bgpic->rect) MEM_freeN(bgpic->rect);
@@ -337,7 +339,7 @@ static void draw_bgpic(void)
 		y2= vb.ymax;
 	}
 	else {
-		/* windowco berekenen */
+		/* calc window coord */
 		initgrabz(0.0, 0.0, 0.0);
 		window_to_3d(vec, 1, 0);
 		fac= MAX3( fabs(vec[0]), fabs(vec[1]), fabs(vec[1]) );
@@ -356,7 +358,7 @@ static void draw_bgpic(void)
 		y2=  cy+ asp*fac*(bgpic->yof+bgpic->size);
 	}
 	
-	/* volledige clip? */
+	/* complete clip? */
 	
 	if(x2 < 0 ) return;
 	if(y2 < 0 ) return;
@@ -384,7 +386,7 @@ static void draw_bgpic(void)
 
 void timestr(double time, char *str)
 {
-	/* formaat 00:00:00.00 (hr:min:sec) string moet 12 lang */
+	/* format 00:00:00.00 (hr:min:sec) string has to be 12 long */
 	int  hr= (int)      time/(60*60);
 	int min= (int) fmod(time/60, 60.0);
 	int sec= (int) fmod(time, 60.0);
@@ -413,7 +415,7 @@ static void drawgrid(void)
 	fy= vec4[1]; 
 	fw= vec4[3];
 
-	wx= (curarea->winx/2.0);	/* ivm afrondfoutjes, grid op verkeerde plek */
+	wx= (curarea->winx/2.0);	/* because of rounding errors, grid at wrong location */
 	wy= (curarea->winy/2.0);
 
 	x= (wx)*fx/fw;
@@ -444,7 +446,7 @@ static void drawgrid(void)
 	
 	persp(0);
 
-	cpack(0x484848);
+	cpack(0x606060);
 	
 	x+= (wx); 
 	y+= (wy);
@@ -465,7 +467,7 @@ static void drawgrid(void)
 		fy+= dx;
 	}
 
-	/* kruis in midden */
+	/* center cross */
 	if(G.vd->view==3) cpack(0xA0D0A0); /* y-as */
 	else cpack(0xA0A0D0);	/* x-as */
 
@@ -496,7 +498,7 @@ static void drawfloor(void)
 	gridlines= vd->gridlines/2;
 	grid= gridlines*vd->grid;
 	
-	cpack(0x484848);
+	cpack(0x606060);
 
 	for(a= -gridlines;a<=gridlines;a++) {
 
@@ -505,7 +507,7 @@ static void drawfloor(void)
 			else cpack(0x402000);
 		}
 		else if(a==1) {
-			cpack(0x484848);
+			cpack(0x606060);
 		}
 		
 	
@@ -518,7 +520,7 @@ static void drawfloor(void)
 		glEnd();
 	}
 	
-	cpack(0x484848);
+	cpack(0x606060);
 	
 	for(a= -gridlines;a<=gridlines;a++) {
 		if(a==0) {
@@ -526,7 +528,7 @@ static void drawfloor(void)
 			else cpack(0);
 		}
 		else if(a==1) {
-			cpack(0x484848);
+			cpack(0x606060);
 		}
 	
 		glBegin(GL_LINE_STRIP);
@@ -591,6 +593,11 @@ void calc_viewborder(struct View3D *v3d, rcti *viewborder_r)
 		/* magic zoom calculation, no idea what
 	     * it signifies, if you find out, tell me! -zr
 		 */
+	/* simple, its magic dude!
+	 * well, to be honest, this gives a natural feeling zooming
+	 * with multiple keypad presses (ton)
+	 */
+	
 	zoomfac= (M_SQRT2 + v3d->camzoom/50.0);
 	zoomfac= (zoomfac*zoomfac)*0.25;
 	
@@ -628,7 +635,7 @@ static void drawviewborder(void)
 	x2= viewborder.xmax;
 	y2= viewborder.ymax;
 
-	/* rand */
+	/* edge */
 	setlinestyle(3);
 	cpack(0);
 	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); 
@@ -652,7 +659,7 @@ static void drawviewborder(void)
 		glRectf(x3,  y3,  x4,  y4); 
 	}
 
-	/* safetykader */
+	/* safety border */
 
 	fac= 0.1;
 	
@@ -698,7 +705,9 @@ void backdrawview3d(int test)
 			return;
 		}
 	}
-
+#ifdef __APPLE__
+	glDrawBuffer(GL_AUX0);
+#endif	
 	if(G.vd->drawtype > OB_WIRE) G.zbuf= TRUE;
 	curarea->win_swap &= ~WIN_BACK_OK;
 	
@@ -727,7 +736,7 @@ void backdrawview3d(int test)
 		base= (G.scene->base.first);
 		while(base) {
 			
-			/* elke base ivm meerdere windows */
+			/* each base, because of multiple windows */
 			base->selcol= 0x070707 | ( ((tel & 0xF00)<<12) + ((tel & 0xF0)<<8) + ((tel & 0xF)<<4) );
 			tel++;
 	
@@ -753,6 +762,10 @@ void backdrawview3d(int test)
 	G.zbuf= FALSE; 
 	glDisable(GL_DEPTH_TEST);
 	glEnable(GL_DITHER);
+
+#ifdef __APPLE__
+	glDrawBuffer(GL_BACK); /* we were in aux buffers */
+#endif
 }
 
 		
@@ -765,6 +778,19 @@ void drawname(Object *ob)
 	BMF_DrawString(G.font, ob->id.name+2);
 }
 
+
+static void draw_selected_name(char *name)
+{
+	char info[128];
+
+	sprintf(info, "(%d) %s", CFRA, name);
+
+	cpack(0xFFFFFF);
+	glRasterPos2i(30,  10);
+	BMF_DrawString(G.fonts, info);
+}
+
+
 static void draw_view_icon(void)
 {
 	BIFIconID icon;
@@ -784,12 +810,12 @@ static void draw_view_icon(void)
 	glDisable(GL_BLEND);
 }
 
-void drawview3d(void)
+void drawview3dspace(ScrArea *sa, void *spacedata)
 {
 	Base *base;
 	Object *ob;
 	
-	setwinmatrixview3d(0);	/* 0= geen pick rect */
+	setwinmatrixview3d(0);	/* 0= no pick rect */
 
 	setviewmatrixview3d();
 
@@ -841,10 +867,10 @@ void drawview3d(void)
 		clear_object_constraint_status(base->object);
 	}
 
-	/* eerst set tekenen */
+	/* draw set first */
 	if(G.scene->set) {
 	
-		/* patchje: kleur blijft constant */ 
+		/* patch: color remains constant */ 
 		G.f |= G_PICKSEL;
 
 		base= G.scene->set->base.first;
@@ -879,13 +905,7 @@ void drawview3d(void)
 		G.f &= ~G_PICKSEL;
 	}
 	
-/* SILLY CODE!!!! */
-/* See next silly code... why is the same code
- * ~ duplicated twice, and then this silly if(FALSE)
- * in part... wacky! and bad!
- */
-
-	/* eerst niet selected en dupli's */
+	/* first draw not selected and the duplis */
 	base= G.scene->base.first;
 	while(base) {
 		
@@ -893,14 +913,15 @@ void drawview3d(void)
 			
 			where_is_object(base->object);
 
+			/* dupli drawing temporal off here */
 			if(FALSE && base->object->transflag & OB_DUPLI) {
 				extern ListBase duplilist;
 				Base tbase;
 
-				/* altijd eerst original tekenen vanwege make_displist */
+				/* draw original always first because of make_displist */
 				draw_object(base);
 
-				/* patchje: kleur blijft constant */ 
+				/* patch: color remains constant */ 
 				G.f |= G_PICKSEL;
 				cpack(0x404040);
 				
@@ -925,7 +946,7 @@ void drawview3d(void)
 		
 		base= base->next;
 	}
-	/*  selected */
+	/* draw selected */
 	base= G.scene->base.first;
 	while(base) {
 		
@@ -936,8 +957,7 @@ void drawview3d(void)
 		base= base->next;
 	}
 
-/* SILLY CODE!!!! */
-	/* dupli's, als laatste om zeker te zijn de displisten zijn ok */
+	/* duplis, draw as last to make sure the displists are ok */
 	base= G.scene->base.first;
 	while(base) {
 		
@@ -946,7 +966,7 @@ void drawview3d(void)
 				extern ListBase duplilist;
 				Base tbase;
 
-				/* patchje: kleur blijft constant */ 
+				/* patch: color remains constant */ 
 				G.f |= G_PICKSEL;
 				cpack(0x404040);
 				
@@ -975,6 +995,9 @@ void drawview3d(void)
 	if(G.vd->persp>1) drawviewborder();
 	drawcursor();
 	draw_view_icon();
+
+	ob= OBACT;
+	if(ob!=0 && (U.uiflag & DRAWVIEWINFO)) draw_selected_name(ob->id.name+2);
 	
 	persp(1);
 
@@ -996,14 +1019,14 @@ void drawview3d(void)
 	 */
 void drawview3d_render(struct View3D *v3d)
 {
+	extern void mywindow_build_and_set_renderwin(void);
 	extern short v3d_windowmode;
 	Base *base;
 	Object *ob;
 
-		/* XXXXXXXX live and die by the hack */
 	free_all_realtime_images();
 	mywindow_build_and_set_renderwin();
-	
+		
 	v3d_windowmode= 1;
 	setwinmatrixview3d(0);
 	v3d_windowmode= 0;
@@ -1045,17 +1068,17 @@ void drawview3d_render(struct View3D *v3d)
 
 	test_all_displists();
 
-	/* niet erg nette calc_ipo en where_is forceer */
+	/* not really nice forcing of calc_ipo and where_is */
 	ob= G.main->object.first;
 	while(ob) {
 		ob->ctime= -123.456;
 		ob= ob->id.next;
 	}
 
-	/* eerst set tekenen */
+	/* first deaw set */
 	if(G.scene->set) {
 	
-		/* patchje: kleur blijft constant */ 
+		/* patch: color remains constant */ 
 		G.f |= G_PICKSEL;
 		
 		base= G.scene->set->base.first;
@@ -1092,7 +1115,7 @@ void drawview3d_render(struct View3D *v3d)
 	for (base = G.scene->base.first; base; base=base->next){
 		clear_object_constraint_status(base->object);
 	}
-	/* eerst niet selected en dupli's */
+	/* first not selected and duplis */
 	base= G.scene->base.first;
 	while(base) {
 		
@@ -1105,10 +1128,10 @@ void drawview3d_render(struct View3D *v3d)
 					extern ListBase duplilist;
 					Base tbase;
 					
-					/* altijd eerst original tekenen vanwege make_displist */
+					/* always draw original first because of make_displist */
 					draw_object(base);
 					
-					/* patchje: kleur blijft constant */ 
+					/* patch: color remains constant */ 
 					G.f |= G_PICKSEL;
 					cpack(0x404040);
 					
@@ -1133,7 +1156,7 @@ void drawview3d_render(struct View3D *v3d)
 		base= base->next;
 	}
 
-	/*  selected */
+	/* draw selected */
 	base= G.scene->base.first;
 	while(base) {
 		
@@ -1170,6 +1193,8 @@ int update_time(void)
 	static double ltime;
 	double time;
 
+	if ((U.mixbufsize)&&(audiostream_pos() != CFRA)&&(G.scene->audio.flag & AUDIO_SYNC)) return 0;
+
 	time = PIL_check_seconds_timer();
 	
 	tottime += (time - ltime);
@@ -1267,6 +1292,9 @@ void sumo_callback(void *obp)
     Mat3ToEul(matf, ob->rot);
 }
 
+/* for test and fun, i've written the next functions to play with dynamics
+   using a variant of play-anim... was never released nor really tested (ton) */
+
 void init_anim_sumo(void)
 {
 	extern Material defmaterial;
@@ -1442,7 +1470,7 @@ void update_anim_sumo(void)
 			
 			if(ob->sumohandle) {
 				if((ob->gameflag & OB_DYNAMIC)==0) {
-					/* evt: optimise, check for anim */
+					/* maybe: optimise, check for anim */
                     scaling[0] = ob->size[0];
                     scaling[1] = ob->size[1];
                     scaling[2] = ob->size[2];
@@ -1537,16 +1565,21 @@ void inner_play_anim_loop(int init, int mode)
 		sa= sa->next;	
 	}
 	
-	/* minimaal swaptime laten voorbijgaan */
+	/* make sure that swaptime passed by */
 	tottime -= swaptime;
 	while (update_time()) PIL_sleep_ms(1);
 
-	if(CFRA==EFRA) {
+	if(CFRA>=EFRA) {
 		if (tottime > 0.0) tottime = 0.0;
 		CFRA= SFRA;
+		audiostream_stop();
+		audiostream_start( CFRA );
 	}
-	else CFRA++;
-	
+	else
+	{
+		if (U.mixbufsize && (G.scene->audio.flag & AUDIO_SYNC)) CFRA = audiostream_pos();
+		else CFRA++;
+	};	
 }
 
 int play_anim(int mode)
@@ -1556,7 +1589,7 @@ int play_anim(int mode)
 	unsigned short event=0;
 	short val;
 	
-	/* patch voor zeer oude scenes */
+	/* patch for very very old scenes */
 	if(SFRA==0) SFRA= 1;
 	if(EFRA==0) EFRA= 250;
 	
@@ -1565,10 +1598,12 @@ int play_anim(int mode)
 	update_time();
 
 	/* waitcursor(1); */
-	G.f |= G_PLAYANIM;		/* in sequence.c en view.c wordt dit afgevangen */
+	G.f |= G_PLAYANIM;		/* in sequence.c and view.c this is handled */
 
 	cfraont= CFRA;
 	oldsa= curarea;
+
+	audiostream_start( CFRA );
 	
 	inner_play_anim_loop(1, mode);	/* 1==init */
 
@@ -1624,6 +1659,7 @@ int play_anim(int mode)
 	do_all_actions();
 	do_all_ikas();
 
+	audiostream_stop();
 
 	if(oldsa!=curarea) areawinset(oldsa->win);
 	
@@ -1642,8 +1678,8 @@ int play_anim(int mode)
 	allqueue(REDRAWIPO, 0);
 	allqueue(REDRAWNLA, 0);
 	allqueue (REDRAWACTION, 0);
-	/* vooropig */
-	update_for_newframe();
+	/* for the time being */
+	update_for_newframe_muted();
 #ifdef NAN_LINEAR_PHYSICS	
 	end_anim_sumo();
 #endif
