@@ -1,5 +1,5 @@
 /**
- * $Id: edit.c,v 1.4 2003/02/08 12:59:25 phase Exp $
+ * $Id: edit.c,v 1.10 2003/05/10 12:22:26 phase Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -76,6 +76,8 @@
 #include "BIF_space.h"
 #include "BIF_editview.h"
 #include "BIF_glutil.h"
+#include "BIF_toolbox.h"
+#include "BIF_editmesh.h"
 
 #include "BSE_view.h"
 #include "BSE_edit.h"
@@ -84,9 +86,10 @@
 #include "BSE_drawview.h"
 
 #include "BDR_editobject.h"
+#include "BDR_editmball.h"
+#include "BDR_editcurve.h"
 
 /* old stuff */
-#include "blendertimer.h"
 #include "blendef.h"
 #include "interface.h"
 #include "mydevice.h"
@@ -125,7 +128,7 @@ int get_border(rcti *rect, short col)
 
 	mywinset(G.curscreen->mainwin);
 	
-	/* pietsje groter, 1 pixel aan de rand */
+	/* slightly larger, 1 pixel at the edge */
 	glReadBuffer(GL_FRONT);
 	glDrawBuffer(GL_FRONT);
 
@@ -146,7 +149,7 @@ int get_border(rcti *rect, short col)
 
 	while(TRUE) {
 	
-		/* als een renderwindow open is en de muis gaat erin */
+		/* for when a renderwindow is open, and a mouse cursor activates it */
 		persp(1);
 		mywinset(curarea->win);
 		persp(0);
@@ -283,7 +286,7 @@ int get_border(rcti *rect, short col)
 			rect->ymax= mval[1];
 			retval= event;
 
-			/* normaliseren */
+			/* normalise */
 			if(rect->xmin>rect->xmax) SWAP(int, rect->xmin, rect->xmax);
 			if(rect->ymin>rect->ymax) SWAP(int, rect->ymin, rect->ymax);
 			
@@ -293,7 +296,7 @@ int get_border(rcti *rect, short col)
 	}
 
 
-	/* wissen */
+	/* clear */
 	if(event!=BKEY) {
 		if ELEM(curarea->spacetype, SPACE_VIEW3D, SPACE_IPO) {
 			scrarea_queue_winredraw(curarea);
@@ -335,7 +338,7 @@ void draw_sel_circle(short *mval, short *mvalo, float rad, float rado, int selec
 	persp(0);
 	glDrawBuffer(GL_FRONT);
 
-	/* cirkel tekenen */
+	/* draw circle */
 
 	if(mvalo && no_mvalo==0) {
 		sdrawXORcirc(mvalo[0], mvalo[1], rado);
@@ -378,7 +381,7 @@ void circle_selectCB(select_CBfunc callback)
 	
 	while(TRUE) {
 		
-		/* als een renderwindow open is en de muis gaat erin */
+		/* for when another window is open and a mouse cursor activates it */
 
 		mywinset(curarea->win);
 
@@ -429,7 +432,7 @@ void circle_selectCB(select_CBfunc callback)
 		}
 	}
 	
-	/* cirkel wissen */
+	/* clear circle */
 	draw_sel_circle(0, mvalo, 0, rad, 1);
 
 	countall();
@@ -542,7 +545,7 @@ void countall()
 		else if (G.obedit->type==OB_ARMATURE){
 			for (ebo=G.edbo.first;ebo;ebo=ebo->next){
 				
-				/* Synch selection to parent for ik children */
+				/* Sync selection to parent for ik children */
 				if ((ebo->flag & BONE_IK_TOPARENT) && ebo->parent){
 					G.totvert--;
 					if (ebo->parent->flag & BONE_TIPSEL)
@@ -618,7 +621,7 @@ void countall()
 		}
 	}
 	
-	allqueue(REDRAWINFO, 1);	/* 1, want header->win==0! */
+	allqueue(REDRAWINFO, 1);	/* 1, because header->win==0! */
 		return;
 	}
 	else if(G.f & (G_FACESELECT + G_VERTEXPAINT + G_TEXTUREPAINT +G_WEIGHTPAINT)) {
@@ -627,7 +630,7 @@ void countall()
 			G.totface= me->totface;
 			G.totvert= me->totvert;
 		}
-		allqueue(REDRAWINFO, 1);	/* 1, want header->win==0! */
+		allqueue(REDRAWINFO, 1);	/* 1, because header->win==0! */
 		return;
 	}
 
@@ -658,10 +661,11 @@ void countall()
 		}
 		base= base->next;
 	}
-	allqueue(REDRAWINFO, 1); 	/* 1, want header->win==0! */
+	allqueue(REDRAWINFO, 1); 	/* 1, because header->win==0! */
 }
 
-void snapmenu()
+
+void snap_sel_to_grid()
 {
 	extern TransVert *transvmain;
 	extern int tottrans;
@@ -669,16 +673,11 @@ void snapmenu()
 	TransVert *tv;
 	Base *base;
 	Object *ob;
-	float gridf, *curs, imat[3][3], bmat[3][3], vec[3], min[3], max[3], centroid[3];
-	int count, a;
-	short event;
-
-	event= pupmenu("SNAP %t|Sel -> Grid%x1|Sel -> Curs%x2|Curs-> Grid%x3|Curs-> Sel%x4");
+	float gridf, imat[3][3], bmat[3][3], vec[3];
+	int a;
 
 	gridf= G.vd->grid;
-	curs= give_cursor();
-	
-	if(event== 1 || event==2) {  /* sel->grid  sel->curs  */
+
 
 		if(G.obedit) {
 #ifdef __NLA
@@ -687,35 +686,29 @@ void snapmenu()
 			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
 #endif
 			if(tottrans==0) return;
-			
+
 			Mat3CpyMat4(bmat, G.obedit->obmat);
-			Mat3Inv(imat, bmat); 
+			Mat3Inv(imat, bmat);
 
 			tv= transvmain;
 			for(a=0; a<tottrans; a++, tv++) {
-			
-				if(event==2) {
-					
-					vec[0]= curs[0]-G.obedit->obmat[3][0];
-					vec[1]= curs[1]-G.obedit->obmat[3][1];
-					vec[2]= curs[2]-G.obedit->obmat[3][2];
-				}
-				else {
-					VECCOPY(vec, tv->loc);
-					Mat3MulVecfl(bmat, vec);
-					VecAddf(vec, vec, G.obedit->obmat[3]);
-					vec[0]= G.vd->grid*floor(.5+ vec[0]/gridf);
-					vec[1]= G.vd->grid*floor(.5+ vec[1]/gridf);
-					vec[2]= G.vd->grid*floor(.5+ vec[2]/gridf);
-					VecSubf(vec, vec, G.obedit->obmat[3]);
-				}
-				Mat3MulVecfl(imat, vec);
-				VECCOPY(tv->loc, vec);
+
+			VECCOPY(vec, tv->loc);
+			Mat3MulVecfl(bmat, vec);
+			VecAddf(vec, vec, G.obedit->obmat[3]);
+			vec[0]= G.vd->grid*floor(.5+ vec[0]/gridf);
+			vec[1]= G.vd->grid*floor(.5+ vec[1]/gridf);
+			vec[2]= G.vd->grid*floor(.5+ vec[2]/gridf);
+			VecSubf(vec, vec, G.obedit->obmat[3]);
+
+			Mat3MulVecfl(imat, vec);
+			VECCOPY(tv->loc, vec);
 
 			}
+
 			MEM_freeN(transvmain);
 			transvmain= 0;
-			
+
 			if ELEM(G.obedit->type, OB_SURF, OB_CURVE) makeDispList(G.obedit);
 
 			if (G.obedit->type == OB_ARMATURE)
@@ -734,20 +727,14 @@ void snapmenu()
 		while(base) {
 			if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
 				ob= base->object;
-				
-				if(event==2) {
-					vec[0]= -ob->obmat[3][0] + curs[0];
-					vec[1]= -ob->obmat[3][1] + curs[1];
-					vec[2]= -ob->obmat[3][2] + curs[2];
-				}
-				else {
-					vec[0]= -ob->obmat[3][0]+G.vd->grid*floor(.5+ ob->obmat[3][0]/gridf);
-					vec[1]= -ob->obmat[3][1]+G.vd->grid*floor(.5+ ob->obmat[3][1]/gridf);
-					vec[2]= -ob->obmat[3][2]+G.vd->grid*floor(.5+ ob->obmat[3][2]/gridf);
-				}
+
+				vec[0]= -ob->obmat[3][0]+G.vd->grid*floor(.5+ ob->obmat[3][0]/gridf);
+				vec[1]= -ob->obmat[3][1]+G.vd->grid*floor(.5+ ob->obmat[3][1]/gridf);
+				vec[2]= -ob->obmat[3][2]+G.vd->grid*floor(.5+ ob->obmat[3][2]/gridf);
+
 				if(ob->parent) {
 					where_is_object(ob);
-					
+
 					Mat3Inv(imat, originmat);
 					Mat3MulVecfl(imat, vec);
 					ob->loc[0]+= vec[0];
@@ -760,23 +747,127 @@ void snapmenu()
 					ob->loc[2]+= vec[2];
 				}
 			}
-			
+
 			base= base->next;
 		}
 		allqueue(REDRAWVIEW3D, 0);
-	}
-	else if(event==3) {   /* curs to grid */
-		curs[0]= G.vd->grid*floor(.5+curs[0]/gridf);
-		curs[1]= G.vd->grid*floor(.5+curs[1]/gridf);
-		curs[2]= G.vd->grid*floor(.5+curs[2]/gridf);
-		
+}
+
+void snap_sel_to_curs()
+{
+	extern TransVert *transvmain;
+	extern int tottrans;
+	extern float originmat[3][3];	/* object.c */
+	TransVert *tv;
+	Base *base;
+	Object *ob;
+	float gridf, *curs, imat[3][3], bmat[3][3], vec[3];
+	int a;
+
+	curs= give_cursor();
+
+		if(G.obedit) {
+#ifdef __NLA
+			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+#else
+			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+#endif
+			if(tottrans==0) return;
+
+			Mat3CpyMat4(bmat, G.obedit->obmat);
+			Mat3Inv(imat, bmat);
+
+			tv= transvmain;
+			for(a=0; a<tottrans; a++, tv++) {
+
+
+				vec[0]= curs[0]-G.obedit->obmat[3][0];
+				vec[1]= curs[1]-G.obedit->obmat[3][1];
+				vec[2]= curs[2]-G.obedit->obmat[3][2];
+
+
+				Mat3MulVecfl(imat, vec);
+				VECCOPY(tv->loc, vec);
+
+			}
+			MEM_freeN(transvmain);
+			transvmain= 0;
+
+			if ELEM(G.obedit->type, OB_SURF, OB_CURVE) makeDispList(G.obedit);
+
+			if (G.obedit->type == OB_ARMATURE)
+				special_trans_update(0);
+
+			allqueue(REDRAWVIEW3D, 0);
+			return;
+		}
+#ifdef __NLA
+		if (G.obpose){
+			allqueue(REDRAWVIEW3D, 0);
+			return;
+		}
+#endif
+		base= (G.scene->base.first);
+		while(base) {
+			if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
+				ob= base->object;
+
+				vec[0]= -ob->obmat[3][0] + curs[0];
+				vec[1]= -ob->obmat[3][1] + curs[1];
+				vec[2]= -ob->obmat[3][2] + curs[2];
+
+
+				if(ob->parent) {
+					where_is_object(ob);
+
+					Mat3Inv(imat, originmat);
+					Mat3MulVecfl(imat, vec);
+					ob->loc[0]+= vec[0];
+					ob->loc[1]+= vec[1];
+					ob->loc[2]+= vec[2];
+				}
+				else {
+					ob->loc[0]+= vec[0];
+					ob->loc[1]+= vec[1];
+					ob->loc[2]+= vec[2];
+				}
+			}
+
+			base= base->next;
+		}
 		allqueue(REDRAWVIEW3D, 0);
-	}
-	else if(event==4) {   /* curs to sel */
-		count= 0;
-		INIT_MINMAX(min, max);
-		centroid[0]= centroid[1]= centroid[2]= 0.0;
-		
+}
+
+void snap_curs_to_grid()
+{
+	float gridf, *curs;
+
+	gridf= G.vd->grid;
+	curs= give_cursor();
+
+	curs[0]= G.vd->grid*floor(.5+curs[0]/gridf);
+	curs[1]= G.vd->grid*floor(.5+curs[1]/gridf);
+	curs[2]= G.vd->grid*floor(.5+curs[2]/gridf);
+
+	allqueue(REDRAWVIEW3D, 0);
+
+}
+
+void snap_curs_to_sel()
+{
+	extern TransVert *transvmain;
+	extern int tottrans;
+	TransVert *tv;
+	Base *base;
+	float *curs, bmat[3][3], vec[3], min[3], max[3], centroid[3];
+	int count, a;
+
+	curs= give_cursor();
+
+	count= 0;
+	INIT_MINMAX(min, max);
+	centroid[0]= centroid[1]= centroid[2]= 0.0;
+
 		if(G.obedit) {
 			tottrans=0;
 #ifdef __NLA
@@ -785,7 +876,7 @@ void snapmenu()
 			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
 #endif
 			if(tottrans==0) return;
-			
+
 			Mat3CpyMat4(bmat, G.obedit->obmat);
 
 			tv= transvmain;
@@ -796,7 +887,7 @@ void snapmenu()
 				VecAddf(centroid, centroid, vec);
 				DO_MINMAX(vec, min, max);
 			}
-			
+
 			if(G.vd->around==V3D_CENTROID) {
 				VecMulf(centroid, 1.0/(float)tottrans);
 				VECCOPY(curs, centroid);
@@ -832,7 +923,316 @@ void snapmenu()
 				}
 			}
 		}
+		allqueue(REDRAWVIEW3D, 0);
+}
+
+void snap_curs_to_firstsel()
+{
+	extern TransVert *transvmain;
+	extern int tottrans;
+	TransVert *tv;
+	Base *base;
+	float *curs, bmat[3][3], vec[3], min[3], max[3], centroid[3];
+	int count, a;
+
+	curs= give_cursor();
+
+	count= 0;
+	INIT_MINMAX(min, max);
+	centroid[0]= centroid[1]= centroid[2]= 0.0;
+
+		if(G.obedit) {
+			tottrans=0;
+#ifdef __NLA
+			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+#else
+			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+#endif
+			if(tottrans==0) return;
+
+			Mat3CpyMat4(bmat, G.obedit->obmat);
+
+			tv= transvmain;
+			VECCOPY(vec, tv->loc);
+				/*Mat3MulVecfl(bmat, vec);
+				VecAddf(vec, vec, G.obedit->obmat[3]);
+				VecAddf(centroid, centroid, vec);
+				DO_MINMAX(vec, min, max);*/
+
+			if(G.vd->around==V3D_CENTROID) {
+				VecMulf(vec, 1.0/(float)tottrans);
+				VECCOPY(curs, vec);
+			}
+			else {
+				curs[0]= vec[0];
+				curs[1]= vec[1];
+				curs[2]= vec[2];
+			}
+			MEM_freeN(transvmain);
+			transvmain= 0;
+		}
+		else {
+			base= (G.scene->base.first);
+			while(base) {
+				if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
+					VECCOPY(vec, base->object->obmat[3]);
+					VecAddf(centroid, centroid, vec);
+					DO_MINMAX(vec, min, max);
+					count++;
+				}
+				base= base->next;
+			}
+			if(count) {
+				if(G.vd->around==V3D_CENTROID) {
+					VecMulf(centroid, 1.0/(float)count);
+					VECCOPY(curs, centroid);
+				}
+				else {
+					curs[0]= (min[0]+max[0])/2;
+					curs[1]= (min[1]+max[1])/2;
+					curs[2]= (min[2]+max[2])/2;
+				}
+			}
+		}
+		allqueue(REDRAWVIEW3D, 0);
+}
+
+void snap_to_center()
+{
+	extern TransVert *transvmain;
+	extern int tottrans;
+	extern float originmat[3][3];
+	TransVert *tv;
+	Base *base;
+	Object *ob;
+	float gridf, snaploc[3], imat[3][3], bmat[3][3], vec[3], min[3], max[3], centroid[3];
+	int count, a;
+
+
+/*calculate the snaplocation (centerpoint) */
+	count= 0;
+	INIT_MINMAX(min, max);
+	centroid[0]= centroid[1]= centroid[2]= 0.0;
+
+		if(G.obedit) {
+			tottrans=0;
+#ifdef __NLA
+			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+#else
+			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+#endif
+			if(tottrans==0) return;
+
+			Mat3CpyMat4(bmat, G.obedit->obmat);
+
+			tv= transvmain;
+			for(a=0; a<tottrans; a++, tv++) {
+				VECCOPY(vec, tv->loc);
+				Mat3MulVecfl(bmat, vec);
+				VecAddf(vec, vec, G.obedit->obmat[3]);
+				VecAddf(centroid, centroid, vec);
+				DO_MINMAX(vec, min, max);
+			}
+
+			if(G.vd->around==V3D_CENTROID) {
+				VecMulf(centroid, 1.0/(float)tottrans);
+				VECCOPY(snaploc, centroid);
+			}
+			else {
+				snaploc[0]= (min[0]+max[0])/2;
+				snaploc[1]= (min[1]+max[1])/2;
+				snaploc[2]= (min[2]+max[2])/2;
+			}
+
+		}
+		else {
+			base= (G.scene->base.first);
+			while(base) {
+				if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
+					VECCOPY(vec, base->object->obmat[3]);
+					VecAddf(centroid, centroid, vec);
+					DO_MINMAX(vec, min, max);
+					count++;
+				}
+				base= base->next;
+			}
+			if(count) {
+				if(G.vd->around==V3D_CENTROID) {
+					VecMulf(centroid, 1.0/(float)count);
+					VECCOPY(snaploc, centroid);
+				}
+				else {
+					snaploc[0]= (min[0]+max[0])/2;
+					snaploc[1]= (min[1]+max[1])/2;
+					snaploc[2]= (min[2]+max[2])/2;
+				}
+			}
+		}
+
+
+/* Snap the selection to the snaplocation (duh!) */
+
+		if(G.obedit) {
+#ifdef __NLA
+			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+#else
+			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
+#endif
+			if(tottrans==0) return;
+
+			Mat3CpyMat4(bmat, G.obedit->obmat);
+			Mat3Inv(imat, bmat);
+
+			tv= transvmain;
+			for(a=0; a<tottrans; a++, tv++) {
+
+
+				vec[0]= snaploc[0]-G.obedit->obmat[3][0];
+				vec[1]= snaploc[1]-G.obedit->obmat[3][1];
+				vec[2]= snaploc[2]-G.obedit->obmat[3][2];
+
+
+				Mat3MulVecfl(imat, vec);
+				VECCOPY(tv->loc, vec);
+
+			}
+			MEM_freeN(transvmain);
+			transvmain= 0;
+
+			if ELEM(G.obedit->type, OB_SURF, OB_CURVE) makeDispList(G.obedit);
+
+			if (G.obedit->type == OB_ARMATURE)
+				special_trans_update(0);
+
+			allqueue(REDRAWVIEW3D, 0);
+			return;
+		}
+#ifdef __NLA
+		if (G.obpose){
+			allqueue(REDRAWVIEW3D, 0);
+			return;
+		}
+#endif
+		base= (G.scene->base.first);
+		while(base) {
+			if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
+				ob= base->object;
+
+				vec[0]= -ob->obmat[3][0] + snaploc[0];
+				vec[1]= -ob->obmat[3][1] + snaploc[1];
+				vec[2]= -ob->obmat[3][2] + snaploc[2];
+
+
+				if(ob->parent) {
+					where_is_object(ob);
+
+					Mat3Inv(imat, originmat);
+					Mat3MulVecfl(imat, vec);
+					ob->loc[0]+= vec[0];
+					ob->loc[1]+= vec[1];
+					ob->loc[2]+= vec[2];
+				}
+				else {
+					ob->loc[0]+= vec[0];
+					ob->loc[1]+= vec[1];
+					ob->loc[2]+= vec[2];
+				}
+			}
+
+			base= base->next;
+		}
 
 		allqueue(REDRAWVIEW3D, 0);
+}
+
+
+void snapmenu()
+{
+	short event;
+
+	event = pupmenu("SNAP %t|Sel -> Grid%x1|Sel -> Curs%x2|Sel -> Center%x3|Curs-> Grid%x4|Curs-> Sel%x5");
+
+	switch (event) {
+	    case 1: /*Selection to grid*/
+		    snap_sel_to_grid();
+		    break;
+	    case 2: /*Selection to cursor*/
+		    snap_sel_to_curs();
+		    break;
+	    case 3: /*Selection to center of selection*/
+		    snap_to_center();
+		    break;
+	    case 4: /*Cursor to grid*/
+		    snap_curs_to_grid();
+		    break;
+	    case 5: /*Cursor to selection*/
+		    snap_curs_to_sel();
+		    break;
+	}
+}
+
+
+void mergemenu(void)
+{
+	extern float doublimit;
+	short event;
+
+	event = pupmenu("MERGE %t|At Center%x1|At Cursor%x2");
+
+	switch (event) {
+
+	    case 1: /*Merge at center of selection*/
+		    snap_to_center();
+		    notice("Removed: %d", removedoublesflag(1, doublimit));
+		    allqueue(REDRAWVIEW3D, 0);
+		    break;
+	    case 2: /*Merge at Cursor*/
+		    snap_sel_to_curs();
+		    notice("Removed: %d", removedoublesflag(1, doublimit));
+		    allqueue(REDRAWVIEW3D, 0);
+		    break;
+	}
+
+}
+
+void delete_context_selected(void) {
+	if(G.obedit) {
+		if(G.obedit->type==OB_MESH) delete_mesh();
+		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) delNurb();
+		else if(G.obedit->type==OB_MBALL) delete_mball();
+		else if (G.obedit->type==OB_ARMATURE) delete_armature();
+	}
+	else delete_obj(0);
+}
+
+void duplicate_context_selected(void) {
+	if(G.obedit) {
+		if(G.obedit->type==OB_MESH) adduplicate_mesh();
+		else if(G.obedit->type==OB_ARMATURE) adduplicate_armature();
+		else if(G.obedit->type==OB_MBALL) adduplicate_mball();
+		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) adduplicate_nurb();
+	}
+	else if(G.obpose){
+		error ("Duplicate not possible in posemode.");
+	}
+	else adduplicate(0);
+}
+
+void toggle_shading(void) {
+	if(G.qual & LR_CTRLKEY) {
+		reshadeall_displist();
+		G.vd->drawtype= OB_SHADED;
+	}
+	else if(G.qual & LR_SHIFTKEY) {
+		if(G.vd->drawtype== OB_SHADED) G.vd->drawtype= OB_WIRE;
+		else G.vd->drawtype= OB_SHADED;
+	}
+	else if(G.qual & LR_ALTKEY) {
+		if(G.vd->drawtype== OB_TEXTURE) G.vd->drawtype= OB_SOLID;
+		else G.vd->drawtype= OB_TEXTURE;
+	}
+	else {
+		if(G.vd->drawtype==OB_SOLID || G.vd->drawtype==OB_SHADED) G.vd->drawtype= OB_WIRE;
+		else G.vd->drawtype= OB_SOLID;
 	}
 }
