@@ -1,5 +1,5 @@
 /**
- * $Id: editface.c,v 1.4 2002/12/20 02:52:58 mein Exp $
+ * $Id: editface.c,v 1.8 2003/06/11 15:43:20 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -269,7 +269,7 @@ void select_linked_tfaces()
 	while(doit) {
 		doit= 0;
 		
-		/* select connected: array vullen */
+		/* select connected: fill array */
 		tface= me->tface;
 		mface= me->mface;
 		a= me->totface;
@@ -286,7 +286,7 @@ void select_linked_tfaces()
 			tface++; mface++;
 		}
 		
-		/* omgekeerd: vanuit array vlakken selecteren */
+		/* reverse: using array select the faces */
 
 		tface= me->tface;
 		mface= me->mface;
@@ -445,8 +445,13 @@ TFace* face_pick(Mesh *me, short x, short y)
 		backdrawview3d(0);
 	}
 	/* Read the pixel under the cursor */
+#ifdef __APPLE__
+	glReadBuffer(GL_AUX0);
+#endif
 	glReadPixels(x+curarea->winrct.xmin, y+curarea->winrct.ymin, 1, 1,
 		GL_RGBA, GL_UNSIGNED_BYTE, &col);
+	glReadBuffer(GL_BACK);
+
 	/* Unbelievable! */
 	if (G.order==B_ENDIAN) {
 		SWITCH_INT(col);
@@ -532,9 +537,12 @@ void face_borderselect()
 	
 	val= get_border(&rect, 3);
 	
-	/* why readbuffer here? shouldn't be necessary */
-	glReadBuffer(GL_BACK);	
-
+	/* why readbuffer here? shouldn't be necessary (maybe a flush or so) */
+	glReadBuffer(GL_BACK);
+#ifdef __APPLE__
+	glReadBuffer(GL_AUX0); /* apple only */
+#endif
+	
 	if(val) {
 		selar= MEM_callocN(me->totface+1, "selar");
 		
@@ -571,6 +579,9 @@ void face_borderselect()
 		allqueue(REDRAWVIEW3D, 0);
 		allqueue(REDRAWIMAGE, 0);
 	}
+#ifdef __APPLE__	
+	glReadBuffer(GL_BACK);
+#endif
 }
 
 #define TEST_STRUBI 1				
@@ -592,9 +603,11 @@ float CalcNormUV(float *a, float *b, float *c)
 #define UV_CUBE_MAPPING 2
 #define UV_CYL_MAPPING 3
 #define UV_SPHERE_MAPPING 4
+#define UV_BOUNDS8_MAPPING 68
 #define UV_BOUNDS4_MAPPING 65
 #define UV_BOUNDS2_MAPPING 66
 #define UV_BOUNDS1_MAPPING 67
+#define UV_STD8_MAPPING 131
 #define UV_STD4_MAPPING 130
 #define UV_STD2_MAPPING 129
 #define UV_STD1_MAPPING 128
@@ -631,9 +644,12 @@ void uv_autocalc_tface()
 	mode= pupmenu(MENUTITLE("UV Calculation")
 	              MENUSTRING("Cube",          UV_CUBE_MAPPING) "|"
 	              MENUSTRING("Cylinder",      UV_CYL_MAPPING) "|"
+                      MENUSTRING("Sphere",	  UV_SPHERE_MAPPING) "|"
+	              MENUSTRING("Bounds to 1/8", UV_BOUNDS8_MAPPING) "|"
 	              MENUSTRING("Bounds to 1/4", UV_BOUNDS4_MAPPING) "|"
 	              MENUSTRING("Bounds to 1/2", UV_BOUNDS2_MAPPING) "|"
 	              MENUSTRING("Bounds to 1/1", UV_BOUNDS1_MAPPING) "|"
+	              MENUSTRING("Standard 1/8",  UV_STD8_MAPPING) "|"
 	              MENUSTRING("Standard 1/4",  UV_STD4_MAPPING) "|"
 	              MENUSTRING("Standard 1/2",  UV_STD2_MAPPING) "|"
 	              MENUSTRING("Standard 1/1",  UV_STD1_MAPPING) "|"
@@ -687,6 +703,9 @@ void uv_autocalc_tface()
 			}
 		}
 
+	case UV_BOUNDS8_MAPPING:		
+		fac = 0.125;
+		goto bounds_mapping;
 	case UV_BOUNDS4_MAPPING:		
 		fac = 0.25;
 		goto bounds_mapping;
@@ -780,6 +799,9 @@ void uv_autocalc_tface()
 		}
 		break;
 
+	case UV_STD8_MAPPING:
+		fac = 0.125;
+		goto standard_mapping;
 	case UV_STD4_MAPPING:
 		fac = 0.25;
 		goto standard_mapping;
