@@ -1,5 +1,5 @@
 /**
-* $Id: editnla.c,v 1.8 2003/01/30 06:19:49 hos Exp $
+* $Id: editnla.c,v 1.15 2003/07/22 02:38:27 hos Exp $
 *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -75,12 +75,12 @@
 #include "BSE_drawipo.h"
 #include "BSE_trans_types.h"
 #include "BSE_edit.h"
+#include "BSE_filesel.h"
 #include "BDR_editobject.h"
 
 #include "interface.h"
 #include "blendef.h"
 #include "mydevice.h"
-#include "blendertimer.h"
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -88,11 +88,6 @@
 
 /* Note: A lot of these pretty much duplicate the behaviour of the
 action windows.  The functions should be shared, not copy-pasted */
-typedef struct NlaParam{
-	SpaceNla *snla;
-	unsigned short event;
-	short val;
-}NlaParam;
 
 static void deselect_nlachannel_keys (int test);
 static void deselect_nlachannels(int test);
@@ -116,28 +111,19 @@ extern int nla_filter (Base* base, int flags);	/* From drawnla.c */
 
 /* ******************** SPACE: NLA ********************** */
 
-/* Protected creator function */
-int calc_memleak (void* ptr){
+void winqreadnlaspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
+{
+	unsigned short event= evt->event;
+	short val= evt->val;
+	char ascii= evt->ascii;
+	SpaceNla *snla = curarea->spacedata.first;
 	int doredraw= 0;
 	short	mval[2];
 	float dx,dy;
 	int	cfra;
-	SpaceNla *snla;
-	NlaParam *params=(NlaParam*) ptr;
-	unsigned short event;
-	short val ;
-
-	if (!ptr)
-		return -1;
-
-	snla= params->snla;
-	event = params->event;
-	val = params->val;
 	
-	if(curarea->win==0) return 0;
-	
-	if (!snla)
-		return 0;
+	if (curarea->win==0) return;
+	if (!snla) return;
 	
 	if(val) {
 		if( uiDoBlocks(&curarea->uiblocks, event)!=UI_NOTHING ) event= 0;
@@ -154,7 +140,7 @@ int calc_memleak (void* ptr){
 		case DKEY:
 			if (G.qual & LR_SHIFTKEY && mval[0]>=NLAWIDTH){
 				duplicate_nlachannel_keys();
-				update_for_newframe();
+				update_for_newframe_muted();
 			}
 			break;
 		case DELKEY:
@@ -163,17 +149,17 @@ int calc_memleak (void* ptr){
 				delete_nlachannel_keys ();
 			else
 				delete_nlachannels();
-			update_for_newframe();
+			update_for_newframe_muted();
 			break;
 		case GKEY:
 			if (mval[0]>=NLAWIDTH)
 				transform_nlachannel_keys ('g');
-			update_for_newframe();
+			update_for_newframe_muted();
 			break;
 		case SKEY:
 			if (mval[0]>=NLAWIDTH)
 				transform_nlachannel_keys ('s');
-			update_for_newframe();
+			update_for_newframe_muted();
 			break;
 		case BKEY:
 			borderselect_nla();
@@ -239,25 +225,6 @@ int calc_memleak (void* ptr){
 	}
 	
 	if(doredraw) scrarea_queue_winredraw(curarea);
-	return 0;
-}
-
-void winqreadnlaspace(unsigned short event, short val, char ascii)
-{
-	NlaParam param;
-	Base *base;
-	bActionStrip *strip, *next;
-	short mval[2];
-	float dx, dy;
-	int cfra;
-
-	param.event = event;
-	param.val = val;
-	param.snla = curarea->spacedata.first;
-
-
-	/* Call the protected (&obfuscated) eventloop function */
-	calc_memleak(&param); /* enable NLA */
 }
 
 static void convert_nla(short mval[2])
@@ -340,19 +307,69 @@ static void convert_nla(short mval[2])
 	}
 }
 
+static Base *nla_base=NULL;	/* global, bad, bad! put it in nla space later, or recode the 2 functions below (ton) */
+
+static void add_nla_block(short event)
+{
+	bAction *act=NULL;
+	bActionStrip *strip;
+	int		cur;
+
+	if (event!=-1){
+		for (cur = 1, act=G.main->action.first; act; act=act->id.next, cur++){
+			if (cur==event){
+				break;
+			}
+		}
+	}
+	
+	/* Bail out if no action was chosen */
+	if (!act){
+		return;
+	}
+	
+	/* Initialize the new action block */
+	strip = MEM_callocN(sizeof(bActionStrip), "bActionStrip");
+	
+	deselect_nlachannel_keys(0);
+	
+	/* Link the action to the strip */
+	strip->act = act;
+	strip->actstart = 1.0;
+	strip->actend = calc_action_end(act);
+	strip->start = G.scene->r.cfra; /* Should be mval[0] */
+	strip->end = strip->start + (strip->actend-strip->actstart);
+	strip->flag = ACTSTRIP_SELECT;
+	strip->repeat = 1.0;
+	
+	act->id.us++;
+	
+	BLI_addtail(&nla_base->object->nlastrips, strip);
+
+}
+
+static void add_nla_databrowse_callback(int val)
+{
+	/* val is not used, databrowse needs it to optional pass an event */
+	short event;
+	
+	if(nla_base==NULL) return;
+	
+	event= G.snla->menunr;	/* set by databrowse or pupmenu */
+	
+	add_nla_block(event);
+}
+
 static void add_nlablock(short mval[2])
 {
 	/* Make sure we are over an armature */
 	Base *base;
-	bAction *act=NULL;
-	bActionStrip *strip;
 	float ymin, ymax;
 	float x, y;
 	rctf	rectf;
 	short event;
 	char *str;
 	short nr;
-	int		cur;
 	
 	areamouseco_to_ipoco(G.v2d, mval, &x, &y);
 	
@@ -392,6 +409,9 @@ static void add_nlablock(short mval[2])
 		}
 	}	
 	
+	/* global... for the call above, because the NLA system seems not to have an 'active strip' stored */
+	nla_base= base;
+	
 	/* Make sure we have an armature */
 	if (!base){
 		error ("Not an armature!");
@@ -400,41 +420,25 @@ static void add_nlablock(short mval[2])
 	
 	/* Popup action menu */
 	IDnames_to_pupstring(&str, "Add action", NULL, &G.main->action, (ID *)G.scene, &nr);
+	
+	if(strncmp(str+13, "DataBrow", 8)==0) {
+		MEM_freeN(str);
 
-	event = pupmenu(str);
-	
-	if (event!=-1){
-		for (cur = 1, act=G.main->action.first; act; act=act->id.next, cur++){
-			if (cur==event){
-				break;
-			}
-		}
+		activate_databrowse((ID *)NULL, ID_AC, 0, 0, &G.snla->menunr, 
+							add_nla_databrowse_callback );
+		
+		return;			
+	}
+	else {
+		event = pupmenu(str);
+		MEM_freeN(str);
+		add_nla_block(event);
 	}
 	
-	MEM_freeN(str);
-	
-	/* Bail out if no action was chosen */
-	if (!act){
-		return;
-	}
-	
-	/* Initialize the new action block */
-	strip = MEM_callocN(sizeof(bActionStrip), "bActionStrip");
-	
-	deselect_nlachannel_keys(0);
-	
-	/* Link the action to the strip */
-	strip->act = act;
-	strip->actstart = 1.0;
-	strip->actend = calc_action_end(act);
-	strip->start = G.scene->r.cfra; /* Should be mval[0] */
-	strip->end = strip->start + (strip->actend-strip->actstart);
-	strip->flag = ACTSTRIP_SELECT;
-	strip->repeat = 1.0;
-	
-	act->id.us++;
-	
-	BLI_addtail(&base->object->nlastrips, strip);
+	/* Ton: this is a callback for databrowse too
+	   Hos: no, I don't think it is
+	   add_nla_block(0);
+	*/
 }
 
 static void mouse_nlachannels(short mval[2])
@@ -979,6 +983,8 @@ static void delete_nlachannel_keys(void)
 	}
 	
 	allspace(REMAKEALLIPO, 0);
+        allspace(REMAKEIPO,0);
+	allqueue (REDRAWVIEW3D, 0);
 	allqueue(REDRAWNLA, 0);
 	allqueue(REDRAWIPO, 0);
 }
@@ -1569,7 +1575,7 @@ void clever_numbuts_nla(void){
 		strip->blendout = (strip->end-strip->start-strip->blendin);
 	
 	
-	update_for_newframe();
+	update_for_newframe_muted();
 	allqueue (REDRAWNLA, 0);
 	allqueue (REDRAWVIEW3D, 0);
 }	
