@@ -1,5 +1,5 @@
 /**
- * $Id: editscreen.c,v 1.6 2003/02/10 01:08:54 sgefant Exp $
+ * $Id: editscreen.c,v 1.18 2003/07/20 20:25:30 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,6 +44,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>  /* isprint */
 #include <stdio.h>
 #include <math.h>
 
@@ -105,10 +106,10 @@
 
 /* TIPS:
  * 
- * - LET OP DE EDGES,  VERTICES ERVAN MOETEN IN VOLGORDE
-	 (laagste pointer eerst). Anders onvoorspelbare effecten!
- * - probleem: flags zijn nog niet echt netjes. Altijd na gebruik
-	 op nul zetten.
+ * - WATCH THE EDGES,  VERTICES HAVE TO BE IN ORDER...
+	 (lowest pointer first). Otherwise unpredictable effects!
+ * - problem: flags here are not nicely implemented. After usage
+	 always reset to zero.
  */
 
 static void testareas(void);
@@ -184,12 +185,12 @@ void setcursor_space(int spacetype, short cur)
 
 /* *********  IN/OUT  ************* */
 
-void getmouseco_sc(short *mval)		/* screen coordinaten */
+void getmouseco_sc(short *mval)		/* screen coordinates */
 {
 	getmouse(mval);
 }
 
-void getmouseco_areawin(short *mval)		/* interne area coordinaten */
+void getmouseco_areawin(short *mval)		/* internal area coordinates */
 {
 	getmouseco_sc(mval);
 	
@@ -199,7 +200,7 @@ void getmouseco_areawin(short *mval)		/* interne area coordinaten */
 	}
 }
 
-void getmouseco_headwin(short *mval)		/* interne area coordinaten */
+void getmouseco_headwin(short *mval)		/* internal area coordinates */
 {
 	getmouseco_sc(mval);
 	
@@ -390,9 +391,16 @@ static void headmenu(ScrArea *sa)
 static void addqueue_ext(short win, unsigned short event, short val, char ascii)
 {
 	if (win<4 || !areawinar[win]) {
-		printf("bad call to addqueue: %d (%d, %d)\n", win, event, val);
-	} else {
-		bwin_qadd(win, event, val, ascii);
+		if(win==0) // other win ids are for mainwin & renderwin
+			printf("bad call to addqueue: %d (%d, %d)\n", win, event, val);
+	} 
+	else {
+		BWinEvent evt;
+		evt.event= event;
+		evt.val= val;
+		evt.ascii= ascii;
+		
+		bwin_qadd(win, &evt);
 	}
 }
 
@@ -419,21 +427,18 @@ static void scrollheader(ScrArea *area);
 static void scrarea_dispatch_header_events(ScrArea *sa)
 {
 	ScrArea *tempsa;
+	BWinEvent evt;
 	short do_redraw=0, do_change=0;
 	
 	areawinset(sa->headwin);
 	
-	while(bwin_qtest(sa->headwin)) {
-		char ascii;
-		short val;
-		unsigned short event= bwin_qread(sa->headwin, &val, &ascii);
+	while(bwin_qread(sa->headwin, &evt)) {
+		if(evt.val) {
+			if( uiDoBlocks(&curarea->uiblocks, evt.event)!=UI_NOTHING ) evt.event= 0;
 
-		if(val) {
-			if( uiDoBlocks(&curarea->uiblocks, event)!=UI_NOTHING ) event= 0;
-
-			switch(event) {
+			switch(evt.event) {
 			case UI_BUT_EVENT:
-				do_headerbuttons(val);
+				do_headerbuttons(evt.val);
 				break;
 			
 			case LEFTMOUSE:
@@ -460,7 +465,7 @@ static void scrarea_dispatch_header_events(ScrArea *sa)
 				break;
 			default:
 				if (winqueue_break == 0) {
-					scrarea_do_winhandle(sa, event, val, ascii);
+					scrarea_do_winhandle(sa, &evt);
 					if (winqueue_break == 0) areawinset(sa->headwin);
 				}
 			}
@@ -469,11 +474,11 @@ static void scrarea_dispatch_header_events(ScrArea *sa)
 		}
 	}
 
-	/* test: bestaat window nog */	
+	/* test: does window still exist? */	
 	tempsa= areawinar[sa->headwin];
 	if(tempsa==0) return;
 	
-	/* dit onderscheid loopt niet lekker... */
+	/* this functional separation does not work as well as i expected... */
 	if(do_change) scrarea_do_headchange(sa);
 	if(do_redraw) scrarea_do_headdraw(sa);
 }
@@ -481,31 +486,28 @@ static void scrarea_dispatch_header_events(ScrArea *sa)
 static void scrarea_dispatch_events(ScrArea *sa)
 {
 	ScrArea *tempsa;
+	BWinEvent evt;
 	short do_redraw=0, do_change=0;
 	
 	if(sa!=curarea || sa->win!=mywinget()) areawinset(sa->win);
 
-	while(bwin_qtest(sa->win)) {
-		char ascii;
-		short val;
-		unsigned short event= bwin_qread(sa->win, &val, &ascii);
-		
-		if(event==REDRAW) {
+	while(bwin_qread(sa->win, &evt)) {
+		if(evt.event==REDRAW) {
 			do_redraw= 1;
 		}
-		else if(event==CHANGED) {
+		else if(evt.event==CHANGED) {
 			sa->win_swap= 0;
 			do_change= 1;
 			do_redraw= 1;
 		}
 		else {
-			scrarea_do_winhandle(sa, event, val, ascii);
+			scrarea_do_winhandle(sa, &evt);
 		}
 		
 		if(winqueue_break) return;
 	}
 
-	/* test: bestaat window nog */	
+	/* test: does window still exist */	
 	tempsa= areawinar[sa->win];
 	if(tempsa==0) return;
 
@@ -536,8 +538,8 @@ void markdirty_all()
 
 int is_allowed_to_change_screen(bScreen *new)
 {
-	/* niet als curscreen is full
-	 * niet als obedit && old->scene!=new->scene
+	/* not when curscreen is full
+	 * not when obedit && old->scene!=new->scene
 	 */
 	
 	if(new==0) return 0;
@@ -808,7 +810,7 @@ unsigned short screen_qread(short *val, char *ascii)
 
 unsigned short extern_qread_ext(short *val, char *ascii)
 {
-	/* bewaart de laatste INPUTCHANGE en de laatste REDRAW */
+	/* stores last INPUTCHANGE and last REDRAW */
 	unsigned short event;
 	
 	event= screen_qread(val, ascii);
@@ -816,7 +818,7 @@ unsigned short extern_qread_ext(short *val, char *ascii)
 	else if(event==REDRAW) ext_redraw= *val;
 	else if(event==INPUTCHANGE) ext_inputchange= *val;
 	else if(event==MOUSEY || event==MOUSEX) ext_mousemove= 1;
-	else if((G.qual & LR_CTRLKEY) && event==F3KEY) {
+	else if((G.qual & (LR_CTRLKEY|LR_ALTKEY)) && event==F3KEY) {
 		BIF_screendump();
 	}
 
@@ -828,6 +830,30 @@ unsigned short extern_qread(short *val)
 	return extern_qread_ext(val, &ascii);
 }
 
+int blender_test_break(void)
+{
+	if (!G.background) {
+		static double ltime= 0;
+		double curtime= PIL_check_seconds_timer();
+
+			/* only check for breaks every 10 milliseconds
+			 * if we get called more often.
+			 */
+		if ((curtime-ltime)>.001) {
+			ltime= curtime;
+
+			while(qtest()) {
+				short val;
+				if (extern_qread(&val) == ESCKEY) {
+					G.afbreek= 1;
+				}
+			}
+		}
+	}
+
+	return (G.afbreek==1);
+}
+
 void reset_autosave(void) {
 	window_set_timer(mainwin, U.savetime*60*1000, AUTOSAVE_FILE);
 }
@@ -841,7 +867,7 @@ static void screen_dispatch_events(void) {
 				
 		winqueue_break= 0;
 		for (sa= G.curscreen->areabase.first; sa; sa= sa->next) {
-				/* bewust eerst header afhandelen, dan rest. Header is soms init */
+				/* first check header, then rest. Header sometimes has initialization code */
 			if (sa->headwin && bwin_qtest(sa->headwin)) {
 				scrarea_dispatch_header_events(sa);
 				events_remaining= 1;
@@ -881,6 +907,7 @@ void screenmain(void)
 {
 	int has_input= 1;
 	int firsttime = 1;
+	
 	while (1) {
 		unsigned short event;
 		short val, towin;
@@ -889,17 +916,24 @@ void screenmain(void)
 		flush_extqd_events();
 		if (nafterqitems && !qtest()) {
 			append_afterqueue();
-			event= 0;
+			event= val= ascii= 0;
 		} else {
 			event= screen_qread(&val, &ascii);
 		}
 		
 		window_make_active(mainwin);
-		
+
 		if (event==INPUTCHANGE) {
 			has_input= val;
 		}
 		
+			/* If the main window is active, find the current active ScrArea
+			 * underneath the mouse cursor, updating the headers & cursor for
+			 * the appropriate internal window if things have changed.
+			 * 
+			 * If the main window is not active, deactivate the internal 
+			 * window.
+			 */
 		if (has_input) {
 			ScrArea *newactarea;
 			int newactwin;
@@ -1041,7 +1075,7 @@ void screenmain(void)
 			}
 		}
 		
-		if (towin) {
+		if (towin && event) {
 			if (blenderqread(event, val))
 				addqueue_ext(G.curscreen->winakt, event, val, ascii);
 		}
@@ -1082,6 +1116,15 @@ void screenmain(void)
 	}
 }
 
+#ifdef _WIN32	// FULLSCREEN
+void mainwindow_toggle_fullscreen(int fullscreen){
+	if (fullscreen) U.uiflag |= FLIPFULLSCREEN;
+	else U.uiflag &= ~FLIPFULLSCREEN;
+
+	window_toggle_fullscreen(mainwin, fullscreen);
+}
+#endif
+
 void mainwindow_raise(void) {
 	window_raise(mainwin);
 }
@@ -1190,12 +1233,12 @@ static void removedouble_scrverts(void)
 		verg= verg->next;
 	}
 	
-	/* vervang pointers in edges en vlakken */
+	/* replace pointers in edges and faces */
 	se= G.curscreen->edgebase.first;
 	while(se) {
 		if(se->v1->newv) se->v1= se->v1->newv;
 		if(se->v2->newv) se->v2= se->v2->newv;
-		/* edges zijn veranderd: dus.... */
+		/* edges changed: so.... */
 		sortscrvert(&(se->v1), &(se->v2));
 		se= se->next;
 	}
@@ -1208,7 +1251,7 @@ static void removedouble_scrverts(void)
 		sa= sa->next;
 	}
 	
-	/* verwijderen */
+	/* remove */
 	verg= G.curscreen->vertbase.first;
 	while(verg) {
 		v1= verg->next;
@@ -1226,7 +1269,7 @@ static void removenotused_scrverts(void)
 	ScrVert *sv, *svn;
 	ScrEdge *se;
 
-	/* ga ervan uit dat de edges goed zijn */
+	/* we assume edges are ok */
 	
 	se= G.curscreen->edgebase.first;
 	while(se) {
@@ -1251,7 +1294,7 @@ static void removedouble_scredges(void)
 {
 	ScrEdge *verg, *se, *sn;
 	
-	/* vergelijken */
+	/* compare */
 	verg= G.curscreen->edgebase.first;
 	while(verg) {
 		se= verg->next;
@@ -1273,7 +1316,7 @@ static void removenotused_scredges(void)
 	ScrArea *sa;
 	int a=0;
 	
-	/* zet flag als edge gebruikt wordt in area */
+	/* sets flags when edge is used in area */
 	sa= G.curscreen->areabase.first;
 	while(sa) {
 		se= screen_findedge(G.curscreen, sa->v1, sa->v2);
@@ -1330,7 +1373,7 @@ void calc_arearcts(ScrArea *sa)
 	}
 	if(sa->winrct.ymin>sa->winrct.ymax) sa->winrct.ymin= sa->winrct.ymax;
 	
-	/* als speedup voor berekeningen */
+	/* for speedup */
 	sa->winx= sa->winrct.xmax-sa->winrct.xmin+1;
 	sa->winy= sa->winrct.ymax-sa->winrct.ymin+1;
 }
@@ -1342,7 +1385,7 @@ static void openheadwin(ScrArea *sa)
 
 	glMatrixMode(GL_MODELVIEW);
 	
-	areawinar[sa->headwin]= sa;	/* anders doet addqueue het niet */
+	areawinar[sa->headwin]= sa;	/* oterwise addqueue does not work */
 	addqueue(sa->headwin, CHANGED, 1);
 }
 
@@ -1351,7 +1394,7 @@ static void openareawin(ScrArea *sa)
 	sa->win= myswinopen(G.curscreen->mainwin, 
 		sa->winrct.xmin, sa->winrct.xmax, sa->winrct.ymin, sa->winrct.ymax);
 
-	areawinar[sa->win]= sa;	/* anders doet addqueue het niet */
+	areawinar[sa->win]= sa;	/* otherwise addqueue does not work */
 	addqueue(sa->win, CHANGED, 1);
 }
 
@@ -1422,8 +1465,8 @@ static void testareas(void)
 {
 	ScrArea *sa;
 
-	/* testen of header er moet zijn, of weg moet, of verplaatst */
-	/* testen of window er moet zijn, of weg moet, of verplaatst */
+	/* test for header, if removed, or moved */
+	/* testen for window, if removed, or moved */
 
 	for (sa= G.curscreen->areabase.first; sa; sa= sa->next) {
 		rcti oldhr= sa->headrct;
@@ -1461,14 +1504,14 @@ static void testareas(void)
 		areawinar[sa->win]= sa;
 	}
 	
-		/* test of winakt in orde is */	
+		/* test if winakt is OK */	
 	if( areawinar[G.curscreen->winakt]==0) G.curscreen->winakt= 0;
 }
 
 static ScrArea *test_edge_area(ScrArea *sa, ScrEdge *se)
 {
-	/* test of edge in area ligt, zo niet, 
-	   vind een area die 'm wel heeft */
+	/* test if edge is in area, if not, 
+	   then find an area that has it */
   
 	ScrEdge *se1=0, *se2=0, *se3=0, *se4=0;
 	
@@ -1482,7 +1525,7 @@ static ScrArea *test_edge_area(ScrArea *sa, ScrEdge *se)
 		
 		sa= G.curscreen->areabase.first;
 		while(sa) {
-			/* een beetje optimaliseren? */
+			/* a bit optimise? */
 			if(se->v1==sa->v1 || se->v1==sa->v2 || se->v1==sa->v3 || se->v1==sa->v4) {
 				se1= screen_findedge(G.curscreen, sa->v1, sa->v2);
 				se2= screen_findedge(G.curscreen, sa->v2, sa->v3);
@@ -1494,7 +1537,7 @@ static ScrArea *test_edge_area(ScrArea *sa, ScrEdge *se)
 		}
 	}
 
-	return sa;	/* is keurig 0 als niet gevonden */
+	return sa;	/* is null when not find */
 }
 
 ScrArea *closest_bigger_area(void)
@@ -1514,7 +1557,7 @@ ScrArea *closest_bigger_area(void)
 		if(sa!=curarea) {
 			if(sa->winy>=curarea->winy) {
 			
-				/* mimimum van vier hoekpunten */
+				/* mimimum of the 4 corners */
 				vec[0]= sa->v1->vec.x; vec[1]= sa->v1->vec.y;
 				len= VecLenf(vec, cent);
 				vec[0]= sa->v2->vec.x; vec[1]= sa->v2->vec.y;
@@ -1526,13 +1569,13 @@ ScrArea *closest_bigger_area(void)
 				
 				len= MIN4(len, len1, len2, len3);
 				
-				/* plus centrum */
+				/* plus centre */
 				vec[0]= (sa->v2->vec.x+sa->v3->vec.x)/2;
 				vec[1]= (sa->v1->vec.y+sa->v2->vec.y)/2;
 
 				len+= 0.5*VecLenf(vec, cent);
 				
-				/* min afmeting */
+				/* min size */
 				len-= sa->winy+sa->winx;
 				
 				if(len<dist) {
@@ -1548,7 +1591,7 @@ ScrArea *closest_bigger_area(void)
 	else return curarea;
 }
 
-/* ************ SCREENBEHEER ************** */
+/* ************ SCREEN MANAGEMENT ************** */
 
 static int statechanged= 0;
 void BIF_wait_for_statechange(void)
@@ -1596,29 +1639,25 @@ static unsigned short convert_for_nonumpad(unsigned short event)
 
 void add_to_mainqueue(Window *win, void *user_data, short evt, short val, char ascii)
 {
-	short qual= window_get_qual(win);
 
 	statechanged= 1;
 
-	if (U.flag&NONUMPAD) {
+	if (U.flag & NONUMPAD) {
 		evt= convert_for_nonumpad(evt);
 	}
-	
-		/* enforce some guarentees about ascii values... these should
-		 * be enforced in ghost probably - zr
-		 */
 
-	if (!val || !isprint(ascii) || (qual&~LR_SHIFTKEY)) {
+	/*  accept the extended ascii set (ton) */
+	if( !val || ascii<32 ) {
 		ascii= '\0';
 	}
 
 	mainqenter_ext(evt, val, ascii);
 }
 
-static bScreen *addscreen(char *name)		/* gebruik de setprefsize() als je anders dan fullscreen wilt */
+static bScreen *addscreen(char *name)		/* use setprefsize() if you want something else than a full windpw */
 {
-	/* deze functie zet de variabele G.curscreen
-	 * omdat alle hulpfuncties moeten weten welk screen
+	/* this function sets variabele G.curscreen,
+	 * that global is about used everywhere!
 	 */
 	bScreen *sc;
 	ScrVert *sv1, *sv2, *sv3, *sv4;
@@ -1645,8 +1684,16 @@ static bScreen *addscreen(char *name)		/* gebruik de setprefsize() als je anders
 	
 	sc->scene= G.scene;
 	
-	if (!mainwin) {
+  	if (!mainwin) {
+#ifdef _WIN32	// FULLSCREEN
+		if (G.windowstate == G_WINDOWSTATE_FULLSCREEN)
+			mainwin= window_open("Blender", sc->startx, sc->starty, sc->sizex, sc->sizey, G_WINDOWSTATE_FULLSCREEN);
+		else
+			mainwin= window_open("Blender", sc->startx, sc->starty, sc->sizex, sc->sizey, start_maximized);
+#else
 		mainwin= window_open("Blender", sc->startx, sc->starty, sc->sizex, sc->sizey, start_maximized);
+#endif
+		
 		if (!mainwin) {
 			printf("ERROR: Unable to open Blender window\n");
 			exit(1);
@@ -1697,7 +1744,7 @@ void setscreen(bScreen *sc)
 	ScrArea *sa;
 	short mval[2];
 	
-	if(sc->full) {				/* vind de bijhorende full */
+	if(sc->full) {				/* find associated full */
 		sc1= G.main->screen.first;
 		while(sc1) {
 			sa= sc1->areabase.first;
@@ -1710,7 +1757,7 @@ void setscreen(bScreen *sc)
 		if(sc1==0) printf("setscreen error\n");
 	}
 
-	/* G.curscreen de-activeren */
+	/* de-activate G.curscreen */
 	if (G.curscreen && G.curscreen != sc) {
 		sa= G.curscreen->areabase.first;
 		while(sa) {
@@ -1756,7 +1803,7 @@ void setscreen(bScreen *sc)
 			if (sl->spacetype==SPACE_OOPS) {
 				SpaceOops *soops= (SpaceOops*) sl;
 
-				/* patch als deze in oude files zit */
+				/* patch for old files */
 				if (soops->v2d.cur.xmin==soops->v2d.cur.xmax) {
 					extern void init_v2d_oops(View2D*);
 					init_v2d_oops(&soops->v2d);
@@ -1776,20 +1823,20 @@ void setscreen(bScreen *sc)
 	mainqenter(DRAWEDGES, 1);
 	dodrawscreen= 1;		/* patch! even gets lost,,,? */
 
-	winqueue_break= 1;		/* overal uit de queue's gaan */
+	winqueue_break= 1;		/* means leave queue everywhere */
 }
 
 static void splitarea(ScrArea *sa, char dir, float fac);
 
-void area_fullscreen(void)	/* met curarea */
+void area_fullscreen(void)	/* with curarea */
 {
-	/* deze funktie toggelt: als area full is wordt de parent weer zichtbaar */
+	/* this function toggles: if area is full then the parent will be restored */
 	bScreen *sc, *oldscreen;
 	ScrArea *newa, *old;
 	short headertype, fulltype;
 	
 	if(curarea->full) {
-		sc= curarea->full;	/* de oude screen */
+		sc= curarea->full;	/* the old screen */
 		fulltype = sc->full;
 
 		// refuse to go out of SCREENAUTOPLAY as long as G_FLAGS_AUTOPLAY
@@ -1798,7 +1845,7 @@ void area_fullscreen(void)	/* met curarea */
 		if (fulltype != SCREENAUTOPLAY || (G.flags & G_FLAGS_AUTOPLAY) == 0) {
 			sc->full= 0;
 		
-			/* vind oude area */
+			/* find old area */
 			old= sc->areabase.first;
 			while(old) {
 				if(old->full) break;
@@ -1829,7 +1876,7 @@ void area_fullscreen(void)	/* met curarea */
 		
 	}
 	else {
-		/* is er maar 1 area? */
+		/* is there only 1 area? */
 		if(G.curscreen->areabase.first==G.curscreen->areabase.last) return;
 		if(curarea->spacetype==SPACE_INFO) return;
 		
@@ -1837,16 +1884,16 @@ void area_fullscreen(void)	/* met curarea */
 		
 		old= curarea;		
 		oldscreen= G.curscreen;
-		sc= addscreen("temp");		/* deze zet G.curscreen */
+		sc= addscreen("temp");		/* this sets G.curscreen */
 
 		splitarea( (ScrArea *)sc->areabase.first, 'h', 0.99);
 		newa= sc->areabase.first;
 		newspace(newa->next, SPACE_INFO);
 		
 		curarea= old;
-		G.curscreen= oldscreen;	/* moet voor setscreen */
+		G.curscreen= oldscreen;	/* needed because of setscreen */
 		
-		/* area kopieeren */
+		/* vopy area */
 		copy_areadata(newa, curarea);
 		
 		curarea->full= oldscreen;
@@ -1882,12 +1929,12 @@ static void area_autoplayscreen(void)
 			
 			old= curarea;		
 			oldscreen= G.curscreen;
-			sc= addscreen("temp");		/* deze zet G.curscreen */
+			sc= addscreen("temp");		/* this sets G.curscreen */
 	
 			newa= sc->areabase.first;
 			
 			curarea= old;
-			G.curscreen= oldscreen;	/* moet voor setscreen */
+			G.curscreen= oldscreen;	/* because of setscreen */
 			
 			/* copy area settings */
 			copy_areadata(newa, curarea);
@@ -1908,10 +1955,10 @@ static void copy_screen(bScreen *to, bScreen *from)
 	ScrEdge *se;
 	ScrArea *sa;
 	ListBase lbase;
-	
-	/* alles van to vrijgeven */
+
+	/* free 'to' */
 	free_screen(to);
-	winqueue_break= 1;	/* overal uit queue's gaan */
+	winqueue_break= 1;	/* leave queues everywhere */
 	
 	duplicatelist(&to->vertbase, &from->vertbase);
 	duplicatelist(&to->edgebase, &from->edgebase);
@@ -1949,7 +1996,7 @@ static void copy_screen(bScreen *to, bScreen *from)
 		sa= sa->next;
 	}
 	
-	/* op nul zetten (nodig?) */
+	/* put at zero (needed?) */
 	s1= from->vertbase.first;
 	while(s1) {
 		s1->newv= 0;
@@ -1963,10 +2010,10 @@ void duplicate_screen(void)
 	
 	if(G.curscreen->full != SCREENNORMAL) return;
 	
-	/* nieuw screen maken: */
+	/* make new screen: */
 
 	oldscreen= G.curscreen;
-	sc= addscreen(oldscreen->id.name+2);	/* deze zet G.curscreen */
+	sc= addscreen(oldscreen->id.name+2);	/* this sets G.curscreen */
 	copy_screen(sc, oldscreen);
 
 	G.curscreen= oldscreen;
@@ -1975,7 +2022,7 @@ void duplicate_screen(void)
 }
 
 
-/* ************ END SCREENBEHEER ************** */
+/* ************ END SCREEN MANAGEMENT ************** */
 /* ************  JOIN/SPLIT/MOVE ************** */
 
 static void joinarea(ScrArea *sa, ScrEdge *onedge)
@@ -1988,8 +2035,8 @@ static void joinarea(ScrArea *sa, ScrEdge *onedge)
 	sa= test_edge_area(sa, onedge);
 	if (sa==0) return;
 
-	/* welke edges kunnen ermee? */
-	/* vind richtingen met zelfde edge */
+	/* which edges? */
+	/* find directions with same edge */
 	sa2= G.curscreen->areabase.first;
 	while(sa2) {
 		if(sa2 != sa) {
@@ -2023,7 +2070,7 @@ static void joinarea(ScrArea *sa, ScrEdge *onedge)
 	
 	
 	if(sa2) {
-		/* nieuwe area is oude sa */
+		/* new area is old sa */
 		if(sa2==left) {
 			sa->v1= sa2->v1;
 			sa->v2= sa2->v2;
@@ -2049,7 +2096,7 @@ static void joinarea(ScrArea *sa, ScrEdge *onedge)
 			screen_addedge(G.curscreen, sa->v3, sa->v4);
 		}
 	
-		/* edge en area weg */
+		/* remove edge and area */
 		/* remlink(&G.curscreen->edgebase, setest); */
 		/* freeN(setest); */
 		del_area(sa2);
@@ -2058,7 +2105,7 @@ static void joinarea(ScrArea *sa, ScrEdge *onedge)
 		
 		removedouble_scredges();
 		removenotused_scredges();
-		removenotused_scrverts();	/* moet als laatste */
+		removenotused_scrverts();	/* as last */
 		
 		testareas();
 		mainqenter(DRAWEDGES, 1);
@@ -2068,16 +2115,16 @@ static void joinarea(ScrArea *sa, ScrEdge *onedge)
 }
 
 static short testsplitpoint(ScrArea *sa, char dir, float fac)
-/* return 0: geen split mogelijk */
-/* else return (integer) screencoordinaat splitpunt */
+/* return 0: no split possible */
+/* else return (integer) screencoordinate split point */
 {
 	short x, y;
 	
-	/* area groot genoeg? */
+	/* area big enough? */
 	if(sa->v4->vec.x- sa->v1->vec.x <= 2*AREAMINX) return 0;
 	if(sa->v2->vec.y- sa->v1->vec.y <= 2*AREAMINY) return 0;
 
-	/* voor zekerheid */
+	/* to be sure */
 	if(fac<0.0) fac= 0.0;
 	if(fac>1.0) fac= 1.0;
 	
@@ -2123,54 +2170,54 @@ static void splitarea(ScrArea *sa, char dir, float fac)
 	areawinset(sa->win);
 	
 	if(dir=='h') {
-		/* nieuwe vertices */
+		/* new vertices */
 		sv1= screen_addvert(sc, sa->v1->vec.x, split);
 		sv2= screen_addvert(sc, sa->v4->vec.x, split);
 		
-		/* nieuwe edges */
+		/* new edges */
 		screen_addedge(sc, sa->v1, sv1);
 		screen_addedge(sc, sv1, sa->v2);
 		screen_addedge(sc, sa->v3, sv2);
 		screen_addedge(sc, sv2, sa->v4);
 		screen_addedge(sc, sv1, sv2);
 		
-		/* nieuwe areas: boven */
+		/* new areas: top */
 		newa= screen_addarea(sc, sv1, sa->v2, sa->v3, sv2, sa->headertype, sa->spacetype);
 		copy_areadata(newa, sa);
 
-		/* area onder */
+		/* area below */
 		sa->v2= sv1;
 		sa->v3= sv2;
 		
 	}
 	else {
-		/* nieuwe vertices */
+		/* new vertices */
 		sv1= screen_addvert(sc, split, sa->v1->vec.y);
 		sv2= screen_addvert(sc, split, sa->v2->vec.y);
 		
-		/* nieuwe edges */
+		/* new edges */
 		screen_addedge(sc, sa->v1, sv1);
 		screen_addedge(sc, sv1, sa->v4);
 		screen_addedge(sc, sa->v2, sv2);
 		screen_addedge(sc, sv2, sa->v3);
 		screen_addedge(sc, sv1, sv2);
 		
-		/* nieuwe areas: links */
+		/* new areas: left */
 		newa= screen_addarea(sc, sa->v1, sa->v2, sv2, sv1, sa->headertype, sa->spacetype);
 		copy_areadata(newa, sa);
 
-		/* area rechts */
+		/* area right */
 		sa->v1= sv1;		
 		sa->v2= sv2;
 	}
 	
-	/* dubbele vertices en edges verwijderen */
+	/* remove double vertices en edges */
 	removedouble_scrverts();
 	removedouble_scredges();
 	removenotused_scredges();
 	
 	mainqenter(DRAWEDGES, 1);
-	dodrawscreen= 1;		/* patch! even gets lost,,,? */
+	dodrawscreen= 1;		/* patch! event gets lost,,,? */
 	testareas();
 }
 
@@ -2192,7 +2239,7 @@ static void scrarea_draw_splitpoint(ScrArea *sa, char dir, float fac)
 static void splitarea_interactive(ScrArea *area, ScrEdge *onedge)
 {
 	ScrArea *sa= area;
-	float fac;
+	float fac= 0.0;
 	unsigned short event;
 	short ok= 0, val, split = 0, mval[2], mvalo[2], first= 1;
 	char dir;
@@ -2203,9 +2250,9 @@ static void splitarea_interactive(ScrArea *area, ScrEdge *onedge)
 	dir= scredge_is_horizontal(onedge)?'v':'h';
 	
 	mywinset(G.curscreen->mainwin);
-	/* hoort al goede matrix te hebben */
+	/* should already have a good matrix */
 
-	/* rekening houden met grid en minsize */
+	/* keep track of grid and minsize */
 	while(ok==0) {
 		getmouseco_sc(mval);
 		
@@ -2258,7 +2305,7 @@ static void splitarea_interactive(ScrArea *area, ScrEdge *onedge)
 	if(ok==1) {
 		splitarea(sa, dir, fac);
 		mainqenter(DRAWEDGES, 1);
-		dodrawscreen= 1;		/* patch! even gets lost,,,? */
+		dodrawscreen= 1;		/* patch! event gets lost,,,? */
 	}
 }
 
@@ -2276,7 +2323,7 @@ View3D *find_biggest_view3d(void)
 ScrArea *find_biggest_area_of_type(int spacecode)
 {
 	ScrArea *sa, *biggest= NULL;
-	int bigsize;
+	int bigsize= 0;
 	
 	for (sa= G.curscreen->areabase.first; sa; sa= sa->next) {
 		if (spacecode==0 || sa->spacetype==spacecode) {
@@ -2306,8 +2353,8 @@ static void select_connected_scredge(bScreen *sc, ScrEdge *edge)
 	int oneselected;
 	char dir;
 	
-	/* select connected, alleen in de juiste richting */
-	/* 'dir' is de richting van de EDGE */
+	/* select connected, only in the right direction */
+	/* 'dir' is the direction of EDGE */
 
 	if(edge->v1->vec.x==edge->v2->vec.x) dir= 'v';
 	else dir= 'h';
@@ -2342,8 +2389,8 @@ static void select_connected_scredge(bScreen *sc, ScrEdge *edge)
 }
 
 void test_scale_screen(bScreen *sc)
-/* testen of screenvertices vergroot/verkleind moeten worden */
-/* testen of offset nog klopt */
+/* test if screen vertices should be scaled */
+/* also check offset */
 {
 	ScrVert *sv=0;
 	ScrEdge *se;
@@ -2494,12 +2541,12 @@ static void moveareas(ScrEdge *edge)
 		}
 	}
 
-	/* nu zijn alle vertices met 'flag==1' degene die verplaatst kunnen worden. */
-	/* we lopen de areas af en testen vrije ruimte met MINSIZE */
+	/* now all verices with 'flag==1' are the ones that can be moved. */
+	/* we check all areas and test for free space with MINSIZE */
 	bigger= smaller= 10000;
 	sa= G.curscreen->areabase.first;
 	while(sa) {
-		if(dir=='h') {	/* als top of down edge select, test hoogte */
+		if(dir=='h') {	/* if top or down edge selected, test height */
 			if(sa->headertype) {
 				headery= HEADERY;
 				areaminy= AREAMINY;
@@ -2511,7 +2558,7 @@ static void moveareas(ScrEdge *edge)
 
 			if(sa->v1->flag && sa->v4->flag) {
 				int y1;
-				if(sa->v2->vec.y==G.curscreen->sizey-1)	/* bovenste edge */
+				if(sa->v2->vec.y==G.curscreen->sizey-1)	/* top edge */
 					y1= sa->v2->vec.y - sa->v1->vec.y-headery-EDGEWIDTH;
 				else 
 					y1= sa->v2->vec.y - sa->v1->vec.y-areaminy;
@@ -2519,14 +2566,14 @@ static void moveareas(ScrEdge *edge)
 			}
 			else if(sa->v2->flag && sa->v3->flag) {
 				int y1;
-				if(sa->v1->vec.y==0)	/* onderste edge */
+				if(sa->v1->vec.y==0)	/* bottom edge */
 					y1= sa->v2->vec.y - sa->v1->vec.y-headery-EDGEWIDTH;
 				else
 					y1= sa->v2->vec.y - sa->v1->vec.y-areaminy;
 				smaller= MIN2(smaller, y1);
 			}
 		}
-		else {	/* als left of right edge select, test breedte */
+		else {	/* if left or right edge selected, test width */
 			if(sa->v1->flag && sa->v2->flag) {
 				int x1= sa->v4->vec.x - sa->v1->vec.x-AREAMINX;
 				bigger= MIN2(bigger, x1);
@@ -2573,7 +2620,7 @@ static void moveareas(ScrEdge *edge)
 	if (doit==1) {
 		for (v1= G.curscreen->vertbase.first; v1; v1= v1->next) {
 			if (v1->flag) {
-					/* zo is AREAGRID netjes */
+					/* that way a nice AREAGRID  */
 				if((dir=='v') && v1->vec.x>0 && v1->vec.x<G.curscreen->sizex-1) {
 					v1->vec.x+= delta;
 					if(delta != bigger && delta != -smaller) v1->vec.x-= (v1->vec.x % AREAGRID);
@@ -2599,7 +2646,7 @@ static void moveareas(ScrEdge *edge)
 	
 	glDrawBuffer(GL_BACK);
 	mainqenter(DRAWEDGES, 1);
-	dodrawscreen= 1;		/* patch! even gets lost,,,? */
+	dodrawscreen= 1;		/* patch! event gets lost,,,? */
 }
 
 static void scrollheader(ScrArea *area)
@@ -2637,8 +2684,8 @@ static void scrollheader(ScrArea *area)
 
 int select_area(int spacetype)
 {
-	/* vanuit editroutines aanroepen, als er meer area's
-	 * zijn van type 'spacetype' kan er een area aangegeven worden
+	/* call from edit routines, when there are more areas
+	 * of type 'spacetype', you can indicate an area manually
 	 */
 	ScrArea *sa, *sact = NULL;
 	int tot=0;
@@ -2709,9 +2756,9 @@ void drawedge(short x1, short y1, short x2, short y2)
 	int a;
 	
 	if(x1==x2) {		/* vertical */
-		if (y2<y1)
-			y1^= y2^= y1^= y2;
-		
+		if (y2<y1) {
+			SWAP(short, y1, y2);
+		}
 		if (y1==0) y1-= EDGE_EXTEND;
 		if (y2==G.curscreen->sizey) y2+= EDGE_EXTEND;
 		
@@ -2729,9 +2776,9 @@ void drawedge(short x1, short y1, short x2, short y2)
 		glEnd();
 	}
 	else {				/* horizontal */
-		if (x2<x1)
-			x1^= x2^= x1^= x2;
-		
+		if (x2<x1) {
+			SWAP(short, x1, x2);
+		}
 		if (x1==0) x1-= EDGE_EXTEND;
 		if (x2==G.curscreen->sizex) x2+= EDGE_EXTEND;
 
@@ -2760,9 +2807,9 @@ static void drawscredge(ScrEdge *se)
 	v1= &(se->v1->vec);
 	v2= &(se->v2->vec);
 	
-	/* borders screen niet tekenen */
-	/* vanwege resolutie verschillen (PC/SGI files) de linit een
-	 * beetje afronden?
+	/* do not draw borders screen */
+	/* bcause of different framebuffer resoltions (PC/SGI etc files) 
+	 * a bit rounding here? should be looked at further...
 	 */
 	se->border= 1;
 	if(v1->x<=1 && v2->x<=1) return;
@@ -2779,7 +2826,7 @@ void drawscreen(void)
 	ScrEdge *se;
 	
 	mywinset(G.curscreen->mainwin);
-	myortho2(-0.5, (float)G.curscreen->sizex-0.5, -0.5, (float)G.curscreen->sizey-0.5);
+	myortho2(-0.5, (float)G.curscreen->sizex-0.5, -0.6, (float)G.curscreen->sizey-0.6);
 
 	/* two times, because there is no 'win_swap' for this  available */
 	glDrawBuffer(GL_FRONT);
@@ -2865,7 +2912,7 @@ int get_cursor(void) {
 }
 
 void set_cursor(int curs) {
-	if (!(R.flag & R_RENDERING)) {
+	if (!(R.flag & R_RENDERING) && G.background == 0) {
 		if (curs!=curcursor) {
 			curcursor= curs;
 			window_set_cursor(mainwin, curs);
