@@ -1,5 +1,5 @@
 /**
- * $Id: filesel.c,v 1.8 2003/02/09 12:47:43 phase Exp $
+ * $Id: filesel.c,v 1.17 2003/07/13 20:16:56 larstiq Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -149,7 +149,7 @@ struct _FSMenuEntry {
 	char *path;
 };
 
-static FSMenuEntry *fsmenu= 0, *lseperator= 0;
+static FSMenuEntry *fsmenu= 0;
 
 int fsmenu_get_nentries(void)
 {
@@ -187,9 +187,16 @@ char *fsmenu_build_menu(void)
 
 	for (fsme= fsmenu; fsme; fsme= fsme->next) {
 		if (!fsme->path) {
-				/* ignore consecutive or trailing seperators */
-			if (fsme->next && fsme->next->path)
-				BLI_dynstr_append(ds, "%l|");
+				/* clean consecutive seperators and ignore trailing ones */
+			if (fsme->next) {
+				if (fsme->next->path) {
+					BLI_dynstr_append(ds, "%l|");
+				} else {
+					FSMenuEntry *next= fsme->next;
+					fsme->next= next->next;
+					MEM_freeN(next);
+				}
+			}
 		} else {
 			BLI_dynstr_append(ds, fsme->path);
 			if (fsme->next) BLI_dynstr_append(ds, "|");
@@ -200,17 +207,21 @@ char *fsmenu_build_menu(void)
 	BLI_dynstr_free(ds);
 	return menustr;
 }
+static FSMenuEntry *fsmenu_get_last_separator(void) 
+{
+	FSMenuEntry *fsme, *lsep=NULL;
+
+	for (fsme= fsmenu; fsme; fsme= fsme->next)
+		if (!fsme->path)
+			lsep= fsme;
+
+	return lsep;
+}
 void fsmenu_insert_entry(char *path, int sorted)
 {
-	FSMenuEntry *fsme, *prev;
+	FSMenuEntry *prev= fsmenu_get_last_separator();
+	FSMenuEntry *fsme= prev?prev->next:fsmenu;
 
-	if (lseperator) {
-		prev= lseperator;
-		fsme= lseperator->next;
-	} else {
-		prev= NULL;
-		fsme= fsmenu;
-	}
 	for (; fsme; prev= fsme, fsme= fsme->next) {
 		if (fsme->path) {
 			if (BLI_streq(path, fsme->path)) {
@@ -239,11 +250,9 @@ void fsmenu_append_seperator(void)
 
 		while (fsme->next) fsme= fsme->next;
 
-		lseperator= MEM_mallocN(sizeof(*fsme), "fsme");
-		lseperator->next= NULL;
-		lseperator->path= NULL;
-
-		fsme->next= lseperator;
+		fsme->next= MEM_mallocN(sizeof(*fsme), "fsme");
+		fsme->next->next= NULL;
+		fsme->next->path= NULL;
 	}
 }
 void fsmenu_remove_entry(int idx)
@@ -285,7 +294,7 @@ static int compare_name(const void *a1, const void *a2)
 {
 	const struct direntry *entry1=a1, *entry2=a2;
 
-	/* type is gelijk aan stat.st_mode */
+	/* type is is equal to stat.st_mode */
 
 	if (S_ISDIR(entry1->type)){
 		if (S_ISDIR(entry2->type)==0) return (-1);
@@ -306,7 +315,7 @@ static int compare_date(const void *a1, const void *a2)
 {
 	const struct direntry *entry1=a1, *entry2=a2;
 	
-	/* type is gelijk aan stat.st_mode */
+	/* type is equal to stat.st_mode */
 
 	if (S_ISDIR(entry1->type)){
 		if (S_ISDIR(entry2->type)==0) return (-1);
@@ -334,7 +343,7 @@ static int compare_size(const void *a1, const void *a2)
 {
 	const struct direntry *entry1=a1, *entry2=a2;
 
-	/* type is gelijk aan stat.st_mode */
+	/* type is equal to stat.st_mode */
 
 	if (S_ISDIR(entry1->type)){
 		if (S_ISDIR(entry2->type)==0) return (-1);
@@ -387,7 +396,7 @@ void filesel_statistics(SpaceFile *sfile, int *totfile, int *selfile, float *tot
 	}
 }
 
-/* *************** HULPFUNKTIES ******************* */
+/* *************** HELP FUNCTIONS ******************* */
 
 /* This is a really ugly function... its purpose is to
  * take the space file name and clean it up, replacing
@@ -404,7 +413,7 @@ void checkdir(char *dir)
 	strcpy(dir, tmp);
 	
 #ifdef WIN32
-	if(dir[0]=='.') {	/* komt voor, o.a. bij FILE_MAIN */
+	if(dir[0]=='.') {	/* happens for example in FILE_MAIN */
 		dir[0]= '\\';
 		dir[1]= 0;
 		return;
@@ -430,7 +439,7 @@ void checkdir(char *dir)
 		strcpy(start,eind);
 	}
 
-	if(a = strlen(dir)){				/* eerst alle '\\' weghalen aan het eind */
+	if(a = strlen(dir)){				/* remove the '\\' at the end */
 		while(a>0 && dir[a-1] == '\\'){
 			a--;
 			dir[a] = 0;
@@ -439,7 +448,7 @@ void checkdir(char *dir)
 
 	strcat(dir, "\\");
 #else	
-	if(dir[0]=='.') {	/* komt voor, o.a. bij FILE_MAIN */
+	if(dir[0]=='.') {	/* happens, for example in FILE_MAIN */
 		dir[0]= '/';
 		dir[1]= 0;
 		return;
@@ -465,7 +474,7 @@ void checkdir(char *dir)
 		strcpy(start,eind);
 	}
 
-	if( (a = strlen(dir)) ){				/* eerst alle '/' weghalen aan het eind */
+	if( (a = strlen(dir)) ){				/* remove all '/' at the end */
 		while(dir[a-1] == '/'){
 			a--;
 			dir[a] = 0;
@@ -486,7 +495,7 @@ void test_flags_file(SpaceFile *sfile)
 	
 	for(num=0; num<sfile->totfile; num++, file++) {
 		file->flags= 0;
-		file->type= file->s.st_mode;	/* restore het geknoei van hieronder */ 
+		file->type= file->s.st_mode;	/* restore the mess below */ 
 		
 			/* Don't check extensions for directories */ 
 		if (file->type&S_IFDIR)
@@ -504,22 +513,91 @@ void test_flags_file(SpaceFile *sfile)
 				file->flags |= PSXFILE;
 			}
 		} else if (sfile->type==FILE_SPECIAL){
-			if(		BLI_testextensie(file->relname, ".jpg") ||
-					BLI_testextensie(file->relname, ".tga") ||
-					BLI_testextensie(file->relname, ".rgb") ||
-					BLI_testextensie(file->relname, ".png") ||
-/*					BLI_testextensie(file->relname, ".bmp") || */
-					BLI_testextensie(file->relname, ".iff") ||
-					BLI_testextensie(file->relname, ".lbm") ||
-					BLI_testextensie(file->relname, ".sgi")) {
-				file->flags |= IMAGEFILE;			
-			}
-			else if(BLI_testextensie(file->relname, ".avi") ||
-					BLI_testextensie(file->relname, ".mv")) {
-				file->flags |= MOVIEFILE;			
-			}
-			else if(BLI_testextensie(file->relname, ".py")) {
+			if(BLI_testextensie(file->relname, ".py")) {
 				file->flags |= PYSCRIPTFILE;			
+			} else if( BLI_testextensie(file->relname, ".ttf")
+					|| BLI_testextensie(file->relname, ".ttc")
+					|| BLI_testextensie(file->relname, ".pfb")
+					|| BLI_testextensie(file->relname, ".otf")
+					|| BLI_testextensie(file->relname, ".otc")) {
+				file->flags |= FTFONTFILE;			
+			} else if (G.have_quicktime){
+				if(		BLI_testextensie(file->relname, ".jpg")
+					||	BLI_testextensie(file->relname, ".jpeg")
+					||	BLI_testextensie(file->relname, ".tga")
+					||	BLI_testextensie(file->relname, ".rgb")
+					||	BLI_testextensie(file->relname, ".bmp")
+					||	BLI_testextensie(file->relname, ".png")
+					||	BLI_testextensie(file->relname, ".iff")
+					||	BLI_testextensie(file->relname, ".lbm")
+					||	BLI_testextensie(file->relname, ".gif")
+					||	BLI_testextensie(file->relname, ".psd")
+					||	BLI_testextensie(file->relname, ".tif")
+					||	BLI_testextensie(file->relname, ".tiff")
+					||	BLI_testextensie(file->relname, ".pct")
+					||	BLI_testextensie(file->relname, ".pict")
+					||	BLI_testextensie(file->relname, ".pntg") //macpaint
+					||	BLI_testextensie(file->relname, ".qtif")
+#ifdef WITH_FREEIMAGE
+				||	BLI_testextensie(file->relname, ".jng")
+				||	BLI_testextensie(file->relname, ".mng")
+				||	BLI_testextensie(file->relname, ".pbm")
+				||	BLI_testextensie(file->relname, ".pgm")
+				||	BLI_testextensie(file->relname, ".ppm")
+				||	BLI_testextensie(file->relname, ".wbmp")
+				||	BLI_testextensie(file->relname, ".cut")
+				||	BLI_testextensie(file->relname, ".ico")
+				||	BLI_testextensie(file->relname, ".koala")
+				||	BLI_testextensie(file->relname, ".pcd")
+				||	BLI_testextensie(file->relname, ".pcx")
+				||	BLI_testextensie(file->relname, ".ras")
+#endif
+					||	BLI_testextensie(file->relname, ".sgi")) {
+					file->flags |= IMAGEFILE;			
+				}
+				else if(BLI_testextensie(file->relname, ".avi")
+					||	BLI_testextensie(file->relname, ".flc")
+					||	BLI_testextensie(file->relname, ".mov")
+					||	BLI_testextensie(file->relname, ".movie")
+					||	BLI_testextensie(file->relname, ".mv")) {
+					file->flags |= MOVIEFILE;			
+				}
+			} else { // no quicktime
+				if(BLI_testextensie(file->relname, ".jpg")
+					||	BLI_testextensie(file->relname, ".tga")
+					||	BLI_testextensie(file->relname, ".rgb")
+					||	BLI_testextensie(file->relname, ".bmp")
+					||	BLI_testextensie(file->relname, ".png")
+					||	BLI_testextensie(file->relname, ".iff")
+					||	BLI_testextensie(file->relname, ".lbm")
+#ifdef WITH_FREEIMAGE
+				||	BLI_testextensie(file->relname, ".jng")
+				||	BLI_testextensie(file->relname, ".mng")
+				||	BLI_testextensie(file->relname, ".pbm")
+				||	BLI_testextensie(file->relname, ".pgm")
+				||	BLI_testextensie(file->relname, ".ppm")
+				||	BLI_testextensie(file->relname, ".wbmp")
+				||	BLI_testextensie(file->relname, ".cut")
+				||	BLI_testextensie(file->relname, ".ico")
+				||	BLI_testextensie(file->relname, ".koala")
+				||	BLI_testextensie(file->relname, ".pcd")
+				||	BLI_testextensie(file->relname, ".pcx")
+				||	BLI_testextensie(file->relname, ".ras")
+				||	BLI_testextensie(file->relname, ".gif")
+				||	BLI_testextensie(file->relname, ".psd")
+				||	BLI_testextensie(file->relname, ".tif")
+				||	BLI_testextensie(file->relname, ".tiff")
+#endif
+					||	BLI_testextensie(file->relname, ".sgi")) {
+					file->flags |= IMAGEFILE;			
+				}
+				else if(BLI_testextensie(file->relname, ".avi")
+					||	BLI_testextensie(file->relname, ".mv")) {
+					file->flags |= MOVIEFILE;			
+				}
+				else if(BLI_testextensie(file->relname, ".wav")) {
+					file->flags |= SOUNDFILE;
+				}				
 			}
 		}
 	}	
@@ -560,7 +638,7 @@ void read_dir(SpaceFile *sfile)
 	int num, len;
 	char wdir[FILE_MAXDIR];
 
-	/* sfile->act wordt gebruikt o.a. bij databrowse: dubbele namen van library objecten */
+	/* sfile->act is used for example in databrowse: double names of library objects */
 	sfile->act= -1;
 
 	if(sfile->type==FILE_MAIN) {
@@ -641,20 +719,20 @@ void parent(SpaceFile *sfile)
 	short a;
 	char *dir;
 	
-	/* als databrowse: geen parent */
+	/* if databrowse: no parent */
 	if(sfile->type==FILE_MAIN && sfile->returnfunc) return;
 
 	dir= sfile->dir;
 	
 #ifdef WIN32
-	if(a = strlen(dir)) {				/* eerst alle '/' weghalen aan het eind */
+	if(a = strlen(dir)) {				/* remove all '/' at the end */
 		while(dir[a-1] == '\\') {
 			a--;
 			dir[a] = 0;
 			if (a<=0) break;
 		}
 	}
-	if(a = strlen(dir)) {				/* daarna alles weghalen tot aan '/' */
+	if(a = strlen(dir)) {				/* then remove all until '/' */
 		while(dir[a-1] != '\\') {
 			a--;
 			dir[a] = 0;
@@ -666,14 +744,14 @@ void parent(SpaceFile *sfile)
 	}
 	else if(sfile->type!=FILE_MAIN) strcpy(dir,"\\");
 #else
-	if( (a = strlen(dir)) ) {				/* eerst alle '/' weghalen aan het eind */
+	if( (a = strlen(dir)) ) {				/* remove all '/' at the end */
 		while(dir[a-1] == '/') {
 			a--;
 			dir[a] = 0;
 			if (a<=0) break;
 		}
 	}
-	if( (a = strlen(dir)) ) {				/* daarna alles weghalen tot aan '/' */
+	if( (a = strlen(dir)) ) {				/* then remove until '/' */
 		while(dir[a-1] != '/') {
 			a--;
 			dir[a] = 0;
@@ -756,7 +834,7 @@ static void calc_file_rcts(SpaceFile *sfile)
 	filebuty2= filebuty1+FILESELHEAD/2 -6;
 	
 	
-	/* aantal kolommen */
+	/* amount of collums */
 	len= sfile->maxnamelen+25;
 	
 	if(sfile->type==FILE_MAIN) len+= 100;
@@ -868,6 +946,14 @@ static void printregel(SpaceFile *sfile, struct direntry *files, int x, int y)
 		cpack(0x4477dd);
 		glRects(x-14,  y,  x-8,  y+7);
 	}
+	else if(files->flags & SOUNDFILE) {
+		cpack(0xa0a000);
+		glRects(x-14,  y,  x-8,  y+7);
+	}	
+	else if(files->flags & FTFONTFILE) {
+		cpack(0xff2371);
+		glRects(x-14,  y,  x-8,  y+7);
+	}
 	
 	if(S_ISDIR(files->type)) cpack(0xFFFFFF);
 	else cpack(0x0);
@@ -921,12 +1007,12 @@ static void printregel(SpaceFile *sfile, struct direntry *files, int x, int y)
 
 static int calc_filesel_regel(SpaceFile *sfile, int nr, int *valx, int *valy)
 {
-	/* sco van de de regel */
+	/* get screen coordinate of a 'regel', dutch for line */
 	int val, coll;
 
 	nr-= sfile->ofs;
 
-	/* aantal regels in de hoogte */
+	/* amount of lines */
 	val= (textrct.ymax-textrct.ymin)/FILESEL_DY;
 	coll= nr/val;
 	nr -= coll*val;
@@ -1011,11 +1097,11 @@ static void draw_filetext(SpaceFile *sfile)
 	if(textrct.ymin+10 >= textrct.ymax) return;
 
 
-	/* kader */
+	/* box */
 	cpack(0x717171);
 	glRecti(textrct.xmin,  textrct.ymin,  textrct.xmax,  textrct.ymax);
 
-	/* kolommen */
+	/* collums */
 	x= textrct.xmin+collumwidth;
 	for(a=1; a<sfile->collums; a++, x+= collumwidth) {
 		cpack(0x303030);
@@ -1026,7 +1112,7 @@ static void draw_filetext(SpaceFile *sfile)
 
 	if(sfile->filelist==0) return;
 	
-	/* test: als muis niet in area staat: de HILITE wissen */
+	/* test: if mouse is not in area: clear HILITE */
 	getmouseco_areawin(mval);
 
 	if(mval[0]<0 || mval[0]>curarea->winx) {
@@ -1042,14 +1128,14 @@ static void draw_filetext(SpaceFile *sfile)
 		printregel(sfile, files, x, y);
 	}
 
-	/* wissen tekenfoutjes, tekst teveel aan de rechterkant: */
+	/* clear drawing errors, with text at the right hand side: */
 	uiEmboss(textrct.xmin, textrct.ymin, textrct.xmax, textrct.ymax, 1);
 	
 	glColor3f(.5625, .5625, .5625);
 	glRecti(textrct.xmax+2,  textrct.ymin,  textrct.xmax+10,  textrct.ymax);
 }
 
-void drawfilespace()
+void drawfilespace(ScrArea *sa, void *spacedata)
 {
 	SpaceFile *sfile;
 	uiBlock *block;
@@ -1069,7 +1155,7 @@ void drawfilespace()
 		
 		calc_file_rcts(sfile);
 		
-		/* act berekenen */ 
+		/* calculate act */ 
 		getmouseco_areawin(mval);
 		act= find_active_file(sfile, mval[0], mval[1]);
 		if(act>=0 && act<sfile->totfile)
@@ -1099,18 +1185,20 @@ void drawfilespace()
 	}
 
 	menu= fsmenu_build_menu();
-	uiDefButS(block, MENU,	3, menu, scrollrct.xmin, filebuty1, scrollrct.xmax-scrollrct.xmin, 21, &sfile->menu, 0, 0, 0, 0, "");
+	if(menu[0])	// happens when no .Bfs is there, and first time browse
+		uiDefButS(block, MENU,	3, menu, scrollrct.xmin, filebuty1, scrollrct.xmax-scrollrct.xmin, 21, &sfile->menu, 0, 0, 0, 0, "");
 	MEM_freeN(menu);
 
 	uiDefBut(block, BUT,		4, "P", scrollrct.xmin, filebuty2, scrollrct.xmax-scrollrct.xmin, 21, 0, 0, 0, 0, 0, "Move to the parent directory (PKEY)");
 
+	uiDrawBlock(block);
+
 	draw_filescroll(sfile);
 	draw_filetext(sfile);
 	
-	/* andere diskfree etc ? */
+	/* others diskfree etc ? */
 	scrarea_queue_headredraw(curarea);	
 	
-	uiDrawBlock(block);
 	
 	curarea->win_swap= WIN_BACK_OK;
 }
@@ -1122,7 +1210,7 @@ static void do_filescroll(SpaceFile *sfile)
 	calc_file_rcts(sfile);
 	
 	filescrollselect= 1;
-	/* voor mooiigheid */
+	/* for beauty */
 
 	glDrawBuffer(GL_FRONT);
 	draw_filescroll(sfile);
@@ -1156,7 +1244,7 @@ static void do_filescroll(SpaceFile *sfile)
 	}
 	filescrollselect= 0;
 
-	/* voor mooiigheid */
+	/* for beauty */
 	glDrawBuffer(GL_FRONT);
 	draw_filescroll(sfile);
 	glDrawBuffer(GL_BACK);
@@ -1195,7 +1283,7 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 	newspace(curarea, SPACE_FILE);
 	scrarea_queue_winredraw(curarea);
 	
-	/* is misschien dubbelop, voor geval area al SPACE_FILE is met andere filename */
+	/* sometime double, when area already is SPACE_FILE with a different file name */
 	addqueue(curarea->headwin, CHANGED, 1);
 	
 
@@ -1207,7 +1295,7 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 	sfile->returnfunc= func;
 	sfile->type= type;
 	sfile->ofs= 0;
-	/* sfile->act wordt gebruikt bij databrowse: dubbele namen van library objecten */
+	/* sfile->act is used for databrowse: double names of library objects */
 	sfile->act= -1;
 	
 	if(BLI_convertstringcode(name, G.sce, G.scene->r.cfra)) sfile->flag |= FILE_STRINGCODE;
@@ -1224,7 +1312,7 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 			strcat(sfile->dir, "/");
 		}
 
-		/* alles vrijgeven */
+		/* free all */
 		if(sfile->libfiledata) BLO_blendhandle_close(sfile->libfiledata);
 		sfile->libfiledata= 0;
 		
@@ -1233,7 +1321,7 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 	else if(type==FILE_LOADLIB) {
 		strcpy(sfile->dir, name);
 		if( is_a_library(sfile, temp, group) ) {
-			/* dit geval is om een reload van library-filelist te veroorzaken */
+			/* to force a reload of the library-filelist */
 			if(sfile->libfiledata==0) {
 				freefilelist(sfile);
 			}
@@ -1245,9 +1333,9 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 		}
 	}
 	else {	/* FILE_BLENDER */
-		split_sfile(sfile, name);	/* test ook de filelist */
+		split_sfile(sfile, name);	/* test filelist too */
 		
-		/* vrijgeven: filelist en libfiledata kloppen niet meer */
+		/* free: filelist and libfiledata became incorrect */
 		if(sfile->libfiledata) BLO_blendhandle_close(sfile->libfiledata);
 		sfile->libfiledata= 0;
 	}
@@ -1265,7 +1353,7 @@ void activate_imageselect(int type, char *title, char *file, void (*func)(char *
 	
 	newspace(curarea, SPACE_IMASEL);
 	
-	/* is misschien dubbelop, voor geval area al SPACE_FILE is met andere filename */
+	/* sometimes double, when area is already SPACE_FILE with a different file name */
 	addqueue(curarea->headwin, CHANGED, 1);
 	addqueue(curarea->win, CHANGED, 1);
 
@@ -1298,7 +1386,7 @@ void activate_databrowse(ID *id, int idcode, int fromcode, int retval, short *me
 	
 	if(id==0) {
 		lb= wich_libbase(G.main, idcode);
-		id= lb->first;
+		id= lb->last;
 	}
 	
 	if(id) strcpy(str, id->name);
@@ -1350,7 +1438,7 @@ static int getotherdir(void)
 		if(sa!=curarea) {
 			if(sa->spacetype==SPACE_FILE) {
 				
-				/* al een gevonden */
+				/* already found one */
 				if(sfile) return 0;
 		
 				sfile= sa->spacedata.first;
@@ -1372,7 +1460,7 @@ static void reread_other_fs(void)
 {
 	SpaceFile *sfile;
 	
-	/* oppassen: alleen aanroepen als getotherdir goed is afgelopen */
+	/* watch it: only call when getotherdir returned OK */
 	
 	sfile= otherarea->spacedata.first;
 	freefilelist(sfile);
@@ -1382,7 +1470,7 @@ static void reread_other_fs(void)
 
 void free_filesel_spec(char *dir)
 {
-	/* alle filesels met 'dir' worden vrijgegeven */
+	/* all filesels with 'dir' are freed */
 	bScreen *sc;
 		
 	sc= G.main->screen.first;
@@ -1499,7 +1587,7 @@ static void do_filesel_buttons(short event, SpaceFile *sfile)
 	else if(event== 3) {
 		char *selected= fsmenu_get_entry(sfile->menu-1);
 		
-		/* welke string */
+		/* which string */
 		if (selected) {
 			strcpy(sfile->dir, selected);
 			BLI_make_exist(sfile->dir);
@@ -1684,7 +1772,7 @@ static void fs_fake_users(SpaceFile *sfile)
 	ID *id;
 	int a;
 	
-	/* alleen bij F4 DATABROWSE */
+	/* only for F4 DATABROWSE */
 	if(sfile->returnfunc) return;
 	
 	for(a=0; a<sfile->totfile; a++) {
@@ -1706,8 +1794,25 @@ static void fs_fake_users(SpaceFile *sfile)
 	scrarea_queue_winredraw(curarea);
 }
 
-void winqreadfilespace(unsigned short event, short val, char ascii)
+
+static int get_hilited_entry(SpaceFile *sfile)
 {
+	int a, count=0;
+
+	for(a=0; a<sfile->totfile; a++) {
+		if(sfile->filelist[a].flags & HILITE) {
+			return a;
+		}
+	}
+	return -1;
+}
+
+
+void winqreadfilespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
+{
+	unsigned short event= evt->event;
+	short val= evt->val;
+	char ascii= evt->ascii;
 	static int acto=0;
 	SpaceFile *sfile;
 	int act, do_draw= 0, i, test, ret = 0;
@@ -1717,7 +1822,7 @@ void winqreadfilespace(unsigned short event, short val, char ascii)
 	sfile= curarea->spacedata.first;
 	if(sfile==0) return;
 	if(sfile->filelist==0) {
-		/* wel buttons doen */
+		/* but do buttons */
 		if(val && event==LEFTMOUSE) {
 			/* FrontbufferButs(TRUE); */
 			/* event= DoButtons(); */
@@ -1732,7 +1837,7 @@ void winqreadfilespace(unsigned short event, short val, char ascii)
 	calc_file_rcts(sfile);
 	getmouseco_areawin(mval);
 
-	/* om hangen te voorkomen */
+	/* prevent looping */
 	if(selecting && !(get_mbut() & R_MOUSE)) selecting= 0;
 
 	if(val) {
@@ -1766,7 +1871,7 @@ void winqreadfilespace(unsigned short event, short val, char ascii)
 			}
 			else if(mval[0]>textrct.xmin && mval[0]<textrct.xmax && mval[1]>textrct.ymin && mval[1]<textrct.ymax) {
 				
-				/* sfile->act wordt gebruikt bij databrowse: dubbelenamen van library objecten */
+				/* sfile->act is used in databrowse: double names of library objects */
 				
 				sfile->act= act= find_active_file(sfile, mval[0], mval[1]);
 				
@@ -1792,7 +1897,7 @@ void winqreadfilespace(unsigned short event, short val, char ascii)
 				/* FrontbufferButs(TRUE); */
 				/* event= DoButtons(); */
 				/* FrontbufferButs(FALSE); */
-					/*  NIET de headerbuttons! */
+					/*  NOT the headerbuttons! */
 				/* if(event) do_filesel_buttons(event, sfile);	 */
 			}
 			break;
@@ -1866,7 +1971,7 @@ void winqreadfilespace(unsigned short event, short val, char ascii)
 				databrowse_replace(sfile, groupname_to_code(sfile->dir));
 				break;
 			}
-			/* doorgeven */
+			/* pass */
 		case MKEY:
 			if(sfile->type==FILE_MAIN) break;
 
@@ -1912,12 +2017,31 @@ void winqreadfilespace(unsigned short event, short val, char ascii)
 			reread_other_fs();
 			
 			break;
+
+		case XKEY:
+			test = get_hilited_entry(sfile);
+
+			if (test != -1 && !(S_ISDIR(sfile->filelist[test].type))){
+				BLI_make_file_string(G.sce, str, sfile->dir, sfile->filelist[test].relname);
+
+				if( okee("Remove %s", str) ) {
+					ret = BLI_delete(str, 0, 0);
+					if (ret) {
+						error("Command failed, see console");
+					} else {
+						freefilelist(sfile);
+						do_draw= 1;
+					}
+				}
+			}
+			break;
+
 		case RKEY:
 			if(sfile->type==FILE_MAIN) {
 				databrowse_replace(sfile, groupname_to_code(sfile->dir));
 				break;
 			}
-			/* doorgeven aan TKEY! */
+			/* pass to TKEY! */
 			
 		case TKEY:
 			if(sfile->type==FILE_MAIN) break;
@@ -1930,7 +2054,7 @@ void winqreadfilespace(unsigned short event, short val, char ascii)
 			if(event==TKEY) sprintf(str, "Touch");
 			else if(event==RKEY) sprintf(str, "Remove from %s", sfile->dir);
 			
-			qual= G.qual;	/* want na okee() heb je de shift losgelaten */
+			qual= G.qual;	/* because after okee() you released the SHIFT */
 			if (!okee(str)) break;
 			
 			for (i = 0; i <sfile->totfile; i++) {
@@ -2063,8 +2187,8 @@ static int groupname_to_code(char *group)
 
 static int is_a_library(SpaceFile *sfile, char *dir, char *group)
 {
-	/* return ok als een blenderfile, in dir staat de filename,
-	 * in group het type libdata
+	/* return ok when a blenderfile, in dir is the filename,
+	 * in group the type of libdata
 	 */
 	int len;
 	char *fd;
@@ -2113,7 +2237,7 @@ static void do_library_append(SpaceFile *sfile)
 		
 		BLO_library_append(sfile, dir, idcode);
 
-		/* DISPLISTEN */
+		/* DISPLISTS */
 		ob= G.main->object.first;
 		set_displist_onlyzero(1);
 		while(ob) {
@@ -2132,7 +2256,7 @@ static void do_library_append(SpaceFile *sfile)
 		}
 		set_displist_onlyzero(0);
 	
-		/* in sfile->dir staat de HELE libnaam */
+		/* in sfile->dir is the whole lib name */
 		strcpy(G.lib, sfile->dir);
 		
 		if((sfile->flag & FILE_LINK)==0) all_local();
@@ -2145,17 +2269,17 @@ static void library_to_filelist(SpaceFile *sfile)
 	int ok, i, nnames, idcode;
 	LinkNode *l, *names;
 	
-	/* name testen */
+	/* name test */
 	ok= is_a_library(sfile, dir, group);
 	if (!ok) {
-		/* vrijgeven */
+		/* free */
 		if(sfile->libfiledata) BLO_blendhandle_close(sfile->libfiledata);
 		sfile->libfiledata= 0;
 		return;
 	}
 	
-	/* en daar gaat ie */
-	/* voorlopig alleen filedata inlezen als libfiledata==0 */
+	/* there we go */
+	/* for the time being only read filedata when libfiledata==0 */
 	if (sfile->libfiledata==0) {
 		sfile->libfiledata= BLO_blendhandle_from_file(dir);
 		if(sfile->libfiledata==0) return;
@@ -2207,7 +2331,7 @@ static void filesel_select_objects(SpaceFile *sfile)
 	Scene *sce;
 	int a;
 	
-	/* alleen bij F4 DATABROWSE */
+	/* only when F4 DATABROWSE */
 	if(sfile->returnfunc) return;
 	
 	if( strcmp(sfile->dir, "Object/")==0 ) {
@@ -2247,7 +2371,7 @@ static void active_file_object(SpaceFile *sfile)
 {
 	Object *ob;
 	
-	/* alleen bij F4 DATABROWSE */
+	/* only when F4 DATABROWSE */
 	if(sfile->returnfunc) return;
 	
 	if( strcmp(sfile->dir, "Object/")==0 ) {
@@ -2285,7 +2409,7 @@ void main_to_filelist(SpaceFile *sfile)
 	
 	if( sfile->dir[0]==0) {
 		
-		/* directories maken */
+		/* make directories */
 		sfile->totfile= 22;
 		sfile->filelist= (struct direntry *)malloc(sfile->totfile * sizeof(struct direntry));
 		
@@ -2320,7 +2444,7 @@ void main_to_filelist(SpaceFile *sfile)
 	}
 	else {
 
-		/* files maken */
+		/* make files */
 		idcode= groupname_to_code(sfile->dir);
 		
 		lb= wich_libbase(G.main, idcode );
@@ -2399,7 +2523,7 @@ void main_to_filelist(SpaceFile *sfile)
 			id= id->next;
 		}
 		
-		/* alleen qsort van libraryblokken */
+		/* only qsort of libraryblokken */
 		if(totlib>1) {
 			qsort(firstlib, totlib, sizeof(struct direntry), compare_name);
 		}
@@ -2421,4 +2545,38 @@ void main_to_filelist(SpaceFile *sfile)
 }
 
 
+void clever_numbuts_filesel()
+{
+	SpaceFile *sfile;
+	char orgname[FILE_MAXDIR+FILE_MAXFILE+12];
+	char filename[FILE_MAXDIR+FILE_MAXFILE+12];
+	char newname[FILE_MAXDIR+FILE_MAXFILE+12];
+	int test;
+	int len;
+	
+	sfile= curarea->spacedata.first;
+
+	if(sfile->type==FILE_MAIN) return;
+	
+	len = 110;
+	test = get_hilited_entry(sfile);
+
+	if (test != -1 && !(S_ISDIR(sfile->filelist[test].type))){
+		BLI_make_file_string(G.sce, orgname, sfile->dir, sfile->filelist[test].relname);
+		strcpy(filename, sfile->filelist[test].relname);
+		
+		add_numbut(0, TEX, "", 0, len, filename, "Rename File");
+
+		if( do_clever_numbuts("Rename File", 1, REDRAW) ) {
+			BLI_make_file_string(G.sce, newname, sfile->dir, filename);
+
+			if( strcmp(orgname, newname) != 0 ) {
+				BLI_rename(orgname, newname);
+				freefilelist(sfile);
+			}
+		}
+
+		scrarea_queue_winredraw(curarea);
+	}
+}
 
