@@ -1,5 +1,5 @@
 /**
- * $Id: mywindow.c,v 1.3 2002/11/25 12:02:05 mein Exp $
+ * $Id: mywindow.c,v 1.6 2003/05/03 12:17:11 phase Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -28,9 +28,11 @@
  * Contributor(s): none yet.
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
- * vervanging voor een aantal fie's zoals swinopen, winset,  (zie onder)
- * dit alles omdat GL en X te traag zijn
- * feb: Opengl en toch maar naar X!
+ *
+ *
+ * for compatibility with old iris code, replacement of swinopen, winset, etc
+ * btw: subwindows in X are way too slow, tried it, and choose for my own system... (ton)
+ * 
  */
 
 #include <string.h>
@@ -63,13 +65,6 @@
 
 #include "winlay.h"
 
-
-typedef struct {
-	unsigned short event;
-	short val;
-	char ascii;
-} QEvent;
-
 typedef struct {
 	struct bWindow *next, *prev;
 	int id, pad;
@@ -102,7 +97,7 @@ void mywindow_init_mainwin(Window *win, int orx, int ory, int sizex, int sizey)
 	mainwindow.ymax= ory+sizey-1;
 	mainwindow.qevents= NULL;
 
-	myortho2(-0.5, (float)sizex-0.5, -0.5, (float)sizey-0.5);
+	myortho2(-0.5, (float)sizex-0.5, -0.6, (float)sizey-0.6);
 	glLoadIdentity();
 		
 	glGetFloatv(GL_PROJECTION_MATRIX, (float *)mainwindow.winmat);
@@ -145,26 +140,18 @@ int bwin_qtest(int winid)
 {
 	return !BLI_gsqueue_is_empty(bwin_from_winid(winid)->qevents);
 }
-unsigned short bwin_qread(int winid, short *val_r, char *ascii_r)
+int bwin_qread(int winid, BWinEvent *evt_r)
 {
 	if (bwin_qtest(winid)) {
-		QEvent evt;
-		BLI_gsqueue_pop(bwin_from_winid(winid)->qevents, &evt);
-		*val_r= evt.val;
-		*ascii_r= evt.ascii;
-		return evt.event;
+		BLI_gsqueue_pop(bwin_from_winid(winid)->qevents, evt_r);
+		return 1;
 	} else {
-		*val_r= 0;
 		return 0;
 	}
 }
-void bwin_qadd(int winid, unsigned short event, short val, char ascii)
+void bwin_qadd(int winid, BWinEvent *evt)
 {
-	QEvent evt;
-	evt.event= event;
-	evt.val= val;
-	evt.ascii= ascii;
-	BLI_gsqueue_push(bwin_from_winid(winid)->qevents, &evt);
+	BLI_gsqueue_push(bwin_from_winid(winid)->qevents, evt);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -383,7 +370,7 @@ int myswinopen(int parentid, int xmin, int xmax, int ymin, int ymax)
 		win->xmax= xmax;
 		win->ymax= ymax;
 	
-		win->qevents= BLI_gsqueue_new(sizeof(QEvent));
+		win->qevents= BLI_gsqueue_new(sizeof(BWinEvent));
 
 		Mat4One(win->viewmat);
 		Mat4One(win->winmat);
@@ -418,7 +405,7 @@ void mywinclose(int winid)
 	if (curswin==winid) curswin= 0;
 }
 
-void mywinposition(int winid, int xmin, int xmax, int ymin, int ymax) /* let op: andere syntax */
+void mywinposition(int winid, int xmin, int xmax, int ymin, int ymax) /* watch: syntax differs from iris */
 {
 	bWindow *win= bwin_from_winid(winid);
 	
@@ -577,7 +564,7 @@ void myswapbuffers(void)
 }
 
 
-/* *********************** PATTERNS ENZO ***************** */
+/* *********************** PATTERNS ETC ***************** */
 
 void setlinestyle(int nr)
 {
@@ -623,7 +610,7 @@ void my_get_frontbuffer_image(int x, int y, int sx, int sy)
 	}
 
 	#ifdef WIN32
-	/* ander coordinatensysteem! */
+	/* different coord system! */
 	y= (G.curscreen->sizey-y);
 	
 	if(curswin>3) {
