@@ -2,7 +2,7 @@
  * 
  * maart 95
  * 
- * $Id: previewrender.c,v 1.3 2002/11/25 12:02:05 mein Exp $
+ * $Id: previewrender.c,v 1.5 2003/07/19 20:31:29 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -77,13 +77,11 @@
 #include "BIF_screen.h"
 #include "BIF_space.h"		/* allqueue */
 #include "BIF_drawimage.h"	/* rectwrite_part */
-//#include "BIF_previewrender.h"
 #include "BIF_mywindow.h"
+#include "PIL_time.h"
 
 #include "RE_renderconverter.h"
 
-//#include "mydevice.h"
-
 #define PR_RECTX	101
 #define PR_RECTY	101
 #define PR_XMIN		10
@@ -260,20 +258,27 @@ static void set_previewrect(int win, int xmin, int ymin, int xmax, int ymax)
 
 static void display_pr_scanline(unsigned int *rect, int recty)
 {
-	/* we display 3 new scanlines, one old */
+	static double lasttime= 0;
+	/* we display 3 new scanlines, one old, the overlap is for wacky 3d cards that cant handle zoom proper */
 
 	if(recty % 2) return;
 	if(recty<2) return;
 	
 	rect+= (recty-2)*PR_RECTX;
 
-	/* iets meer uitvergroten in y om GL/mesa bugje te verhelpen */
+	/* enlarge a bit in the y direction, to avoid GL/mesa bug */
 	glPixelZoom(pr_facx, pr_facy);
 
 	glRasterPos2f( (float)PR_XMIN+0.5, 1.0+(float)PR_YMIN + (recty*PR_FACY) );
 	glDrawPixels(PR_RECTX, 3, GL_RGBA, GL_UNSIGNED_BYTE,  rect);
 
 	glPixelZoom(1.0, 1.0);
+	
+	/* flush opengl for cards with frontbuffer slowness */
+	if(recty==PR_RECTY-1 || (PIL_check_seconds_timer() - lasttime > 0.05)) {
+		lasttime= PIL_check_seconds_timer();
+		glFinish();
+	}
 }
 
 static void draw_tex_crop(Tex *tex)
@@ -404,7 +409,7 @@ static void lamp_preview_pixel(LampRen *la, int x, int y, char *rect)
 		else {
 			t= inpr-t;
 			if(t<la->spotbl && la->spotbl!=0.0) {
-				/* zachte gebied */
+				/* soft area */
 				i= t/la->spotbl;
 				t= i*i;
 				i= t*i;
@@ -540,7 +545,7 @@ static void texture_preview_pixel(Tex *tex, int x, int y, char *rect)
 		texvec[0]= 0.5+v1*x;
 		texvec[1]= 0.5+v1*y;
 		
-		/* geen coordmapping, uitzondering: repeat */
+		/* no coordinate mapping, exception: repeat */
 		if(tex->xrepeat>1) {
 			texvec[0] *= tex->xrepeat;
 			if(texvec[0]>1.0) texvec[0] -= (int)(texvec[0]);
@@ -585,7 +590,7 @@ static void texture_preview_pixel(Tex *tex, int x, int y, char *rect)
 		texvec[2]= 0.0;
 	}
 	
-	/* geeft geen Tin terug */
+	/* does not return Tin */
 	if(tex->type==TEX_STUCCI) {
 		tex->nor= R.vn;
 		R.vn[0]= 1.0;
@@ -659,7 +664,7 @@ static void shade_preview_pixel(float *vec,
 	
 	R.refcol[0]= R.refcol[1]= R.refcol[2]= R.refcol[3]= 0.0;
 
-	/* texture afhandeling */
+	/* texture handling */
 	if(mat->texco) {
 		
 		VECCOPY(R.lo, vec);
@@ -745,22 +750,32 @@ static void shade_preview_pixel(float *vec,
 			
 			if(mat->spec)  {
 				
-				lv[0]+= view[0];
-				lv[1]+= view[1];
-				lv[2]+= view[2];
-				Normalise(lv);
-				
 				if(inp>0.0) {
-					v1= lv[0]*R.vn[0]+lv[1]*R.vn[1]+lv[2]*R.vn[2];
-					if(v1>0.0) {
-						v1= RE_Spec(v1, mat->har);
-						inprspec= v1*mat->spec;
-						isr+= inprspec*mat->specr;
-						isg+= inprspec*mat->specg;
-						isb+= inprspec*mat->specb;
-					}
+					/* specular shaders */
+					float specfac;
+					
+					if(mat->spec_shader==MA_SPEC_PHONG) 
+						specfac= Phong_Spec(R.vn, lv, view, mat->har);
+					else if(mat->spec_shader==MA_SPEC_COOKTORR) 
+						specfac= CookTorr_Spec(R.vn, lv, view, mat->har);
+					else if(mat->spec_shader==MA_SPEC_BLINN) 
+						specfac= Blinn_Spec(R.vn, lv, view, mat->refrac, (float)mat->har);
+					else 
+						specfac= Toon_Spec(R.vn, lv, view, mat->param[2], mat->param[3]);
+				
+					inprspec= specfac*mat->spec;
+					
+					isr+= inprspec*mat->specr;
+					isg+= inprspec*mat->specg;
+					isb+= inprspec*mat->specb;
+
 				}
 			}
+			/* diffuse shaders */
+			if(mat->diff_shader==MA_DIFF_ORENNAYAR) inp= OrenNayar_Diff(R.vn, lv, view, mat->roughness);
+			else if(mat->diff_shader==MA_DIFF_TOON) inp= Toon_Diff(R.vn, lv, view, mat->param[0], mat->param[1]);
+			// else Lambert
+
 			inp= (mat->ref*inp + mat->emit);
 			
 			if(a==0) la= pr1_col;
@@ -838,7 +853,7 @@ void BIF_previewrender(SpaceButs *sbuts)
 	if ELEM4(sbuts->mainb, BUTS_MAT, BUTS_TEX, BUTS_LAMP, BUTS_WORLD);
 	else return;
 	
-	har.flarec= 0;	/* verderop test op postrender flare */
+	har.flarec= 0;	/* below is a test for postrender flare */
 	
 	if(qtest()) {
 		addafterqueue(curarea->win, RENDERPREVIEW, 1);
@@ -895,12 +910,12 @@ void BIF_previewrender(SpaceButs *sbuts)
 		if(ob==0 || ob->type!=OB_LAMP) return;
 		la= ob->data;
 		init_render_world();
-		init_render_textures();	/* ze mogen niet twee keer!! (brightness) */
+		init_render_textures();	/* do not do it twice!! (brightness) */
 		R.totlamp= 0;
 		RE_add_render_lamp(ob, 0);	/* 0=no shadbuf */
 		lar= R.la[0];
 		
-		/* uitzonderingen: */
+		/* exceptions: */
 		lar->spottexfac= 1.0;
 		lar->spotsi= cos( M_PI/3.0 );
 		lar->spotbl= (1.0-lar->spotsi)*la->spotblend;
