@@ -1,5 +1,5 @@
 /**
- * $Id: renderwin.c,v 1.4 2003/02/08 13:51:55 sgefant Exp $
+ * $Id: renderwin.c,v 1.22 2003/07/19 20:19:31 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -29,6 +29,13 @@
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
+ 
+#ifdef WIN32
+/* for the multimedia timer */
+#include <windows.h>
+#include <mmsystem.h>
+#endif
+
 #include <string.h>
 #include <stdarg.h>
 #include <math.h>
@@ -39,6 +46,11 @@
 
 #ifdef WIN32
 #include "BLI_winstuff.h"
+#else
+ /* for signal callback, not (fully) supported at windows */
+#include <sys/time.h>
+#include <signal.h>
+
 #endif
 
 #include "BLI_blenlib.h"
@@ -70,14 +82,32 @@
 #include "BSE_filesel.h"
 #include "BSE_headerbuttons.h"
 
-#include "blendertimer.h"
-
 #include "blendef.h"
 #include "mydevice.h"
 #include "winlay.h"
 #include "render.h"
 
-/***/
+/* ------------ renderwin struct, to prevent too much global vars --------- */
+/* ------------ only used for display in a 2nd window  --------- */
+
+
+/* flags escape presses during event handling
+* so we can test for user break later.
+*/
+#define RW_FLAGS_ESCAPE		(1<<0)
+/* old zoom style (2x, locked to mouse, exits
+* when mouse leaves window), to be removed
+* at some point.
+*/
+#define RW_FLAGS_OLDZOOM		(1<<1)
+/* on when image is being panned with middlemouse
+*/
+#define RW_FLAGS_PANNING		(1<<2)
+/* on when the mouse is dragging over the image
+* to examine pixel values.
+*/
+#define RW_FLAGS_PIXEL_EXAMINING	(1<<3)
+
 
 typedef struct {
 	Window *win;
@@ -88,22 +118,6 @@ typedef struct {
 	int mbut[3];
 	int lmouse[2];
 	
-	/* flags escape presses during event handling 
-	 * so we can test for user break later.
-	 */
-#define RW_FLAGS_ESCAPE		(1<<0)
-	/* old zoom style (2x, locked to mouse, exits
-	 * when mouse leaves window), to be removed
-	 * at some point.
-	 */
-#define RW_FLAGS_OLDZOOM		(1<<1)
-	/* on when image is being panned with middlemouse
-	 */
-#define RW_FLAGS_PANNING		(1<<2)
-	/* on when the mouse is dragging over the image
-	 * to examine pixel values.
-	 */
-#define RW_FLAGS_PIXEL_EXAMINING	(1<<3)
 	unsigned int flags;
 	
 	float pan_mouse_start[2], pan_ofs_start[2];
@@ -111,7 +125,13 @@ typedef struct {
 	char *info_text;
 } RenderWin;
 
-static RenderWin *renderwin_new(Window *win)
+static RenderWin *render_win= NULL;
+
+/* --------------- help functions for RenderWin struct ---------------------------- */
+
+
+/* only called in function open_renderwin */
+static RenderWin *renderwin_alloc(Window *win)
 {
 	RenderWin *rw= MEM_mallocN(sizeof(*rw), "RenderWin");
 	rw->win= win;
@@ -120,30 +140,17 @@ static RenderWin *renderwin_new(Window *win)
 	rw->flags= 0;
 	rw->zoomofs[0]= rw->zoomofs[1]= 0;
 	rw->info_text= NULL;
-	
+
 	rw->lmouse[0]= rw->lmouse[1]= 0;
 	rw->mbut[0]= rw->mbut[1]= rw->mbut[2]= 0;
 
 	return rw;
 }
-static void renderwin_destroy(RenderWin *rw)
-{
-	if (rw->info_text) MEM_freeN(rw->info_text);
-	window_destroy(rw->win);
-	MEM_freeN(rw);
-}
 
-/***/
-
-static void close_renderwin(void);
-
-static RenderWin *render_win= NULL;
-
-/**/
 
 static void renderwin_queue_redraw(RenderWin *rw)
 {
-	window_queue_redraw(rw->win);
+	window_queue_redraw(rw->win); // to ghost
 }
 
 static void renderwin_reshape(RenderWin *rw)
@@ -180,8 +187,8 @@ static int renderwin_win_to_image_co(RenderWin *rw, int winco[2], int imgco_r[2]
 	
 	renderwin_get_disprect(rw, disprect);
 	
-	imgco_r[0]= (winco[0]-disprect[0][0])/rw->zoom;
-	imgco_r[1]= (winco[1]-disprect[0][1])/rw->zoom;
+	imgco_r[0]= (int) ((winco[0]-disprect[0][0])/rw->zoom);
+	imgco_r[1]= (int) ((winco[1]-disprect[0][1])/rw->zoom);
 	
 	return (imgco_r[0]>=0 && imgco_r[1]>=0 && imgco_r[0]<R.rectx && imgco_r[1]<R.recty);
 }
@@ -196,8 +203,8 @@ static int renderwin_win_to_ndc(RenderWin *rw, int win_co[2], float ndc_r[2])
 
 	window_get_size(rw->win, &w, &h);
 
-	ndc_r[0]= (float) (win_co[0]*2)/(w-1) - 1.0;
-	ndc_r[1]= (float) (win_co[1]*2)/(h-1) - 1.0;
+	ndc_r[0]=  ((float)(win_co[0]*2)/(w-1) - 1.0);
+	ndc_r[1]=  ((float)(win_co[1]*2)/(h-1) - 1.0);
 
 	return (fabs(ndc_r[0])<=1.0 && fabs(ndc_r[1])<=1.0);
 }
@@ -208,6 +215,28 @@ static void renderwin_set_infotext(RenderWin *rw, char *info_text)
 	rw->info_text= info_text?BLI_strdup(info_text):NULL;
 }
 
+static void renderwin_reset_view(RenderWin *rw)
+{
+	int w, h, rectx, recty;
+
+	if (rw->info_text) renderwin_set_infotext(rw, NULL);
+
+	/* now calculate a zoom for when image is larger than window */
+	window_get_size(rw->win, &w, &h);
+		/* at this point the r.rectx/y values are not correct yet */
+	rectx= (G.scene->r.size*G.scene->r.xsch)/100;
+	recty= (G.scene->r.size*G.scene->r.ysch)/100;
+
+	if(rectx>w || recty>h) {
+		if(rectx-w > recty-h) rw->zoom= ((float)w)/((float)rectx);
+		else rw->zoom= ((float)h)/((float)recty);
+	}
+	else rw->zoom= 1.0;
+
+	rw->zoomofs[0]= rw->zoomofs[1]= 0;
+	renderwin_queue_redraw(rw);
+}
+
 static void renderwin_draw(RenderWin *rw, int just_clear)
 {
 	float disprect[2][2];
@@ -243,12 +272,283 @@ static void renderwin_draw(RenderWin *rw, int just_clear)
 	window_swap_buffers(rw->win);
 }
 
-	/* XXX, this is not good, we do this without any regard to state
-	 * ... better is to make this an optimization of a more clear
-	 * implementation. the bug shows up when you do something like
-	 * open the window, then draw part of the progress, then get
-	 * a redraw event. whatever can go wrong will.
-	 */
+/* ------ interactivity calls for RenderWin ------------- */
+
+static void renderwin_mouse_moved(RenderWin *rw)
+{
+	if (rw->flags & RW_FLAGS_PIXEL_EXAMINING) {
+		int imgco[2];
+		char buf[64];
+
+		if (R.rectot && renderwin_win_to_image_co(rw, rw->lmouse, imgco)) {
+			unsigned char *pxl= (char*) &R.rectot[R.rectx*imgco[1] + imgco[0]];
+
+			sprintf(buf, "R: %d, G: %d, B: %d, A: %d", pxl[0], pxl[1], pxl[2], pxl[3]);
+			renderwin_set_infotext(rw, buf);
+			renderwin_queue_redraw(rw);
+		} else {
+			renderwin_set_infotext(rw, NULL);
+			renderwin_queue_redraw(rw);
+		}
+	} 
+	else if (rw->flags & RW_FLAGS_PANNING) {
+		int delta_x= rw->lmouse[0] - rw->pan_mouse_start[0];
+		int delta_y= rw->lmouse[1] - rw->pan_mouse_start[1];
+	
+		rw->zoomofs[0]= rw->pan_ofs_start[0] - delta_x/rw->zoom;
+		rw->zoomofs[1]= rw->pan_ofs_start[1] - delta_y/rw->zoom;
+		rw->zoomofs[0]= CLAMPIS(rw->zoomofs[0], -R.rectx/2, R.rectx/2);
+		rw->zoomofs[1]= CLAMPIS(rw->zoomofs[1], -R.recty/2, R.recty/2);
+
+		renderwin_queue_redraw(rw);
+	} 
+	else if (rw->flags & RW_FLAGS_OLDZOOM) {
+		float ndc[2];
+		int w, h;
+
+		window_get_size(rw->win, &w, &h);
+		renderwin_win_to_ndc(rw, rw->lmouse, ndc);
+
+		rw->zoomofs[0]= -0.5*ndc[0]*(w-R.rectx*rw->zoom)/rw->zoom;
+		rw->zoomofs[1]= -0.5*ndc[1]*(h-R.recty*rw->zoom)/rw->zoom;
+
+		renderwin_queue_redraw(rw);
+	}
+}
+
+static void renderwin_mousebut_changed(RenderWin *rw)
+{
+	if (rw->mbut[0]) {
+		rw->flags|= RW_FLAGS_PIXEL_EXAMINING;
+	} 
+	else if (rw->mbut[1]) {
+		rw->flags|= RW_FLAGS_PANNING;
+		rw->pan_mouse_start[0]= rw->lmouse[0];
+		rw->pan_mouse_start[1]= rw->lmouse[1];
+		rw->pan_ofs_start[0]= rw->zoomofs[0];
+		rw->pan_ofs_start[1]= rw->zoomofs[1];
+	} 
+	else {
+		if (rw->flags & RW_FLAGS_PANNING) {
+			rw->flags &= ~RW_FLAGS_PANNING;
+			renderwin_queue_redraw(rw);
+		}
+		if (rw->flags & RW_FLAGS_PIXEL_EXAMINING) {
+			rw->flags&= ~RW_FLAGS_PIXEL_EXAMINING;
+			renderwin_set_infotext(rw, NULL);
+			renderwin_queue_redraw(rw);
+		}
+	}
+}
+
+
+/* handler for renderwin, passed on to Ghost */
+static void renderwin_handler(Window *win, void *user_data, short evt, short val, char ascii)
+{
+	RenderWin *rw= user_data;
+
+	if (evt==RESHAPE) {
+		renderwin_reshape(rw);
+	} 
+	else if (evt==REDRAW) {
+		renderwin_draw(rw, 0);
+#ifndef __APPLE__
+	} 
+	else if (evt==WINCLOSE) {
+		BIF_close_render_display();
+#endif
+	} 
+	else if (evt==INPUTCHANGE) {
+		rw->active= val;
+
+		if (!val && (rw->flags&RW_FLAGS_OLDZOOM)) {
+			rw->flags&= ~RW_FLAGS_OLDZOOM;
+			renderwin_reset_view(rw);
+		}
+	} 
+	else if (ELEM(evt, MOUSEX, MOUSEY)) {
+		rw->lmouse[evt==MOUSEY]= val;
+		renderwin_mouse_moved(rw);
+	} 
+	else if (ELEM3(evt, LEFTMOUSE, MIDDLEMOUSE, RIGHTMOUSE)) {
+		int which= (evt==LEFTMOUSE)?0:(evt==MIDDLEMOUSE)?1:2;
+		rw->mbut[which]= val;
+		renderwin_mousebut_changed(rw);
+	} 
+	else if (val) {
+		if (evt==ESCKEY) {
+			if (rw->flags&RW_FLAGS_OLDZOOM) {
+				rw->flags&= ~RW_FLAGS_OLDZOOM;
+				renderwin_reset_view(rw);
+			} 
+			else {
+				rw->flags|= RW_FLAGS_ESCAPE;
+				mainwindow_raise();
+				mainwindow_make_active();
+				rw->active= 0;
+			}
+		} 
+		else if (evt==JKEY) {
+			BIF_swap_render_rects();
+		} 
+		else if (evt==ZKEY) {
+			if (rw->flags&RW_FLAGS_OLDZOOM) {
+				rw->flags&= ~RW_FLAGS_OLDZOOM;
+				renderwin_reset_view(rw);
+			} else {
+				rw->zoom= 2.0;
+				rw->flags|= RW_FLAGS_OLDZOOM;
+				renderwin_mouse_moved(rw);
+			}
+		} 
+		else if (evt==PADPLUSKEY) {
+			if (rw->zoom<15.9) {
+				rw->zoom*= 2.0;
+				renderwin_queue_redraw(rw);
+			}
+		} 
+		else if (evt==PADMINUS) {
+			if (rw->zoom>0.26) {
+				rw->zoom*= 0.5;
+				renderwin_queue_redraw(rw);
+			}
+		} 
+		else if (evt==PADENTER || evt==HOMEKEY) {
+			if (rw->flags&RW_FLAGS_OLDZOOM) {
+				rw->flags&= ~RW_FLAGS_OLDZOOM;
+			}
+			renderwin_reset_view(rw);
+		} 
+		else if (evt==F3KEY) {
+			mainwindow_raise();
+			mainwindow_make_active();
+			rw->active= 0;
+			areawinset(find_biggest_area()->win);
+			BIF_save_rendered_image();
+		} 
+		else if (evt==F11KEY) {
+			BIF_toggle_render_display();
+		} 
+		else if (evt==F12KEY) {
+			BIF_do_render(0);
+		}
+	}
+}
+
+
+/* opens window and allocs struct */
+static void open_renderwin(int winpos[2], int winsize[2])
+{
+	Window *win;
+
+	win= window_open("Blender:Render", winpos[0], winpos[1], winsize[0], winsize[1], 0);
+
+	render_win= renderwin_alloc(win);
+
+	/* Ghost calls handler */
+	window_set_handler(win, renderwin_handler, render_win);
+
+	winlay_process_events(0);
+	window_make_active(render_win->win);
+	winlay_process_events(0);
+
+	renderwin_draw(render_win, 1);
+	renderwin_draw(render_win, 1);
+}
+
+/* -------------- callbacks for render loop: Window (RenderWin) ----------------------- */
+
+/* calculations for window size and position */
+void calc_renderwin_rectangle(int posmask, int renderpos_r[2], int rendersize_r[2]) 
+{
+	int scr_w, scr_h, x, y, div= 0;
+	float ndc_x= 0.0, ndc_y= 0.0;
+
+		/* XXX, we temporarily hack the screen size and position so
+		 * the window is always 60 pixels away from a side, really need
+		 * a GHOST_GetMaxWindowBounds or so - zr
+		 */
+	winlay_get_screensize(&scr_w, &scr_h);
+	
+	rendersize_r[0]= (G.scene->r.size*G.scene->r.xsch)/100;
+	rendersize_r[1]= (G.scene->r.size*G.scene->r.ysch)/100;
+	if(G.scene->r.mode & R_PANORAMA) {
+		rendersize_r[0]*= G.scene->r.xparts;
+		rendersize_r[1]*= G.scene->r.yparts;
+	}
+	/* increased size of clipping for OSX, should become an option instead */
+	rendersize_r[0]= CLAMPIS(rendersize_r[0], 100, scr_w-120);
+	rendersize_r[1]= CLAMPIS(rendersize_r[1], 100, scr_h-120);
+
+	for (y=-1; y<=1; y++) {
+		for (x=-1; x<=1; x++) {
+			if (posmask & (1<<((y+1)*3 + (x+1)))) {
+				ndc_x+= x;
+				ndc_y+= y;
+				div++;
+			}
+		}
+	}
+
+	if (div) {
+		ndc_x/= div;
+		ndc_y/= div;
+	}
+		
+	renderpos_r[0]= 60 + (scr_w-90-rendersize_r[0])*(ndc_x*0.5 + 0.5);
+	renderpos_r[1]= 30 + (scr_h-90-rendersize_r[1])*(ndc_y*0.5 + 0.5);
+}
+	
+/* init renderwin, alloc/open/resize */
+static void renderwin_init_display_cb(void) 
+{
+	if (G.afbreek == 0) {
+		int rendersize[2], renderpos[2];
+
+		calc_renderwin_rectangle(R.winpos, renderpos, rendersize);
+
+		if (!render_win) {
+			open_renderwin(renderpos, rendersize);
+			renderwin_reset_view(render_win); // incl. autozoom for large images
+		} else {
+			int win_x, win_y;
+			int win_w, win_h;
+
+			window_get_position(render_win->win, &win_x, &win_y);
+			window_get_size(render_win->win, &win_w, &win_h);
+
+				/* XXX, this is nasty and I guess bound to cause problems,
+				 * but to ensure the window is at the user specified position
+				 * and size we reopen the window all the time... we need
+				 * a ghost _set_position to fix this -zr
+				 */
+			BIF_close_render_display();
+			open_renderwin(renderpos, rendersize);
+
+			renderwin_reset_view(render_win);
+			render_win->flags&= ~RW_FLAGS_ESCAPE;
+			render_win->active= 1;
+		}
+	}
+}
+
+/* callback for redraw render win */
+static void renderwin_clear_display_cb(short ignore) 
+{
+	if (render_win) {
+		window_make_active(render_win->win);	
+		renderwin_draw(render_win, 1);
+	}
+}
+
+/* XXX, this is not good, we do this without any regard to state
+* ... better is to make this an optimization of a more clear
+* implementation. the bug shows up when you do something like
+* open the window, then draw part of the progress, then get
+* a redraw event. whatever can go wrong will.
+*/
+
+/* in render window; display a couple of scanlines of rendered image (see callback below) */
 static void renderwin_progress(RenderWin *rw, int start_y, int nlines, int rect_w, int rect_h, unsigned char *rect)
 {
 	float disprect[2][2];
@@ -271,270 +571,22 @@ static void renderwin_progress(RenderWin *rw, int start_y, int nlines, int rect_
 	glDrawBuffer(GL_BACK);
 }
 
-static void renderwin_mouse_moved(RenderWin *rw)
-{
-	if (rw->flags&RW_FLAGS_PIXEL_EXAMINING) {
-		int imgco[2];
-		char buf[64];
 
-		if (R.rectot && renderwin_win_to_image_co(rw, rw->lmouse, imgco)) {
-			unsigned char *pxl= (char*) &R.rectot[R.rectx*imgco[1] + imgco[0]];
-
-			sprintf(buf, "R: %d, G: %d, B: %d, A: %d", pxl[0], pxl[1], pxl[2], pxl[3]);
-			renderwin_set_infotext(rw, buf);
-			renderwin_queue_redraw(rw);
-		} else {
-			renderwin_set_infotext(rw, NULL);
-			renderwin_queue_redraw(rw);
-		}
-	} else if (rw->flags&RW_FLAGS_PANNING) {
-		int delta_x= rw->lmouse[0] - rw->pan_mouse_start[0];
-		int delta_y= rw->lmouse[1] - rw->pan_mouse_start[1];
-	
-		rw->zoomofs[0]= rw->pan_ofs_start[0] - delta_x/rw->zoom;
-		rw->zoomofs[1]= rw->pan_ofs_start[1] - delta_y/rw->zoom;
-		rw->zoomofs[0]= CLAMPIS(rw->zoomofs[0], -R.rectx/2, R.rectx/2);
-		rw->zoomofs[1]= CLAMPIS(rw->zoomofs[1], -R.recty/2, R.recty/2);
-
-		renderwin_queue_redraw(rw);
-	} else if (rw->flags&RW_FLAGS_OLDZOOM) {
-		float ndc[2];
-		int w, h;
-
-		window_get_size(rw->win, &w, &h);
-		renderwin_win_to_ndc(rw, rw->lmouse, ndc);
-
-		rw->zoomofs[0]= -0.5*ndc[0]*(w-R.rectx*rw->zoom)/rw->zoom;
-		rw->zoomofs[1]= -0.5*ndc[1]*(h-R.recty*rw->zoom)/rw->zoom;
-
-		renderwin_queue_redraw(rw);
-	}
-}
-
-static void renderwin_mousebut_changed(RenderWin *rw)
-{
-	if (rw->mbut[0]) {
-		rw->flags|= RW_FLAGS_PIXEL_EXAMINING;
-	} else if (rw->mbut[1]) {
-		rw->flags|= RW_FLAGS_PANNING;
-		rw->pan_mouse_start[0]= rw->lmouse[0];
-		rw->pan_mouse_start[1]= rw->lmouse[1];
-		rw->pan_ofs_start[0]= rw->zoomofs[0];
-		rw->pan_ofs_start[1]= rw->zoomofs[1];
-	} else {
-		if (rw->flags&RW_FLAGS_PANNING) {
-			rw->flags&= ~RW_FLAGS_PANNING;
-			renderwin_queue_redraw(rw);
-		}
-		if (rw->flags&RW_FLAGS_PIXEL_EXAMINING) {
-			rw->flags&= ~RW_FLAGS_PIXEL_EXAMINING;
-			renderwin_set_infotext(rw, NULL);
-			renderwin_queue_redraw(rw);
-		}
-	}
-}
-
-static void renderwin_reset_view(RenderWin *rw)
-{
-	if (rw->info_text) renderwin_set_infotext(rw, NULL);
-
-	rw->zoom= 1.0;
-	rw->zoomofs[0]= rw->zoomofs[1]= 0;
-	renderwin_queue_redraw(rw);
-}
-
-static void renderwin_handler(Window *win, void *user_data, short evt, short val, char ascii)
-{
-	RenderWin *rw= user_data;
-	
-	if (evt==RESHAPE) {
-		renderwin_reshape(rw);
-	} else if (evt==REDRAW) {
-		renderwin_draw(rw, 0);
-#ifndef __APPLE__
-	} else if (evt==WINCLOSE) {
-		close_renderwin();
-#endif
-	} else if (evt==INPUTCHANGE) {
-		rw->active= val;
-		
-		if (!val && (rw->flags&RW_FLAGS_OLDZOOM)) {
-			rw->flags&= ~RW_FLAGS_OLDZOOM;
-			renderwin_reset_view(rw);
-		}
-	} else if (ELEM(evt, MOUSEX, MOUSEY)) {
-		rw->lmouse[evt==MOUSEY]= val;
-		renderwin_mouse_moved(rw);
-	} else if (ELEM3(evt, LEFTMOUSE, MIDDLEMOUSE, RIGHTMOUSE)) {
-		int which= (evt==LEFTMOUSE)?0:(evt==MIDDLEMOUSE)?1:2;
-		rw->mbut[which]= val;
-		renderwin_mousebut_changed(rw);
-	} else if (val) {
-		if (evt==ESCKEY) {
-			if (rw->flags&RW_FLAGS_OLDZOOM) {
-				rw->flags&= ~RW_FLAGS_OLDZOOM;
-				renderwin_reset_view(rw);
-			} else {
-				rw->flags|= RW_FLAGS_ESCAPE;
-				mainwindow_raise();
-			}
-		} else if (evt==JKEY) {
-			BIF_swap_render_rects();
-		} else if (evt==ZKEY) {
-			if (rw->flags&RW_FLAGS_OLDZOOM) {
-				rw->flags&= ~RW_FLAGS_OLDZOOM;
-				renderwin_reset_view(rw);
-			} else {
-				rw->zoom= 2.0;
-				rw->flags|= RW_FLAGS_OLDZOOM;
-				renderwin_mouse_moved(rw);
-			}
-		} else if (evt==PADPLUSKEY) {
-			if (rw->zoom<15.9) {
-				rw->zoom*= 2.0;
-				renderwin_queue_redraw(rw);
-			}
-		} else if (evt==PADMINUS) {
-			if (rw->zoom>0.26) {
-				rw->zoom*= 0.5;
-				renderwin_queue_redraw(rw);
-			}
-		} else if (evt==PADENTER || evt==HOMEKEY) {
-			if (rw->flags&RW_FLAGS_OLDZOOM) {
-				rw->flags&= ~RW_FLAGS_OLDZOOM;
-			}
-			renderwin_reset_view(rw);
-		} else if (evt==F3KEY) {
-			mainwindow_raise();
-			mainwindow_make_active();
-			areawinset(find_biggest_area()->win);
-			BIF_save_rendered_image();
-		} else if (evt==F11KEY) {
-			BIF_toggle_render_display();
-		} else if (evt==F12KEY) {
-			BIF_do_render(0);
-		}
-	}
-}
-
-/**/
-
-	/* Render window render callbacks */
-
-void calc_renderwin_rectangle(int posmask, int renderpos_r[2], int rendersize_r[2]) 
-{
-	int scr_w, scr_h, x, y, div= 0;
-	float ndc_x= 0.0, ndc_y= 0.0;
-
-		/* XXX, we temporarily hack the screen size and position so
-		 * the window is always 30 pixels away from a side, really need
-		 * a GHOST_GetMaxWindowBounds or so - zr
-		 */
-	winlay_get_screensize(&scr_w, &scr_h);
-	
-	rendersize_r[0]= (G.scene->r.size*G.scene->r.xsch)/100;
-	rendersize_r[1]= (G.scene->r.size*G.scene->r.ysch)/100;
-	if(G.scene->r.mode & R_PANORAMA) {
-		rendersize_r[0]*= G.scene->r.xparts;
-		rendersize_r[1]*= G.scene->r.yparts;
-	}
-
-	rendersize_r[0]= CLAMPIS(rendersize_r[0], 100, scr_w-90);
-	rendersize_r[1]= CLAMPIS(rendersize_r[1], 100, scr_h-90);
-
-	for (y=-1; y<=1; y++) {
-		for (x=-1; x<=1; x++) {
-			if (posmask & (1<<((y+1)*3 + (x+1)))) {
-				ndc_x+= x;
-				ndc_y+= y;
-				div++;
-			}
-		}
-	}
-
-	if (div) {
-		ndc_x/= div;
-		ndc_y/= div;
-	}
-		
-	renderpos_r[0]= 60 + (scr_w-90-rendersize_r[0])*(ndc_x*0.5 + 0.5);
-	renderpos_r[1]= 60 + (scr_h-90-rendersize_r[1])*(ndc_y*0.5 + 0.5);
-}
-	
-static void open_renderwin(int winpos[2], int winsize[2])
-{
-	Window *win;
-
-	win= window_open("Blender:Render", winpos[0], winpos[1], winsize[0], winsize[1], 0);
-			
-	render_win= renderwin_new(win);
-			
-	window_set_handler(win, renderwin_handler, render_win);
-
-	winlay_process_events(0);
-	window_make_active(render_win->win);
-	winlay_process_events(0);
-			
-	renderwin_draw(render_win, 1);
-	renderwin_draw(render_win, 1);
-}
-
-static void close_renderwin(void)
-{
-	if (render_win) {
-		renderwin_destroy(render_win);
-		render_win= NULL;
-	}
-}
-
-static void renderwin_init_display_cb(void) 
-{
-	if (G.afbreek == 0) {
-		int rendersize[2], renderpos[2];
-
-		calc_renderwin_rectangle(R.winpos, renderpos, rendersize);
-
-		if (!render_win) {
-			open_renderwin(renderpos, rendersize);
-		} else {
-			int win_x, win_y;
-			int win_w, win_h;
-
-			window_get_position(render_win->win, &win_x, &win_y);
-			window_get_size(render_win->win, &win_w, &win_h);
-
-				/* XXX, this is nasty and I guess bound to cause problems,
-				 * but to ensure the window is at the user specified position
-				 * and size we reopen the window all the time... we need
-				 * a ghost _set_position to fix this -zr
-				 */
-			close_renderwin();
-			open_renderwin(renderpos, rendersize);
-
-			renderwin_reset_view(render_win);
-			render_win->flags&= ~RW_FLAGS_ESCAPE;
-		}
-	}
-}
-static void renderwin_clear_display_cb(short ignore) 
-{
-	if (render_win) {
-		window_make_active(render_win->win);	
-		renderwin_draw(render_win, 1);
-	}
-}
-
-static void renderwin_progress_display_cb(int y1, int y2, int w, int h, unsigned int *rect) 
+/* in render window; display a couple of scanlines of rendered image */
+static void renderwin_progress_display_cb(int y1, int y2, int w, int h, unsigned int *rect)
 {
 	if (render_win) {
 		renderwin_progress(render_win, y1, y2-y1+1, w, h, (unsigned char*) rect);
 	}
 }
 
-	/* Render view render callbacks */
+
+/* -------------- callbacks for render loop: in View3D ----------------------- */
+
 
 static View3D *render_view3d = NULL;
 
+/* init Render view callback */
 static void renderview_init_display_cb(void)
 {
 	ScrArea *sa;
@@ -558,6 +610,8 @@ static void renderview_init_display_cb(void)
 	}
 }
 
+
+/* in 3d view; display a couple of scanlines of rendered image */
 static void renderview_progress_display_cb(int y1, int y2, int w, int h, unsigned int *rect)
 {
 	if (render_view3d) {
@@ -586,31 +640,16 @@ static void renderview_progress_display_cb(int y1, int y2, int w, int h, unsigne
 
 		glFlush();
 		glDrawBuffer(GL_BACK);
-		
 		v3d->flag |= V3D_DISPIMAGE;
-					
 		v3d->area->win_swap= WIN_FRONT_OK;
+		
 	}
 }
 
-	/* Shared render callbacks */
+/* -------------- callbacks for render loop: interactivity ----------------------- */
 
-static int test_break(void)
-{
-	if (!G.afbreek) {
-		if (MISC_test_break()) {
-			;
-		} else if (render_win) {
-			winlay_process_events(0);
-			// render_win can be closed in winlay_process_events()
-			if (render_win == 0 || (render_win->flags & RW_FLAGS_ESCAPE))
-				G.afbreek= 1;
-		}
-	}
-	
-	return G.afbreek;
-}
 
+/* callback for print info in top header in interface */
 static void printrenderinfo_cb(double time, int sample)
 {
 	extern int mem_in_use;
@@ -630,47 +669,147 @@ static void printrenderinfo_cb(double time, int sample)
 	
 	screen_draw_info_text(G.curscreen, str);
 }
-	
-/***/
 
-void BIF_renderwin_set_custom_cursor(unsigned char mask[16][2], unsigned char bitmap[16][2])
+/* -------------- callback system to allow ESC from rendering ----------------------- */
+
+/* POSIX & WIN32: this function is called all the time, and should not use cpu or resources */
+static int test_break()
 {
-	if (render_win) {
-		window_set_custom_cursor(render_win->win, mask, bitmap);
-	}
-}
 
-static void do_crap(int force_dispwin)
-{
-	if (R.displaymode == R_DISPLAYWIN || force_dispwin) {
-		RE_set_initrenderdisplay_callback(NULL);
-		RE_set_clearrenderdisplay_callback(renderwin_clear_display_cb);	
-		RE_set_renderdisplay_callback(renderwin_progress_display_cb);
+	if(G.afbreek==2) { /* code for testing queue */
 
-		renderwin_init_display_cb();
-	} else {
-		BIF_close_render_display();
-		
-		RE_set_initrenderdisplay_callback(renderview_init_display_cb);
-		RE_set_clearrenderdisplay_callback(NULL);	
-		RE_set_renderdisplay_callback(renderview_progress_display_cb);
+		G.afbreek= 0;
+
+		blender_test_break(); /* tests blender interface */
+
+		if (G.afbreek==0 && render_win) { /* tests window */
+			winlay_process_events(0);
+			// render_win can be closed in winlay_process_events()
+			if (render_win == 0 || (render_win->flags & RW_FLAGS_ESCAPE))
+				G.afbreek= 1;
+		}
 	}
 
-	RE_set_test_break_callback(test_break);	
-	RE_set_timecursor_callback(set_timecursor);
-	RE_set_printrenderinfo_callback(printrenderinfo_cb);
+	if(G.afbreek==1) return 1;
+	else return 0;
 }
 
+
+
+#ifdef _WIN32
+/* we use the multimedia time here */
+static UINT uRenderTimerId;
+
+void CALLBACK interruptESC(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
+{
+	if(G.afbreek==0) G.afbreek= 2;	/* code for read queue */
+}
+
+/* WIN32: init SetTimer callback */
+static void init_test_break_callback()
+{
+	timeBeginPeriod(50);
+	uRenderTimerId = timeSetEvent(250, 1, interruptESC, 0, TIME_PERIODIC);
+}
+
+/* WIN32: stop SetTimer callback */
+static void end_test_break_callback()
+{
+	timeEndPeriod(50);
+	timeKillEvent(uRenderTimerId);
+}
+
+#else
+/* all other OS's support signal(SIGVTALRM) */
+
+/* POSIX: this function goes in the signal() callback */
+static void interruptESC(int sig)
+{
+
+	if(G.afbreek==0) G.afbreek= 2;	/* code for read queue */
+
+	/* call again, timer was reset */
+	signal(SIGVTALRM, interruptESC);
+}
+
+/* POSIX: initialize timer and signal */
+static void init_test_break_callback()
+{
+
+	struct itimerval tmevalue;
+
+	tmevalue.it_interval.tv_sec = 0;
+	tmevalue.it_interval.tv_usec = 250000;
+	/* wanneer de eerste ? */
+	tmevalue.it_value.tv_sec = 0;
+	tmevalue.it_value.tv_usec = 10000;
+
+	signal(SIGVTALRM, interruptESC);
+	setitimer(ITIMER_VIRTUAL, &tmevalue, 0);
+
+}
+
+/* POSIX: stop timer and callback */
+static void end_test_break_callback()
+{
+	struct itimerval tmevalue;
+
+	tmevalue.it_value.tv_sec = 0;
+	tmevalue.it_value.tv_usec = 0;
+	setitimer(ITIMER_VIRTUAL, &tmevalue, 0);
+	signal(SIGVTALRM, SIG_IGN);
+
+}
+
+
+#endif
+
+
+
+/* -------------- callbacks for render loop: init & run! ----------------------- */
+
+
+/* - initialize displays
+   - both opengl render as blender render
+   - set callbacks
+   - cleanup
+*/
+
 static void do_render(View3D *ogl_render_view3d, int anim, int force_dispwin)
 {
-	do_crap(force_dispwin);
+	if (R.displaymode == R_DISPLAYWIN || force_dispwin) {
+		RE_set_initrenderdisplay_callback(NULL);
+		RE_set_clearrenderdisplay_callback(renderwin_clear_display_cb);
+		RE_set_renderdisplay_callback(renderwin_progress_display_cb);
+
+		renderwin_init_display_cb();
+	} 
+	else {
+#ifndef __APPLE__		
+		BIF_close_render_display();
+#endif
+		RE_set_initrenderdisplay_callback(renderview_init_display_cb);
+		RE_set_clearrenderdisplay_callback(NULL);
+		RE_set_renderdisplay_callback(renderview_progress_display_cb);
+	}
+
+	init_test_break_callback();
+	RE_set_test_break_callback(test_break);
 	
-	if (render_win) window_set_cursor(render_win->win, CURSOR_WAIT);
+	RE_set_timecursor_callback(set_timecursor);
+	RE_set_printrenderinfo_callback(printrenderinfo_cb);
+	
+	if (render_win) {
+		window_set_cursor(render_win->win, CURSOR_WAIT);
+		// when opening new window... not cross platform identical behaviour, so
+		// for now call it each time
+		if(ogl_render_view3d) init_gl_stuff();
+	}
 	waitcursor(1);
 
 	G.afbreek= 0;
 	if(G.obedit && !(G.scene->r.scemode & R_OGL)) {
-		exit_editmode(0);	/* 0 = geen freedata */
+		exit_editmode(0);	/* 0 = no free data */
 	}
 
 	if(anim) {
@@ -679,7 +818,8 @@ static void do_render(View3D *ogl_render_view3d, int anim, int force_dispwin)
 	else {
 		RE_initrender(ogl_render_view3d);
 	}
-	update_for_newframe();
+
+	if(anim) update_for_newframe_muted();  // only when anim, causes redraw event which frustrates dispview
 	R.flag= 0;
 	
 	if (render_win) window_set_cursor(render_win->win, CURSOR_STD);
@@ -688,22 +828,13 @@ static void do_render(View3D *ogl_render_view3d, int anim, int force_dispwin)
 	free_filesel_spec(G.scene->r.pic);
 
 	G.afbreek= 0;
+	end_test_break_callback();
 
-	mainwindow_make_active();
-}
-
-void BIF_do_render(int anim)
-{
-	do_render(NULL, anim, 0);
-}
-
-void BIF_do_ogl_render(View3D *ogl_render_view3d, int anim)
-{
-	G.scene->r.scemode |= R_OGL;
-	do_render(ogl_render_view3d, anim, 1);
-	G.scene->r.scemode &= ~R_OGL;
+	/* Leave this window active, otherwise in dispiew it will destroy the image as well
+	   window_make_active() raises window at osx and sends redraws */
 }
 
+/* finds area with a 'dispview' set */
 static ScrArea *find_dispimage_v3d(void)
 {
 	ScrArea *sa;
@@ -719,20 +850,7 @@ static ScrArea *find_dispimage_v3d(void)
 	return NULL;
 }
 
-static void redraw_render_display(void)
-{
-	if (R.displaymode == R_DISPLAYWIN) {
-			// don't open render_win if rendering has been
-			// canceled or the render_win has been actively closed
-		if (render_win) {
-			renderwin_queue_redraw(render_win);
-		}
-	} else {
-		renderview_init_display_cb();
-		renderview_progress_display_cb(0, R.recty-1, R.rectx, R.recty, R.rectot);
-	}
-}
-
+/* used for swapping with spare buffer, when images are different size */
 static void scalefastrect(unsigned int *recto, unsigned int *rectn, int oldx, int oldy, int newx, int newy)
 {
 	unsigned int *rect, *newrect;
@@ -756,6 +874,22 @@ static void scalefastrect(unsigned int *recto, unsigned int *rectn, int oldx, in
 	}
 }
 
+/* -------------- API: externally called --------------- */
+
+/* set up display, render an image or scene */
+void BIF_do_render(int anim)
+{
+	do_render(NULL, anim, 0);
+}
+
+/* set up display, render the current area view in an image */
+void BIF_do_ogl_render(View3D *ogl_render_view3d, int anim)
+{
+	G.scene->r.scemode |= R_OGL;
+	do_render(ogl_render_view3d, anim, 1);
+	G.scene->r.scemode &= ~R_OGL;
+}
+
 void BIF_swap_render_rects(void)
 {
 	unsigned int *temp;
@@ -780,23 +914,58 @@ void BIF_swap_render_rects(void)
 	R.rectot= R.rectspare;
 	R.rectspare= temp;
 
-	redraw_render_display();
+	/* redraw */
+	if (R.displaymode == R_DISPLAYWIN) {
+		// don't open render_win if rendering has been
+	    // canceled or the render_win has been actively closed
+		if (render_win) {
+			renderwin_queue_redraw(render_win);
+		}
+	} else {
+		renderview_init_display_cb();
+		renderview_progress_display_cb(0, R.recty-1, R.rectx, R.recty, R.rectot);
+	}
 }				
 
-void BIF_toggle_render_display(void)
+/* called from usiblender.c too, to free and close renderwin */
+void BIF_close_render_display(void)
+{
+	if (render_win) {
+
+		if (render_win->info_text) MEM_freeN(render_win->info_text);
+
+		window_destroy(render_win->win); /* ghost close window */
+		MEM_freeN(render_win);
+
+		render_win= NULL;
+	}
+}
+
+
+/* typical with F11 key, show image or hide/close */
+void BIF_toggle_render_display(void) 
 {
 	ScrArea *sa= find_dispimage_v3d();
 	
-	if (render_win && render_win->active) {
-		if (R.displaymode == R_DISPLAYVIEW) {
-			BIF_close_render_display();
+	if(R.rectot==NULL);		// do nothing
+	else if (render_win && R.displaymode==R_DISPLAYWIN) {
+		if(render_win->active) {
+			mainwindow_raise();
+			mainwindow_make_active();
+			render_win->active= 0;
 		}
-		mainwindow_raise();
-	} else if (sa) {
+		else {
+			window_raise(render_win->win);
+			window_make_active(render_win->win);
+			render_win->active= 1;
+		}
+	} 
+	else if (sa && R.displaymode==R_DISPLAYVIEW) {
 		View3D *vd= sa->spacedata.first;
 		vd->flag &= ~V3D_DISPIMAGE;
 		scrarea_queue_winredraw(sa);
-	} else {
+	} 
+	else {
 		if (R.displaymode == R_DISPLAYWIN) {
 			renderwin_init_display_cb();
 		} else {
@@ -809,7 +978,9 @@ void BIF_toggle_render_display(void)
 	}
 }
 
-void BIF_close_render_display(void)
+void BIF_renderwin_set_custom_cursor(unsigned char mask[16][2], unsigned char bitmap[16][2])
 {
-	close_renderwin();
+	if (render_win) {
+		window_set_custom_cursor(render_win->win, mask, bitmap);
+	}
 }
