@@ -1,5 +1,5 @@
 /**
- * $Id: toolbox.c,v 1.6 2002/12/30 20:15:07 mein Exp $
+ * $Id: toolbox.c,v 1.16 2003/07/20 23:04:09 hos Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -52,6 +52,7 @@
 #include "MEM_guardedalloc.h"
 
 #include "BMF_Api.h"
+#include "BIF_language.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
@@ -60,6 +61,7 @@
 #include "DNA_image_types.h"
 #include "DNA_object_types.h"
 #include "DNA_screen_types.h"
+#include "DNA_userdef_types.h"
 #include "DNA_view3d_types.h"
 
 #include "BKE_plugin_types.h"
@@ -79,6 +81,8 @@
 #include "BIF_editseq.h"
 #include "BIF_editlattice.h"
 #include "BIF_editsima.h"
+#include "BIF_editoops.h"
+#include "BIF_imasel.h"
 #include "BIF_screen.h"
 #include "BIF_tbcallback.h"
 #include "BIF_editnla.h"
@@ -89,6 +93,7 @@
 
 #include "BSE_editipo.h"
 #include "BSE_buttons.h"
+#include "BSE_filesel.h"
 
 #include "IMB_imbuf.h"
 
@@ -98,23 +103,12 @@
 #include "interface.h"
 #include "render.h"
 
-/*   ********  NOTES  ***********	*****************************
-	
-	- Toolbox items zelf invullen
-	- de colormap kleuren staan met comments in de bgntoolbox()
-	- de funktie extern_qread eventueel vervangen
-	- let op de benaming van bijzondere toetsen (NumL etc) 
-	- meelinken: Button.c,  ivm rawcodes
-	
-	*****************************	*****************************
-*/	
-
 
 static int tbx1, tbx2, tby1, tby2, tbfontyofs, tbmain=0;
 static int tbmemx=TBOXX/2, tbmemy=(TBOXEL-0.5)*TBOXH, tboldwin, addmode= 0;
 static int oldcursor;
 
-	/* variabelen per item */
+	/* variables per item */
 static char *tbstr, *tbstr1, *keystr; 	
 static void (*tbfunc)(int);
 static int tbval;
@@ -156,7 +150,7 @@ void tboxSetCallback(Tbox_callbackfunc f)
 
 void tbox_setinfo(int x, int y)
 {
-	/* afhankelijk van tbmain worden vars gezet */
+	/* dependant of tbmain vars are set */
 	tbstr= 0;
 	tbstr1= 0;
 	tbfunc= 0;
@@ -494,7 +488,7 @@ void bgnpupdraw(int startx, int starty, int endx, int endy)
 
 	mywinset(G.curscreen->mainwin);
 	
-	/* pietsje groter, 1 pixel aan de rand */
+	/* tinsy bit larger, 1 pixel on the rand */
 	
 	glReadBuffer(GL_FRONT);
 	glDrawBuffer(GL_FRONT);
@@ -506,7 +500,7 @@ void bgnpupdraw(int startx, int starty, int endx, int endy)
 	oldcursor= get_cursor();
 	set_cursor(CURSOR_STD);
 	
-	tbfontyofs= (TBOXH-11)/2 +2;	/* toolbox, hier stond ooit getheigh */
+	tbfontyofs= (TBOXH-11)/2 +1;	/* ypos text in toolbox */
 }
 
 void endpupdraw(void)
@@ -566,11 +560,18 @@ void asciitoraw(int ch, unsigned short *event, unsigned short *qual)
 
 void tbox_execute(void)
 {
-	/* als tbfunc: functie aanroepen */
-	/* als tbstr1 is een string: value tbval in queue stopen */
+	/* if tbfunc: call function */
+	/* if tbstr1 is a string: put value tbval in queue */
 	unsigned short event=0;
 	unsigned short qual1=0, qual2=0;
 
+	/* needed to check for valid selected objects */
+	Base *base=NULL;
+	Object *ob=NULL;
+
+	base= BASACT;
+	if (base) ob= base->object;
+
 	if(tbfunc) {
 		tbfunc(tbval);
 	}
@@ -594,30 +595,50 @@ void tbox_execute(void)
 		}
 		/* ctrl-s (Shear): switch into editmode ### */
 		else if(strcmp(tbstr1, "c|s")==0) {
-			if (!G.obedit) {
-				enter_editmode();
-				/* ### put these into a deselectall_gen() */
-				if(G.obedit->type==OB_MESH) deselectall_mesh();
-				else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) deselectall_nurb();
-				else if(G.obedit->type==OB_MBALL) deselectall_mball();
-				else if(G.obedit->type==OB_LATTICE) deselectall_Latt();
-				/* ### */
+			/* check that a valid object is selected to prevent crash */
+			if(!ob) error("Only selected objects can be sheared");
+			else if ((ob->type==OB_LAMP) || (ob->type==OB_EMPTY) || (ob->type==OB_FONT) || (ob->type==OB_CAMERA)) {
+				error("Only editable 3D objects can be sheared");
+			}
+			else if ((base->lay & G.vd->lay)==0) {
+				error("Only objects on visible layers can be sheared");
+			}
+			else {
+				if (!G.obedit) {
+					enter_editmode();
+					/* ### put these into a deselectall_gen() */
+					if(G.obedit->type==OB_MESH) deselectall_mesh();
+					else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) deselectall_nurb();
+					else if(G.obedit->type==OB_MBALL) deselectall_mball();
+					else if(G.obedit->type==OB_LATTICE) deselectall_Latt();
+					/* ### */
+				}
+				qual1 = LEFTCTRLKEY;
+				event = SKEY;
 			}
-			qual1 = LEFTCTRLKEY;
-			event = SKEY;
 		}
 		else if(strcmp(tbstr1, "W")==0) {
-			if (!G.obedit) {
-				enter_editmode();
-				/* ### put these into a deselectall_gen() */
-				if(G.obedit->type==OB_MESH) deselectall_mesh();
-				else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) deselectall_nurb();
-				else if(G.obedit->type==OB_MBALL) deselectall_mball();
-				else if(G.obedit->type==OB_LATTICE) deselectall_Latt();
-				/* ### */
+			if (!ob) error ("Only selected objects can be warped");
+		        /* check that a valid object is selected to prevent crash */
+			else if ((ob->type==OB_LAMP) || (ob->type==OB_EMPTY) || (ob->type==OB_FONT) || (ob->type==OB_CAMERA)) {
+				error("Only editable 3D objects can be warped");
+			}
+			else if ((base->lay & G.vd->lay)==0) {
+				error("Only objects on visible layers can be warped");
+			}
+			else {
+				if (!G.obedit) {
+					enter_editmode();
+					/* ### put these into a deselectall_gen() */
+					if(G.obedit->type==OB_MESH) deselectall_mesh();
+					else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) deselectall_nurb();
+					else if(G.obedit->type==OB_MBALL) deselectall_mball();
+					else if(G.obedit->type==OB_LATTICE) deselectall_Latt();
+					/* ### */
+				}
+				qual1 = LEFTSHIFTKEY;
+				event = WKEY;
 			}
-			qual1 = LEFTSHIFTKEY;
-			event = WKEY;
 		}
 
 		else if(strlen(tbstr1)<4 || (strlen(tbstr1)==4 && tbstr1[2]=='F')) {
@@ -808,11 +829,10 @@ void tbox_drawelem_text(x, y, type)
 		glColor3ub(240, 240, 240);
 	}
 	
-	/* tekst */
+	/* text */
 	tbox_setinfo(x, y);
 	if(tbstr && tbstr[0]) {
 		len1= 5+BMF_GetStringWidth(G.font, tbstr);
-//		if(tbstr1) len2= 5+BMF_GetStringWidth(G.font, tbstr1); else len2= 0;
 		if(keystr) len2= 5+BMF_GetStringWidth(G.font, keystr); else len2= 0;
 		
 		while(len1>0 && (len1+len2+5>x2-x1) ) {
@@ -821,9 +841,8 @@ void tbox_drawelem_text(x, y, type)
 		}
 		
 		glRasterPos2i(x1+5, y1+tbfontyofs);
-		BMF_DrawString(G.font, tbstr);
+		BIF_DrawString(G.font, tbstr, (U.transopts & TR_MENUS), (type==0 || type==2)?0:1);
 		
-//		if(tbstr1 && tbstr1[0]) {
 		if(keystr && keystr[0]) {
 			if(type & 1) {
 				ColorFunc(TBOXBLACK);
@@ -831,14 +850,12 @@ void tbox_drawelem_text(x, y, type)
 				glRecti(x2-len2-2,  y1+2,  x2-3,  y2-2);
 				ColorFunc(TBOXWHITE);
 				glRasterPos2i(x2-len2,  y1+tbfontyofs);
-//				BMF_DrawString(G.font, tbstr1);
-				BMF_DrawString(G.font, keystr);
+				BIF_DrawString(G.font, keystr, (U.transopts & TR_MENUS), 1);
 			}
 			else {
 				ColorFunc(TBOXBLACK);
 				glRasterPos2i(x2-len2,  y1+tbfontyofs);
-//				BMF_DrawString(G.font, tbstr1);
-				BMF_DrawString(G.font, keystr);
+				BIF_DrawString(G.font, keystr, (U.transopts & TR_MENUS), 0);
 			}
 		}
 	}
@@ -880,7 +897,7 @@ void drawtoolbox(void)
 
 	tbox_getactive(&actx, &acty);
 
-	/* de background */
+	/* background */
 	for(x=0; x<2; x++) {
 		
 		for(y=0; y<TBOXEL; y++) {
@@ -898,7 +915,7 @@ void drawtoolbox(void)
 		}
 	}
 
-	/* de text */
+	/* text */
 	for(x=0; x<2; x++) {
 		
 		for(y=0; y<TBOXEL; y++) {
@@ -932,7 +949,7 @@ void toolbox(void)
 	drawtoolbox();
 	
 	/* 
-	 *	De aktieve window wordt in queue terug gestopt.
+	 *	The active window will be put back in the queue.
 	 */
 
 	while(1) {
@@ -962,7 +979,7 @@ void toolbox(void)
 					}
 					break;
 				case ESCKEY:
-					/* altkeys: om conflicten met overdraw en stow/push/pop te voorkomen */
+					/* alt keys: to prevent conflicts with over-draw and stow/push/pop at sgis */
 #ifndef MAART
 /* Temporary for making screen dumps (Alt+PrtSc) */
 				case LEFTALTKEY:
@@ -978,7 +995,7 @@ void toolbox(void)
 		
 		tbox_getactive(&actx, &acty);
 		
-		/* muisafhandeling en redraw */
+		/* mouse handling and redraw */
 		if(xo!=actx || yo!=acty) {
 			if(actx==0) {
 				if (acty==0) addmode=0;
@@ -1008,6 +1025,8 @@ void toolbox(void)
 
 /* ************************************  */
 
+/* this va_ stuff allows printf() style codes in these menus */
+
 static int vconfirm(char *title, char *itemfmt, va_list ap)
 {
 	char *s, buf[512];
@@ -1074,7 +1093,7 @@ int saveover(char *file)
 	return (!BLI_exists(file) || confirm("SAVE OVER", file));
 }
 
-/* ****************** EXTRAATJE **************** */
+/* ****************** EXTRA STUFF **************** */
 
 short button(short *var, short min, short max, char *str)
 {
@@ -1093,7 +1112,8 @@ short button(short *var, short min, short max, char *str)
 	if(mval[1]>G.curscreen->sizey) mval[1]= G.curscreen->sizey-10;
 
 	block= uiNewBlock(&listb, "button", UI_EMBOSSX, UI_HELV, G.curscreen->mainwin);
-	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1);
+	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1|
+				   UI_BLOCK_ENTER_OK);
 
 	x1=mval[0]-150; 
 	y1=mval[1]-20; 
@@ -1251,9 +1271,9 @@ void draw_numbuts_tip(char *str, int x1, int y1, int x2, int y2)
 	cpack(0x0);
 
 	temp= 0;
-	while( BMF_GetStringWidth(G.fonts, str+temp)>(x2 - x1-24)) temp++;
+	while( BIF_GetStringWidth(G.fonts, str+temp, (U.transopts & TR_BUTTONS))>(x2 - x1-24)) temp++;
 	glRasterPos2i(x1+16, y2-30);
-	BMF_DrawString(G.fonts, str+temp);
+	BIF_DrawString(G.fonts, str+temp, (U.transopts & TR_BUTTONS), 0);
 }
 
 int do_clever_numbuts(char *name, int tot, int winevent)
@@ -1272,7 +1292,7 @@ int do_clever_numbuts(char *name, int tot, int winevent)
 	sizex= 235;
 	sizey= 30+20*(tot+1);
 	
-	/* midden */
+	/* center */
 	if(mval[0]<sizex/2) mval[0]=sizex/2;
 	if(mval[1]<sizey/2) mval[1]=sizey/2;
 	if(mval[0]>G.curscreen->sizex -sizex/2) mval[0]= G.curscreen->sizex -sizex/2;
@@ -1286,8 +1306,8 @@ int do_clever_numbuts(char *name, int tot, int winevent)
 	block= uiNewBlock(&listb, "numbuts", UI_EMBOSSX, UI_HELV, G.curscreen->mainwin);
 	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1|UI_BLOCK_ENTER_OK);
 	
-	/* LET OP: TEX BUTTON UITZONDERING */
-	/* WAARSCHUWING: ALLEEN EEN ENKELE BITJES-BUTTON MOGELIJK: ER WORDT OP KOPIEDATA GEWERKT! */
+	/* WATCH IT: TEX BUTTON EXCEPTION */
+	/* WARNING: ONLY A SINGLE BIT-BUTTON POSSIBLE: WE WORK AT COPIED DATA! */
 
 	uiDefBut(block, LABEL, 0, name,	(short)(x1+15), (short)(y2-35), (short)(sizex-60), 19, 0, 1.0, 0.0, 0, 0, ""); 
 
@@ -1359,7 +1379,7 @@ void add_numbut(int nr, int type, char *str, float min, float max, void *poin, c
 		strcpy(numbuts[nr].tip, "");
 	
 	
-	/* LET OP: TEX BUTTON UITZONDERING */
+	/*WATCH: TEX BUTTON EXCEPTION */
 	
 	numbpoin[nr]= poin;
 	
@@ -1423,11 +1443,21 @@ void clever_numbuts(void)
 	else if(curarea->spacetype==SPACE_IMAGE) {
 		clever_numbuts_sima();
 	}
+	else if(curarea->spacetype==SPACE_IMASEL) {
+		clever_numbuts_imasel();
+	}
 	else if(curarea->spacetype==SPACE_BUTS){
 		clever_numbuts_buts();
 	}
-	
-	
+	else if(curarea->spacetype==SPACE_OOPS) {
+		clever_numbuts_oops();
+	}
+	else if(curarea->spacetype==SPACE_ACTION){
+		stupid_damn_numbuts_action();
+	}
+	else if(curarea->spacetype==SPACE_FILE) {
+		clever_numbuts_filesel();
+	}
 }
 
 
