@@ -1,5 +1,5 @@
 /**
- * $Id: view.c,v 1.3 2002/11/25 12:02:06 mein Exp $
+ * $Id: view.c,v 1.6 2003/07/11 20:02:52 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,6 +31,7 @@
  */
 
 #include <math.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -87,7 +88,6 @@ void persp3d(View3D *v3d, int a)
 {
 	ScrArea *area= v3d->area;
 	
-		/* oppasen met optimaliseren: dan laatste mode in area bewaren */
 		/* only 3D windows */
 	if(a== 0) {
 		bwin_get_winmatrix(area->win, area->winmat);
@@ -126,7 +126,6 @@ void persp_general(int a)
 
 void persp(int a)
 {
-	/* oppasen met optimaliseren: dan laatste mode in area bewaren */
 	/* only 3D windows */
 
 	if(curarea->spacetype!=SPACE_VIEW3D) persp_general(a);
@@ -156,7 +155,7 @@ void initgrabz(float x, float y, float z)
 
 void window_to_3d(float *vec, short mx, short my)
 {
-	/* altijd initzgrab aanroepen */
+	/* always call initzgrab */
 	float dx, dy;
 	float fmx, fmy, winx, winy;
 	
@@ -176,7 +175,7 @@ void window_to_3d(float *vec, short mx, short my)
 	vec[2]= (G.vd->persinv[0][2]*dx + G.vd->persinv[1][2]*dy);
 }
 
-void project_short(float *vec, short *adr)	/* clipt */
+void project_short(float *vec, short *adr)	/* clips */
 {
 	float fx, fy, vec4[4];
 
@@ -228,7 +227,7 @@ void project_short_noclip(float *vec, short *adr)
 
 int boundbox_clip(float obmat[][4], BoundBox *bb)
 {
-	/* return 1: afbeelden */
+	/* return 1: draw */
 	
 	float mat[4][4];
 	float vec[4], min, max;
@@ -325,7 +324,7 @@ void sdrawbox(short x1, short y1, short x2, short y2)
 	glEnd();
 }
 
-/* trackball: deze is t.o.v. een 100% bol formule wel zo mooi */
+/* trackball: better one than a full spherical solution */
 
 void calctrackballvecfirst(rcti *area, short *mval, float *vec)
 {
@@ -333,7 +332,7 @@ void calctrackballvecfirst(rcti *area, short *mval, float *vec)
 	
 	radius= TRACKBALLSIZE;
 	
-	/* x en y normaliseren */
+	/* normalise x and y */
 	x= (area->xmax + area->xmin)/2 -mval[0];
 	x/= (float)((area->xmax - area->xmin)/2);
 	y= (area->ymax + area->ymin)/2 -mval[1];
@@ -350,7 +349,7 @@ void calctrackballvecfirst(rcti *area, short *mval, float *vec)
 
 	vec[0]= x;
 	vec[1]= y;
-	vec[2]= -z;		/* jawel! */
+	vec[2]= -z;		/* yah yah! */
 
 	if( fabs(vec[2])>fabs(vec[1]) && fabs(vec[2])>fabs(vec[0]) ) {
 		vec[0]= 0.0;
@@ -392,16 +391,16 @@ void calctrackballvec(rcti *area, short *mval, float *vec)
 
 	vec[0]= x;
 	vec[1]= y;
-	vec[2]= -z;		/* jawel! */
+	vec[2]= -z;		/* yah yah! */
 
 }
 
 void viewmove(int mode)
 {
 	float firstvec[3], newvec[3], dvec[3];
-	float oldquat[4], q1[4], q2[4], si, phi;
+	float oldquat[4], q1[4], si, phi;
 	int firsttime=1;
-	short mval[2], mvalo[2];
+	short mvalball[2], mval[2], mvalo[2];
 	
 	/* sometimes this routine is called from headerbuttons */
 	areawinset(curarea->win);
@@ -410,7 +409,11 @@ void viewmove(int mode)
 	initgrabz(-G.vd->ofs[0], -G.vd->ofs[1], -G.vd->ofs[2]);
 	
 	QUATCOPY(oldquat, G.vd->viewquat);
-	getmouseco_sc(mvalo);		/* werk met screencoordinaten ivm trackball functie */
+	
+	getmouseco_sc(mvalo);		/* work with screen coordinates because of trackball function */
+	mvalball[0]= mvalo[0];			/* needed for turntable to work */
+	mvalball[1]= mvalo[1];
+	
 	calctrackballvec(&curarea->winrct, mvalo, firstvec);
 
 	/* cumultime(0); */
@@ -422,9 +425,9 @@ void viewmove(int mode)
 			
 			if(firsttime) {
 				firsttime= 0;
-				/* wordt hier geroteerd, gezoomd of transleerd */
+				/* are we translating, rotating or zooming? */
 				if(mode==0) {
-					if(G.vd->view!=0) scrarea_queue_headredraw(curarea);	/* voor button */
+					if(G.vd->view!=0) scrarea_queue_headredraw(curarea);	/*for button */
 					G.vd->view= 0;
 				}
 						
@@ -436,55 +439,49 @@ void viewmove(int mode)
 			}
 
 
-			if(mode==0) {	/* viewroteer */
-				
-				if(U.flag & TRACKBALL) {
-					calctrackballvec(&curarea->winrct, mval, newvec);
-					
-					VecSubf(dvec, newvec, firstvec);
-					
-					si= sqrt(dvec[0]*dvec[0]+ dvec[1]*dvec[1]+ dvec[2]*dvec[2]);
-					si/= (2.0*TRACKBALLSIZE);
-					
-					if(si<1.0) {
-						Crossf(q1+1, firstvec, newvec);
-	
-						Normalise(q1+1);
+			if(mode==0) {	/* view rotate */
 			
-						phi= asin(si);
+				/* if turntable method, we don't change mvalball[0] */
+			
+				if(U.flag & TRACKBALL) mvalball[0]= mval[0];
+				mvalball[1]= mval[1];
+				
+				calctrackballvec(&curarea->winrct, mvalball, newvec);
+				
+				VecSubf(dvec, newvec, firstvec);
+				
+				si= sqrt(dvec[0]*dvec[0]+ dvec[1]*dvec[1]+ dvec[2]*dvec[2]);
+				si/= (2.0*TRACKBALLSIZE);
+				
+				/* is there an acceptable solution? (180 degrees limitor) */
+				if(si<1.0) {
+					Crossf(q1+1, firstvec, newvec);
+
+					Normalise(q1+1);
 		
-						si= sin(phi);
-						q1[0]= cos(phi);
-						q1[1]*= si;
-						q1[2]*= si;
-						q1[3]*= si;
-						
-						QuatMul(G.vd->viewquat, q1, oldquat);
-					}
-				}
-				else {
-					/* roteren om z-as (x beweging) en liggende as (y) */
-					
-					phi= 2*(mval[0]-mvalo[0]);
-					phi/= (float)curarea->winx;
+					phi= asin(si);
+	
 					si= sin(phi);
 					q1[0]= cos(phi);
-					q1[1]= q1[2]= 0.0;
-					q1[3]= si;
+					q1[1]*= si;
+					q1[2]*= si;
+					q1[3]*= si;
 					
-					/* liggende as */
-					VECCOPY(q2+1, G.vd->viewinv[0]);
-					Normalise(q2+1);
-					phi= (mvalo[1]-mval[1]);
-					phi/= (float)curarea->winy;
-					si= sin(phi);
-					q2[0]= cos(phi);
-					q2[1]*= si;
-					q2[2]*= si;
-					q2[3]*= si;
+					QuatMul(G.vd->viewquat, q1, oldquat);
+
+					if( (U.flag & TRACKBALL)==0 ) {
 					
-					QuatMul(q1, q1, q2);
-					QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
+						/* rotate around z-axis (mouse x moves)  */
+						
+						phi= 2*(mval[0]-mvalball[0]);
+						phi/= (float)curarea->winx;
+						si= sin(phi);
+						q1[0]= cos(phi);
+						q1[1]= q1[2]= 0.0;
+						q1[3]= si;
+						
+						QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
+					}
 				}
 			}
 			else if(mode==1) {	/* translate */
@@ -507,11 +504,11 @@ void viewmove(int mode)
 			else if(mode==2) {
 				G.vd->dist*= 1.0+(float)(mvalo[0]-mval[0]+mvalo[1]-mval[1])/1000.0;
 				
-				/* deze limits ook in toets.c */
+				/* these limits are in toets.c too */
 				if(G.vd->dist<0.001*G.vd->grid) G.vd->dist= 0.001*G.vd->grid;
 				if(G.vd->dist>10.0*G.vd->far) G.vd->dist=10.0*G.vd->far;
 				
-				mval[1]= mvalo[1]; /* blijft ie zoomen */
+				mval[1]= mvalo[1]; /* keeps zooming that way */
 				mval[0]= mvalo[0];
 			}
 			
@@ -526,7 +523,7 @@ void viewmove(int mode)
 		}
 		else BIF_wait_for_statechange();
 
-		/* dit moet onderaan, anders pakt de get_mbut het niet op de PC... */
+		/* this in the end, otherwise get_mbut does not work on a PC... */
 		if( !(get_mbut() & (L_MOUSE|M_MOUSE))) break;
 	}
 
@@ -535,7 +532,7 @@ void viewmove(int mode)
 
 short v3d_windowmode=0;
 
-void setwinmatrixview3d(rctf *rect)		/* rect: voor picking */
+void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 {
 	Camera *cam=0;
 	float d, near, far, winx = 0.0, winy = 0.0;
@@ -613,7 +610,7 @@ void setwinmatrixview3d(rctf *rect)		/* rect: voor picking */
 		
 		if(G.vd->persp==2 && (G.special1 & G_HOLO)) {
 			if(cam && (cam->flag & CAM_HOLO2)) {
-				tfac= fac/4.0;	/* de fac is 1280/640 gecorr voor obszoom */
+				tfac= fac/4.0;	/* the fac is 1280/640 corrected for obszoom */
 
 				if(cam->netend==0.0) cam->netend= EFRA;
 				fac= (G.scene->r.cfra-1.0)/(cam->netend)-0.5;
@@ -676,7 +673,7 @@ void obmat_to_viewmat(Object *ob)
 	Mat4Ortho(bmat);
 	Mat4Invert(G.vd->viewmat, bmat);
 	
-	/* viewquat berekenen, o.a. voor add object */
+	/* view quat calculation, needed for add object */
 	Mat3CpyMat4(tmat, G.vd->viewmat);
 	Mat3ToQuat(tmat, G.vd->viewquat);
 }
@@ -685,7 +682,6 @@ void obmat_to_viewmat(Object *ob)
 void setviewmatrixview3d()
 {
 	Camera *cam;
-/*  	float bepaalphitheta(); */
 
 	if(G.special1 & G_HOLO) RE_holoview();
 
@@ -743,9 +739,9 @@ short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short
 		glEnable(GL_DEPTH_TEST);
 	}
 
-	glSelectBuffer( MAXPICKBUF, buffer);
+	glSelectBuffer( MAXPICKBUF, (GLuint *)buffer);
 	glRenderMode(GL_SELECT);
-	glInitNames();	/* deze twee fies zijn waarvoor? Anders werkt het niet */
+	glInitNames();	/* these two calls whatfor? It doesnt work otherwise */
 	glPushName(-1);
 	code= 1;
 	
@@ -767,7 +763,7 @@ short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short
 			base= base->next;
 		}
 	}
-	glPopName();	/* zie boven (pushname) */
+	glPopName();	/* see above (pushname) */
 	hits= glRenderMode(GL_RENDER);
 	if(hits<0) error("Too many objects in selectbuf");
 
@@ -797,8 +793,8 @@ unsigned int free_localbit()
 	
 	lay= 0;
 	
-	/* soms kunnen we een localview kwijtrijaken: als een area gesloten wordt */
-	/* alle area's aflopen: welke localviews zijn in gebruik */
+	/* sometimes we loose a localview: when an area is closed */
+	/* check all areas: which localviews are in use? */
 	sc= G.main->screen.first;
 	while(sc) {
 		sa= sc->areabase.first;
@@ -901,7 +897,7 @@ void initlocalview()
 		scrarea_queue_winredraw(curarea);
 	}
 	else {
-		/* flags wissen */
+		/* clear flags */
 		base= FIRSTBASE;
 		while(base) {
 			if( base->lay & locallay ) {
@@ -918,7 +914,7 @@ void initlocalview()
 	}
 }
 
-void centreview()	/* localview zonder local! */
+void centreview()	/* like a localview without local! */
 {
 	Base *base;
 	float size, min[3], max[3], afm[3];
@@ -1011,7 +1007,7 @@ void endlocalview(ScrArea *sa)
 		v3d->localvd= 0;
 		v3d->localview= 0;
 
-		/* als in ander window de layers zijn veranderd */
+		/* for when in other window the layers have changed */
 		if(v3d->scenelock) v3d->lay= G.scene->lay;
 		
 		base= FIRSTBASE;
@@ -1026,7 +1022,7 @@ void endlocalview(ScrArea *sa)
 		}
 
 		countall();
-		allqueue(REDRAWVIEW3D, 0);	/* ivm select */
+		allqueue(REDRAWVIEW3D, 0);	/* because of select */
 		
 	}
 }
