@@ -1,5 +1,5 @@
 /**
- * $Id: vpaint.c,v 1.3 2002/11/25 12:02:06 mein Exp $
+ * $Id: vpaint.c,v 1.6 2003/07/05 14:24:05 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,6 +31,7 @@
  */
 
 #include <math.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -110,8 +111,8 @@ MDeformVert *wpaintundobuf=NULL;
 /* Function prototypes */
 int calc_vp_alpha_dl(DispList *disp, MVert *mvert, int vert, short *mval);
 
-/* in tegenstelling tot cpack teken kleuren, zijn de MCOL kleuren (vpaint kleuren) per byte! 
-   en dus endian ongevoelig. Mcol = ABGR!!! Oppassen met cpack functies */
+/* in contradiction to cpack drawing colors, the MCOL colors (vpaint colors) are per byte! 
+   so not endian sensitive. Mcol = ABGR!!! so be cautious with cpack calls */
 
 unsigned int rgba_to_mcol(float r, float g, float b, float a)
 {
@@ -145,8 +146,8 @@ unsigned int vpaint_get_current_col(void)
 
 void do_shared_vertexcol(Mesh *me)
 {
-	/* als geen mcol: niet doen */
-	/* als tface: alleen de betreffende vlakken, anders alles */
+	/* if no mcol: do not do */
+	/* if tface: only the involved faces, otherwise all */
 	MFace *mface;
 	TFace *tface;
 	int a;
@@ -559,7 +560,7 @@ static unsigned int mcol_mul(unsigned int col1, unsigned int col2, int fac)
 	cp2= (char *)&col2;
 	cp=  (char *)&col;
 	
-	/* eerstmullen, dan fac blenden */
+	/* first mul, then blend the fac */
 	cp[0]= 255;
 	cp[1]= (mfac*cp1[1] + fac*((cp2[1]*cp1[1])>>8)  )>>8;
 	cp[2]= (mfac*cp1[2] + fac*((cp2[2]*cp1[2])>>8)  )>>8;
@@ -596,8 +597,12 @@ static int sample_backbuf_area(int x, int y)
 	y2= y+Gvp.size;
 	CLAMP(y1, 0, curarea->winy);
 	CLAMP(y2, 0, curarea->winy);
-	
+#ifdef __APPLE__
+	glReadBuffer(GL_AUX0);
+#endif
 	glReadPixels(x1+curarea->winrct.xmin, y1+curarea->winrct.ymin, x2-x1+1, y2-y1+1, GL_RGBA, GL_UNSIGNED_BYTE,  rect);
+	glReadBuffer(GL_BACK);	
+
 	if(G.order==B_ENDIAN) IMB_convert_rgba_to_abgr( (int)(4*Gvp.size*Gvp.size), rect);
 
 	rt= rect;
@@ -632,8 +637,13 @@ static unsigned int sample_backbuf(int x, int y)
 	
 	x+= curarea->winrct.xmin;
 	y+= curarea->winrct.ymin;
-	
+
+#ifdef __APPLE__
+	glReadBuffer(GL_AUX0);
+#endif
 	glReadPixels(x,  y, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE,  &col);
+	glReadBuffer(GL_BACK);	
+
 	if(G.order==B_ENDIAN) SWITCH_INT(col);
 		
 	return framebuffer_to_index(col);
@@ -789,12 +799,12 @@ void weight_paint(void)
 
 //	if(me->tface==NULL && me->mcol==NULL) return;
 	
-	/* imat voor normalen */
+	/* imat for normals */
 	Mat4MulMat4(mat, ob->obmat, G.vd->viewmat);
 	Mat4Invert(imat, mat);
 	Mat3CpyMat4(vpimat, imat);
 	
-	/* projektiematrix laden */
+	/* load projection matrix */
 	mymultmatrix(ob->obmat);
 	mygetsingmatrix(mat);
 	myloadmatrix(G.vd->viewmat);
@@ -819,7 +829,7 @@ void weight_paint(void)
 
 			firsttime= 0;
 
-			/* welke vlakken doen mee */
+			/* which faces are involved */
 			if(Gvp.flag & VP_AREA) {
 				totindex= sample_backbuf_area(mval[0], mval[1]);
 			}
@@ -828,7 +838,6 @@ void weight_paint(void)
 				if(indexar[0]) totindex= 1;
 				else totindex= 0;
 			}
-			
 			MTC_Mat4SwapMat4(G.vd->persmat, mat);
 			
 			if(Gvp.flag & VP_COLINDEX) {
@@ -914,7 +923,7 @@ void weight_paint(void)
 		me->mcol= 0;
 	}
 	
-	/* cirkel wissen */
+	/* clear circle */
 	draw_sel_circle(0, mvalo, 0, Gvp.size, 1);
 	
 	makeDispList(ob);
@@ -948,12 +957,12 @@ void vertex_paint()
 
 	if(me->tface==NULL && me->mcol==NULL) return;
 	
-	/* imat voor normalen */
+	/* imat for normals */
 	Mat4MulMat4(mat, ob->obmat, G.vd->viewmat);
 	Mat4Invert(imat, mat);
 	Mat3CpyMat4(vpimat, imat);
 	
-	/* projektiematrix laden */
+	/* load projection matrix */
 	mymultmatrix(ob->obmat);
 	mygetsingmatrix(mat);
 	myloadmatrix(G.vd->viewmat);
@@ -979,7 +988,7 @@ void vertex_paint()
 
 			firsttime= 0;
 
-			/* welke vlakken doen mee */
+			/* which faces are involved */
 			if(Gvp.flag & VP_AREA) {
 				totindex= sample_backbuf_area(mval[0], mval[1]);
 			}
@@ -1074,8 +1083,8 @@ void vertex_paint()
 	
 			scrarea_do_windraw(curarea);
 			screen_swapbuffers();
-			backdrawview3d(0);
 			draw_sel_circle(mval, 0, Gvp.size, Gvp.size, 0);
+			backdrawview3d(0);
 	
 			mvalo[0]= mval[0];
 			mvalo[1]= mval[1];
@@ -1087,7 +1096,7 @@ void vertex_paint()
 		me->mcol= 0;
 	}
 	
-	/* cirkel wissen */
+	/* clear circle */
 	draw_sel_circle(0, mvalo, 0, Gvp.size, 1);
 	
 	allqueue(REDRAWVIEW3D, 0);
@@ -1121,7 +1130,7 @@ void set_wpaint(void)		/* toggle */
 		setcursor_space(SPACE_VIEW3D, CURSOR_VPAINT);
 	}
 	else {
-		freefastshade();	/* voor zekerheid */
+		freefastshade();	/* to be sure */
 		if (ob)
 			makeDispList(ob);
 		if(!(G.f & G_FACESELECT))
@@ -1165,7 +1174,7 @@ void set_vpaint(void)		/* toggle */
 		setcursor_space(SPACE_VIEW3D, CURSOR_VPAINT);
 	}
 	else {
-		freefastshade();	/* voor zekerheid */
+		freefastshade();	/* to be sure */
 		if (ob) makeDispList(ob);
 		if((G.f & G_FACESELECT)==0) setcursor_space(SPACE_VIEW3D, CURSOR_STD);
 	}
