@@ -1,7 +1,7 @@
 /**
  * Functions for writing windows avi-format files.
  *
- * $Id: writeavicodec.c,v 1.3 2002/11/25 12:02:06 mein Exp $
+ * $Id: writeavicodec.c,v 1.9 2003/05/24 20:04:34 hos Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,7 +37,7 @@
 #include <config.h>
 #endif
 
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(FREE_WINDOWS)
 
 #define  INC_OLE2
 #include <windows.h>
@@ -50,6 +50,7 @@
 
 #include "MEM_guardedalloc.h"
 #include "BLI_blenlib.h"
+#include "DNA_userdef_types.h"
 
 #include "render_types.h"
 #include "render.h"
@@ -345,7 +346,7 @@ HRESULT STDMETHODCALLTYPE AVIBallReadFormat   (PAVISTREAM ps, LONG lPos,LPVOID l
 	*/
 	
     *lpcbFormat = sizeof(BITMAPINFOHEADER);
-	
+
     return AVIERR_OK;
 }
 
@@ -508,6 +509,9 @@ static void init_bmi(BITMAPINFOHEADER *bmi)
 
 static void opts_to_acd(AviCodecData *acd)
 {
+	HIC hic;
+	ICINFO icinfo;
+
 	acd->fccType = opts.fccType;
 	acd->fccHandler = opts.fccHandler;
 	acd->dwKeyFrameEvery = opts.dwKeyFrameEvery;
@@ -527,6 +531,18 @@ static void opts_to_acd(AviCodecData *acd)
 		acd->lpParms = MEM_mallocN(opts.cbParms, "avi.lpParms");
 		memcpy(acd->lpParms, opts.lpParms, opts.cbParms);
 	}
+
+	if ((hic=ICOpen(ICTYPE_VIDEO,acd->fccHandler,ICMODE_QUERY))!=NULL) {
+		icinfo.dwSize=sizeof(ICINFO);
+		if (ICGetInfo(hic,&icinfo,sizeof(ICINFO))) {
+			WideCharToMultiByte(CP_ACP,0,icinfo.szDescription,-1,acd->avicodecname,128,NULL,NULL);
+		} else
+			sprintf(acd->avicodecname, "undefined");
+		if (ICClose(hic)!=ICERR_OK)
+;//			return 0;
+	} else {
+		sprintf(acd->avicodecname, "Full Frames (Uncompressed)");	//heh, nasty
+	}
 }
 
 
@@ -761,23 +777,30 @@ void append_avi_codec(int frame)
 }
 
 
-int get_codec_settings(void)
+int get_avicodec_settings(void)
 {
-	char name[2048];
 	int ret_val = 0;
 	AVICOMPRESSOPTIONS *aopts[1] = {&opts};
 	AviCodecData *acd = G.scene->r.avicodecdata;
 	static PAVISTREAM psdummy;
+	UINT	uiFlags;
 
 	acd_to_opts(G.scene->r.avicodecdata);
 
 	psdummy = NewBall();
 
+	if(U.uiflag & ALLWINCODECS)
+		uiFlags = ICMF_CHOOSE_ALLCOMPRESSORS;
+	else
+		uiFlags = ICMF_CHOOSE_KEYFRAME | ICMF_CHOOSE_DATARATE;
+
 	if (psdummy == NULL) {
 		ret_val = 1;
 	} else {
 		if (!AVISaveOptions(NULL,
-				ICMF_CHOOSE_KEYFRAME | ICMF_CHOOSE_DATARATE,
+				uiFlags,
+//				ICMF_CHOOSE_KEYFRAME | ICMF_CHOOSE_DATARATE,
+//				ICMF_CHOOSE_ALLCOMPRESSORS,
 				1,
 				&psdummy,
 				(LPAVICOMPRESSOPTIONS *) &aopts))
