@@ -1,5 +1,5 @@
 /**
- * $Id: BlenderPlayerCtl.cpp,v 1.4 2002/12/22 21:46:44 maarten Exp $
+ * $Id: BlenderPlayerCtl.cpp,v 1.5 2003/03/24 17:32:38 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -94,7 +94,6 @@ extern "C"
 static char THIS_FILE[] = __FILE__;
 #endif
 
-
 IMPLEMENT_DYNCREATE(CBlenderPlayerCtrl, COleControl)
 
 CRITICAL_SECTION CBlenderPlayerCtrl::s_cs;
@@ -869,7 +868,7 @@ bool CBlenderPlayerCtrl::activateBlenderData(CBlenderDataPathProperty& blenderDa
 		CWaitCursor waitCursor;
 		blenderData.SeekToBegin();
 		unsigned char* buf = (unsigned char*)::malloc(filelen);
-		blenderData.ReadHuge(buf, filelen);
+		blenderData.Read(buf, filelen);
 
 		// Load the Blender file from memory
 		success = LoadGameData(buf, filelen);
